@@ -3,7 +3,7 @@
     Gerbera - https://gerbera.io/
 
     config_setup.cc - this file is part of Gerbera.
-    Copyright (C) 2020-2023 Gerbera Contributors
+    Copyright (C) 2020-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,13 +22,16 @@
 
 /// \file config_setup.cc
 
+#define GRB_LOG_FAC GrbLogFacility::config
 #include "config_setup.h" // API
 
-#include <numeric>
-
-#include "client_config.h"
 #include "config_definition.h"
 #include "config_options.h"
+#include "config_val.h"
+#include "exceptions.h"
+#include "util/logger.h"
+
+#include <numeric>
 
 pugi::xml_node ConfigSetup::getXmlElement(const pugi::xml_node& root) const
 {
