@@ -3,7 +3,7 @@
     Gerbera - https://gerbera.io/
 
     config_setup_array.cc - this file is part of Gerbera.
-    Copyright (C) 2020-2023 Gerbera Contributors
+    Copyright (C) 2020-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -21,14 +21,18 @@
 */
 
 /// \file config_setup_array.cc
+#define GRB_LOG_FAC GrbLogFacility::config
 
-#include "config/config_setup.h" // API
+#include "config_setup_array.h" // API
 
-#include <numeric>
-
-#include "config/client_config.h"
+#include "common.h"
 #include "config/config_definition.h"
 #include "config/config_options.h"
+#include "config/config_val.h"
+#include "config_setup_int.h"
+#include "util/logger.h"
+
+#include <numeric>
 
 /// \brief Creates an array of strings from an XML nodeset.
 /// \param element starting element of the nodeset.
@@ -48,7 +52,7 @@ bool ConfigArraySetup::createOptionFromNode(const pugi::xml_node& element, std::
     if (element) {
         for (auto&& it : element.select_nodes(ConfigDefinition::mapConfigOption(nodeOption))) {
             const pugi::xml_node& child = it.node();
-            std::string attrValue = attrOption != CFG_MAX ? child.attribute(ConfigDefinition::removeAttribute(attrOption).c_str()).as_string() : child.text().as_string();
+            std::string attrValue = attrOption != ConfigVal::MAX ? child.attribute(ConfigDefinition::removeAttribute(attrOption).c_str()).as_string() : child.text().as_string();
             if (itemCheck) {
                 if (!itemCheck(attrValue))
                     throw_std_runtime_error("Invalid array {} value {} empty '{}'", element.path(), xpath, attrValue);
@@ -70,7 +74,7 @@ void ConfigArraySetup::makeOption(const pugi::xml_node& root, const std::shared_
 
 bool ConfigArraySetup::updateItem(std::size_t i, const std::string& optItem, const std::shared_ptr<Config>& config, const std::shared_ptr<ArrayOption>& value, const std::string& optValue, const std::string& status) const
 {
-    auto index = getItemPath(i);
+    auto index = getItemPath(i, {});
     if (optItem == index || !status.empty()) {
         auto realIndex = value->getIndex(i);
         if (realIndex < std::numeric_limits<std::size_t>::max()) {
@@ -123,12 +127,12 @@ bool ConfigArraySetup::updateDetail(const std::string& optItem, std::string& opt
     return false;
 }
 
-std::string ConfigArraySetup::getItemPath(int index, config_option_t propOption, config_option_t propOption2, config_option_t propOption3, config_option_t propOption4) const
+std::string ConfigArraySetup::getItemPath(int index, const std::vector<ConfigVal>& propOptions) const
 {
     if (index < 0) {
         return fmt::format("{}/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption));
     }
-    return attrOption != CFG_MAX ? fmt::format("{}/{}[{}]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), index, ConfigDefinition::ensureAttribute(attrOption)) : fmt::format("{}/{}[{}]", xpath, ConfigDefinition::mapConfigOption(nodeOption), index);
+    return attrOption != ConfigVal::MAX ? fmt::format("{}/{}[{}]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), index, ConfigDefinition::ensureAttribute(attrOption)) : fmt::format("{}/{}[{}]", xpath, ConfigDefinition::mapConfigOption(nodeOption), index);
 }
 
 std::vector<std::string> ConfigArraySetup::getXmlContent(const pugi::xml_node& optValue)
