@@ -3,7 +3,7 @@
     Gerbera - https://gerbera.io/
 
     config_setup_vector.cc - this file is part of Gerbera.
-    Copyright (C) 2022-2023 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -21,14 +21,16 @@
 */
 
 /// \file config_setup_vector.cc
+#define GRB_LOG_FAC GrbLogFacility::config
 
-#include "config/config_setup.h" // API
+#include "config_setup_vector.h" // API
 
-#include <numeric>
-
-#include "config/client_config.h"
 #include "config/config_definition.h"
 #include "config/config_options.h"
+#include "config/config_val.h"
+#include "util/logger.h"
+
+#include <numeric>
 
 /// \brief Creates a vector from an XML nodeset.
 /// \param optValue starting element of the nodeset.
@@ -47,6 +49,7 @@ bool ConfigVectorSetup::createOptionFromNode(const pugi::xml_node& optValue, std
     if (optValue) {
         const auto dictNodes = optValue.select_nodes(ConfigDefinition::mapConfigOption(nodeOption));
         std::vector<std::string> attrList;
+        attrList.reserve(optionList.size());
         for (auto& opt : optionList) {
             attrList.push_back(ConfigDefinition::removeAttribute(opt));
         }
@@ -67,7 +70,7 @@ bool ConfigVectorSetup::createOptionFromNode(const pugi::xml_node& optValue, std
                 }
             }
             if (itemNotEmpty) {
-                for (auto& attr : attrList) {
+                for (const auto& attr : attrList) {
                     std::string value = child.attribute(attr.c_str()).as_string();
                     if (value.empty()) {
                         return false;
@@ -152,9 +155,9 @@ bool ConfigVectorSetup::updateDetail(const std::string& optItem, std::string& op
     return false;
 }
 
-std::string ConfigVectorSetup::getItemPath(int index, config_option_t propOption, config_option_t propOption2, config_option_t propOption3, config_option_t propOption4) const
+std::string ConfigVectorSetup::getItemPath(int index, const std::vector<ConfigVal>& propOptions) const
 {
-    auto opt = ConfigDefinition::ensureAttribute(propOption);
+    auto opt = ConfigDefinition::ensureAttribute(propOptions.size() > 0 ? propOptions[0] : ConfigVal::MAX);
 
     if (index > ITEM_PATH_ROOT)
         return fmt::format("{}/{}[{}]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), index, opt);
