@@ -4,7 +4,7 @@
 
     autoscan_list.cc - this file is part of Gerbera.
 
-    Copyright (C) 2021-2023 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,19 +22,23 @@
 */
 
 /// \file autoscan_list.cc
-#define LOG_FAC log_facility_t::autoscan
+#define GRB_LOG_FAC GrbLogFacility::autoscan
 
 #include "autoscan_list.h"
 
-#include "autoscan.h"
-#include "content_manager.h"
+#include "config/result/autoscan.h"
+#include "content.h"
 #include "database/database.h"
+#include "exceptions.h"
 #include "util/timer.h"
 
 #ifdef HAVE_INOTIFY
-#include "autoscan_inotify.h"
+#include "content/inotify/autoscan_inotify.h"
+#include "content/inotify/mt_inotify.h"
 #endif
 
+#include <algorithm>
+
 void AutoscanList::updateLMinDB(Database& database)
 {
     AutoLock lock(mutex);
@@ -191,7 +195,7 @@ void AutoscanList::notifyAll(Timer::Subscriber* sub)
 }
 
 void AutoscanList::initTimer(
-    std::shared_ptr<ContentManager>& content,
+    std::shared_ptr<Content>& content,
     std::shared_ptr<Timer>& timer
 #ifdef HAVE_INOTIFY
     ,
@@ -216,4 +220,4 @@ void AutoscanList::initTimer(
             timer->addTimerSubscriber(content.get(), autoScanDir->getInterval(), std::move(param), false);
         }
     }
-}
\ No newline at end of file
+}
