@@ -4,7 +4,7 @@
 
     autoscan_list.h - this file is part of Gerbera.
 
-    Copyright (C) 2021-2023 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,7 +30,7 @@
 
 // forward declaration
 class AutoscanDirectory;
-class ContentManager;
+class Content;
 class Context;
 class Database;
 class Timer;
@@ -81,7 +81,7 @@ public:
     std::vector<std::shared_ptr<AutoscanDirectory>> getArrayCopy() const;
 
     void initTimer(
-        std::shared_ptr<ContentManager>& content,
+        std::shared_ptr<Content>& content,
         std::shared_ptr<Timer>& timer
 #ifdef HAVE_INOTIFY
         ,
