@@ -4,7 +4,7 @@
 
     import_service.cc - this file is part of Gerbera.
 
-    Copyright (C) 2023 Gerbera Contributors
+    Copyright (C) 2023-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,20 +22,22 @@
 */
 
 /// \file import_service.cc
-#define LOG_FAC log_facility_t::content
+#define GRB_LOG_FAC GrbLogFacility::content
 
 #include "import_service.h" // API
 
-#include <fmt/chrono.h>
-#include <regex>
-
-#include "autoscan.h"
+#include "autoscan_setting.h"
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
+#include "config/config_val.h"
+#include "config/result/autoscan.h"
 #include "content_manager.h"
 #include "database/database.h"
+#include "exceptions.h"
 #include "layout/builtin_layout.h"
-#include "upnp_common.h"
+#include "metadata/metadata_enums.h"
+#include "metadata/metadata_handler.h"
+#include "metadata/metadata_service.h"
 #include "util/mime.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
@@ -43,9 +45,14 @@
 #ifdef HAVE_JS
 #include "layout/js_layout.h"
 #include "scripting/import_script.h"
+#include "scripting/metafile_parser_script.h"
 #include "scripting/playlist_parser_script.h"
 #endif
 
+#include <algorithm>
+#include <fmt/chrono.h>
+#include <regex>
+
 bool UpnpMap::checkValue(const std::string& op, const std::string& expect, const std::string& actual) const
 {
     if (op == "=" && actual.find(expect) != std::string::npos)
@@ -79,7 +86,7 @@ bool UpnpMap::isMatch(const std::shared_ptr<CdsItem>& item, const std::string& m
                 match = checkValue(op, stoiString(expect), item->getTrackNumber());
             } else if (root == "partnumber") {
                 match = checkValue(op, stoiString(expect), item->getPartNumber());
-            } else if (std::find_if(MetadataHandler::mt_keys.begin(), MetadataHandler::mt_keys.end(), [val = root](auto&& kvp) { return kvp.second == val; }) != MetadataHandler::mt_keys.end()) {
+            } else if (std::find_if(MetaEnumMapper::mt_keys.begin(), MetaEnumMapper::mt_keys.end(), [val = root](auto&& kvp) { return kvp.second == val; }) != MetaEnumMapper::mt_keys.end()) {
                 match = checkValue(op, expect, item->getMetaData(root));
             }
         }
@@ -118,24 +125,26 @@ ImportService::ImportService(std::shared_ptr<Context> context)
     , mime(this->context->getMime())
     , database(this->context->getDatabase())
 {
-    hasReadableNames = config->getBoolOption(CFG_IMPORT_READABLE_NAMES);
-    hasDefaultDate = config->getBoolOption(CFG_IMPORT_DEFAULT_DATE);
-    mimetypeContenttypeMap = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
-    mimetypeUpnpclassMap = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_UPNP_CLASS_LIST);
-    configLayoutMapping = config->getDictionaryOption(CFG_IMPORT_LAYOUT_MAPPING);
-    containerImageParentCount = config->getIntOption(CFG_IMPORT_RESOURCES_CONTAINERART_PARENTCOUNT);
-    containerImageMinDepth = config->getIntOption(CFG_IMPORT_RESOURCES_CONTAINERART_MINDEPTH);
-    virutalDirKeys = config->getArrayOption(CFG_IMPORT_VIRTUAL_DIRECTORY_KEYS);
-    noMediaName = config->getOption(CFG_IMPORT_NOMEDIA_FILE);
+    hasReadableNames = config->getBoolOption(ConfigVal::IMPORT_READABLE_NAMES);
+    hasCaseSensitiveNames = config->getBoolOption(ConfigVal::IMPORT_CASE_SENSITIVE_TAGS);
+    hasDefaultDate = config->getBoolOption(ConfigVal::IMPORT_DEFAULT_DATE);
+    mimetypeContenttypeMap = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+    mimetypeUpnpclassMap = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_UPNP_CLASS_LIST);
+    configLayoutMapping = config->getDictionaryOption(ConfigVal::IMPORT_LAYOUT_MAPPING);
+    containerImageParentCount = config->getIntOption(ConfigVal::IMPORT_RESOURCES_CONTAINERART_PARENTCOUNT);
+    containerImageMinDepth = config->getIntOption(ConfigVal::IMPORT_RESOURCES_CONTAINERART_MINDEPTH);
+    virtualDirKeys = config->getArrayOption(ConfigVal::IMPORT_VIRTUAL_DIRECTORY_KEYS);
+    noMediaName = config->getOption(ConfigVal::IMPORT_NOMEDIA_FILE);
     UpnpMap::initMap(upnpMap, mimetypeUpnpclassMap);
 }
 
 void ImportService::run(std::shared_ptr<ContentManager> content, std::shared_ptr<AutoscanDirectory> autoScan, fs::path path)
 {
     this->content = std::move(content);
+    metadataService = std::make_shared<MetadataService>(context, this->content);
     if (autoScan) {
         this->autoscanDir = std::move(autoScan);
-        this->rootPath = path;
+        this->rootPath = std::move(path);
     }
 }
 
@@ -165,6 +174,9 @@ void ImportService::initLayout(LayoutType layoutType)
     if (!playlistParserScript) {
         playlistParserScript = std::make_unique<PlaylistParserScript>(content, rootPath.string());
     }
+    if (!metafileParserScript) {
+        metafileParserScript = std::make_unique<MetafileParserScript>(content, rootPath.string());
+    }
 #endif
 }
 
@@ -173,6 +185,7 @@ void ImportService::destroyLayout()
     layout = nullptr;
 #ifdef HAVE_JS
     playlistParserScript = nullptr;
+    metafileParserScript = nullptr;
 #endif
 }
 
@@ -203,11 +216,19 @@ void ImportService::clearCache()
 
 void ImportService::doImport(const fs::path& location, AutoScanSetting& settings, std::unordered_set<int>& currentContent, const std::shared_ptr<GenericTask>& task)
 {
-    log_debug("start {}", location.string());
-    contentStateCache.clear();
+    log_debug("start {} root '{}'", location.string(), rootPath.string());
+    if (activeScan.empty()) {
+        contentStateCache.clear();
+        activeScan = location;
+    } else {
+        log_debug("Additional scan {}, already active {}", location.c_str(), activeScan.c_str());
+    }
 
+    if (!rootPath.empty()) {
+        auto rootDirEntry = fs::directory_entry(rootPath);
+        cacheState(rootPath, rootDirEntry, ImportState::New, toSeconds(rootDirEntry.last_write_time(ec)));
+    }
     auto rootEntry = fs::directory_entry(location);
-    contentStateCache[location] = std::make_shared<ContentState>(rootEntry, ImportState::New, toSeconds(rootEntry.last_write_time(ec)));
     if (ec) {
         log_error("Failed to start {}, {}", location.c_str(), ec.message());
         return;
@@ -218,6 +239,7 @@ void ImportService::doImport(const fs::path& location, AutoScanSetting& settings
         return;
     }
 
+    cacheState(location, rootEntry, ImportState::New, toSeconds(rootEntry.last_write_time(ec)));
     if (isDir) {
         readDir(location, settings);
     } else {
@@ -231,7 +253,9 @@ void ImportService::doImport(const fs::path& location, AutoScanSetting& settings
 
     // update currentContent
     for (auto&& [itemPath, stateEntry] : contentStateCache) {
-        if (stateEntry && stateEntry->getObject() && stateEntry->getState() == ImportState::Existing) {
+        if (!stateEntry)
+            continue;
+        if (stateEntry->getObject() && stateEntry->getState() == ImportState::Existing) {
             auto entry = currentContent.find(stateEntry->getObject()->getID());
             if (entry != currentContent.end()) {
                 currentContent.erase(stateEntry->getObject()->getID());
@@ -244,10 +268,13 @@ void ImportService::doImport(const fs::path& location, AutoScanSetting& settings
         log_debug("Updating last_modified for autoscan directory {}", autoscanDir->getLocation().c_str());
         database->updateAutoscanDirectory(autoscanDir);
     }
+    if (activeScan == location)
+        activeScan = "";
 }
 
 std::shared_ptr<CdsObject> ImportService::getObject(const fs::path& location) const
 {
+    log_debug("start {}", location.string());
     if (contentStateCache.find(location) != contentStateCache.end()) {
         return contentStateCache.at(location)->getObject();
     }
@@ -268,42 +295,61 @@ void ImportService::readDir(const fs::path& location, AutoScanSetting settings)
         if (isHiddenFile(entryPath, true, dirEntry, settings)) {
             continue;
         }
-        contentStateCache[entryPath] = std::make_shared<ContentState>(dirEntry, ImportState::New, toSeconds(dirEntry.last_write_time(ec)));
+        cacheState(entryPath, dirEntry, ImportState::New, toSeconds(dirEntry.last_write_time(ec)));
         if (dirEntry.is_directory(ec) && settings.recursive) {
             if (!ec) {
                 readDir(entryPath, settings);
             } else {
-                contentStateCache[entryPath]->setObject(ImportState::Broken, nullptr);
+                cacheState(entryPath, dirEntry, ImportState::Broken);
                 log_error("ImportService::readDir {}: Failed to read {}, {}", location.c_str(), entryPath.c_str(), ec.message());
             }
         } else if (ec) {
-            contentStateCache[entryPath]->setObject(ImportState::Broken, nullptr);
+            cacheState(entryPath, dirEntry, ImportState::Broken);
             log_error("ImportService::readDir {}: Failed to read {}, {}", location.c_str(), entryPath.c_str(), ec.message());
         }
     }
     log_debug("end {}", location.string());
 }
 
-void ImportService::readFile(const fs::path& location)
+void ImportService::cacheState(const fs::path& entryPath, const fs::directory_entry& dirEntry, ImportState state, std::chrono::seconds mtime, const std::shared_ptr<CdsObject>& cdsObject)
 {
-    auto dirEntry = fs::directory_entry(location.parent_path());
-    auto entryPath = dirEntry.path();
-    while (entryPath != "/" && entryPath != rootPath) {
-        contentStateCache[entryPath] = std::make_shared<ContentState>(dirEntry, ImportState::New, toSeconds(dirEntry.last_write_time(ec)));
-        dirEntry.assign(entryPath.parent_path(), ec);
-        if (ec) {
-            contentStateCache[entryPath]->setObject(ImportState::Broken, nullptr);
-            log_error("ImportService::readFile {}: Failed to navigate up {}, {}", location.c_str(), entryPath.c_str(), ec.message());
-            break;
-        }
-        entryPath = dirEntry.path();
+    if (contentStateCache.find(entryPath) == contentStateCache.end()) {
+        contentStateCache[entryPath] = std::make_shared<ContentState>(dirEntry, state, mtime, cdsObject);
+    } else {
+        state = contentStateCache[entryPath]->getState() < state ? state : contentStateCache[entryPath]->getState();
+        if (!cdsObject)
+            contentStateCache[entryPath]->setState(state);
+        else
+            contentStateCache[entryPath]->setObject(state, cdsObject);
+        if (mtime > std::chrono::seconds::zero())
+            contentStateCache[entryPath]->setMTime(mtime);
     }
 }
 
-void ImportService::removeHidden(AutoScanSetting& settings)
+void ImportService::readFile(const fs::path& location)
+{
+    auto dirEntry = fs::directory_entry(location.parent_path());
+    cacheState(location.parent_path(), dirEntry, ImportState::New, toSeconds(dirEntry.last_write_time(ec)));
+
+    auto entryPath = dirEntry.path().parent_path();
+    while (entryPath != "/" && entryPath != rootPath) {
+        dirEntry.assign(entryPath, ec);
+        if (ec) {
+            cacheState(entryPath, dirEntry, ImportState::Broken);
+            log_error("ImportService::readFile {}: Failed to navigate up {}, {}", location.c_str(), entryPath.c_str(), ec.message());
+            break;
+        }
+        cacheState(entryPath, dirEntry, ImportState::New, toSeconds(dirEntry.last_write_time(ec)));
+        entryPath = entryPath.parent_path();
+    }
+}
+
+void ImportService::removeHidden(const AutoScanSetting& settings)
 {
     auto hiddenPaths = std::vector<fs::path>();
     for (auto&& [itemPath, stateEntry] : contentStateCache) {
+        if (!stateEntry)
+            continue;
         auto dirEntry = stateEntry->getDirEntry();
         if (isHiddenFile(itemPath, dirEntry.is_directory(ec), dirEntry, settings)) {
             hiddenPaths.push_back(itemPath);
@@ -327,31 +373,31 @@ bool ImportService::isHiddenFile(const fs::path& entryPath, bool isDirectory, co
     if ((name[0] == '.' && !settings.hidden)
         || (!settings.followSymlinks && dirEntry.is_symlink())
         || config->getConfigFilename() == entryPath) {
-        contentStateCache[entryPath] = std::make_shared<ContentState>(dirEntry, ImportState::ToDelete);
+        cacheState(entryPath, dirEntry, ImportState::ToDelete);
         return true;
     }
     if (!noMediaName.empty()) {
         auto noMediaFile = (isDirectory) ? entryPath / noMediaName : entryPath.parent_path() / noMediaName;
         if (contentStateCache.find(noMediaFile) != contentStateCache.end()) {
-            return contentStateCache[entryPath]->getState() != ImportState::Broken; // broken means: file not found
+            return contentStateCache[noMediaFile]->getState() != ImportState::Broken; // broken means: file not found
+        }
+        auto noMediaEntry = fs::directory_entry(noMediaFile, ec);
+        if (!noMediaEntry.exists(ec) || ec) {
+            cacheState(noMediaEntry, noMediaEntry, ImportState::Broken, toSeconds(dirEntry.last_write_time(ec)));
         } else {
-            auto noMediaEntry = fs::directory_entry(noMediaFile, ec);
-            if (!noMediaEntry.exists(ec) || ec) {
-                contentStateCache[noMediaEntry] = std::make_shared<ContentState>(noMediaEntry, ImportState::Broken, toSeconds(dirEntry.last_write_time(ec)));
-            } else {
-                // if file exists it will be automatically created
-                return true;
-            }
+            // if file exists it will be automatically created
+            return true;
         }
     }
     return false;
 }
 
+///\brief create containers for all discovered folders
 void ImportService::createContainers(int parentContainerId, AutoScanSetting& settings)
 {
     log_debug("start {} {}", rootPath.string(), parentContainerId);
     for (auto&& [contPath, stateEntry] : contentStateCache) {
-        if (stateEntry->getState() != ImportState::New)
+        if (!stateEntry || stateEntry->getState() != ImportState::New)
             continue;
         auto dirEntry = stateEntry->getDirEntry();
         if (dirEntry.exists(ec) && dirEntry.is_directory(ec)) {
@@ -383,6 +429,7 @@ void ImportService::createContainers(int parentContainerId, AutoScanSetting& set
     log_debug("end {}", rootPath.string());
 }
 
+///\brief create items for all discovered files
 void ImportService::createItems(AutoScanSetting& settings)
 {
     log_debug("start {}", rootPath.string());
@@ -392,7 +439,12 @@ void ImportService::createItems(AutoScanSetting& settings)
     fs::path contPath;
 
     for (auto&& [itemPath, stateEntry] : contentStateCache) {
+        if (!stateEntry) {
+            log_debug("broken entry {}", itemPath.string());
+            continue;
+        }
         auto cdsObj = stateEntry->getObject();
+        // cache containers as parent item for following item
         if (cdsObj && cdsObj->isContainer()) {
             std::shared_ptr<CdsContainer> container = std::dynamic_pointer_cast<CdsContainer>(cdsObj);
             if (contPath != "") {
@@ -409,10 +461,14 @@ void ImportService::createItems(AutoScanSetting& settings)
                 autoscanDir->setCurrentLMT(contPath, std::chrono::seconds::zero());
             }
         }
-        if (stateEntry->getState() != ImportState::New)
+        if (stateEntry->getState() != ImportState::New) {
+            log_debug("wrong state entry {}", itemPath.string());
             continue;
+        }
+        // create items from files
         auto dirEntry = stateEntry->getDirEntry();
-        if (isRegularFile(dirEntry, ec)) { // item
+        if (isRegularFile(dirEntry, ec)) {
+            // Start with cached item
             auto contState = contentStateCache[itemPath.parent_path()];
             if (contState)
                 parentContainer = std::dynamic_pointer_cast<CdsContainer>(contState->getObject());
@@ -420,10 +476,12 @@ void ImportService::createItems(AutoScanSetting& settings)
                 log_error("No Container parent for Item {}", itemPath.string());
 
             if (!cdsObj) {
+                // Search item in database
                 log_debug("Searching Item {} in database", itemPath.string());
                 cdsObj = database->findObjectByPath(itemPath, DbFileType::File);
             }
             if (cdsObj && cdsObj->isItem() && stateEntry->getMTime() != cdsObj->getMTime()) {
+                // Update changed item in database
                 log_debug("Updating Item {} in database {}", itemPath.string(), cdsObj->getID());
                 auto item = std::dynamic_pointer_cast<CdsItem>(cdsObj);
                 item->clearMetaData();
@@ -436,6 +494,7 @@ void ImportService::createItems(AutoScanSetting& settings)
                 stateEntry->setObject(ImportState::Created, cdsObj);
                 log_debug("Item changed {} {}", itemPath.string(), cdsObj->getID());
             } else if (cdsObj) {
+                // Store local item
                 if (contState && contState->getMTime() < cdsObj->getMTime()) {
                     contState->setMTime(cdsObj->getMTime());
                     if (lastModifiedNewMax < cdsObj->getMTime())
@@ -444,9 +503,11 @@ void ImportService::createItems(AutoScanSetting& settings)
                 stateEntry->setObject(ImportState::Existing, cdsObj);
                 log_debug("Item found {} {}", itemPath.string(), cdsObj->getID());
             } else {
+                // Create item from scratch
                 log_debug("Creating Item {}", itemPath.string());
-                cdsObj = createSingleItem(dirEntry);
-                if (cdsObj) {
+                auto [skip, newCdsObj] = createSingleItem(dirEntry);
+                if (newCdsObj) {
+                    cdsObj = newCdsObj;
                     if (contState) {
                         contState->setMTime(cdsObj->getMTime());
                         if (lastModifiedNewMax < cdsObj->getMTime())
@@ -458,7 +519,8 @@ void ImportService::createItems(AutoScanSetting& settings)
                 } else {
                     stateEntry->setObject(ImportState::Broken, cdsObj);
                     cdsObj = nullptr;
-                    log_error("Object not created for file {}", dirEntry.path().string());
+                    if (!skip)
+                        log_error("Object not created for file {}", dirEntry.path().string());
                 }
             }
             if (contState && cdsObj) {
@@ -467,6 +529,8 @@ void ImportService::createItems(AutoScanSetting& settings)
             if (parentContainer) {
                 stateEntry->setParentObject(parentContainer);
             }
+        } else {
+            log_debug("Not a file {}", itemPath.string());
         }
     }
     if (autoscanDir && contPath != "") {
@@ -475,15 +539,18 @@ void ImportService::createItems(AutoScanSetting& settings)
     log_debug("end {}", rootPath.string());
 }
 
-std::shared_ptr<CdsObject> ImportService::createSingleItem(const fs::directory_entry& dirEntry) // ToDo: Use StateEntry here
+std::pair<bool, std::shared_ptr<CdsObject>> ImportService::createSingleItem(const fs::directory_entry& dirEntry) // ToDo: Use StateEntry here
 {
-    auto objectPath = dirEntry.path();
+    const auto& objectPath = dirEntry.path();
 
     /* retrieve information about item and decide if it should be included */
-    std::string mimetype = mime->getMimeType(objectPath, MIMETYPE_DEFAULT);
+    auto [skip, mimetype] = mime->getMimeType(objectPath, MIMETYPE_DEFAULT);
     if (mimetype.empty()) {
-        log_error("Mime not found for file {}", objectPath.c_str());
-        return nullptr;
+        if (skip)
+            log_debug("Mime set empty for file {}", objectPath.c_str());
+        else
+            log_error("Mime not found for file {}", objectPath.c_str());
+        return { skip, nullptr };
     }
     log_debug("Mime '{}' for file {}", mimetype, objectPath.c_str());
 
@@ -518,17 +585,17 @@ std::shared_ptr<CdsObject> ImportService::createSingleItem(const fs::directory_e
     item->setTitle(f2i->convert(title));
     updateSingleItem(dirEntry, item, mimetype);
 
-    return item;
+    return { skip, item };
 }
 
-void ImportService::updateSingleItem(const fs::directory_entry& dirEntry, std::shared_ptr<CdsItem> item, const std::string& mimetype)
+void ImportService::updateSingleItem(const fs::directory_entry& dirEntry, const std::shared_ptr<CdsItem>& item, const std::string& mimetype)
 {
     auto mTime = toSeconds(dirEntry.last_write_time(ec));
     item->setMTime(mTime);
     item->setUTime(mTime);
     item->setSizeOnDisk(getFileSize(dirEntry));
 
-    MetadataHandler::extractMetaData(context, content, item, dirEntry);
+    metadataService->extractMetaData(item, dirEntry);
     updateItemData(item, mimetype);
 }
 
@@ -545,7 +612,8 @@ void ImportService::fillLayout(const std::shared_ptr<GenericTask>& task)
 /// \param object used to make code compatible with legacy scan
 void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state, std::shared_ptr<CdsObject> object, const std::shared_ptr<CdsContainer>& parent, const std::shared_ptr<GenericTask>& task)
 {
-    std::shared_ptr<CdsObject> cdsObject = state ? state->getObject() : object;
+    std::shared_ptr<CdsObject> cdsObject = state ? state->getObject() : std::move(object);
+    log_debug("cds {}, layout {}, autoscanDir {}", !!cdsObject, !!layout, !!autoscanDir);
 
     if (cdsObject && cdsObject->isItem() && layout) {
         try {
@@ -556,17 +624,11 @@ void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state,
                 // only lock mutex while processing item layout
                 std::scoped_lock<decltype(layoutMutex)> lock(layoutMutex);
                 // get ref'd objects with last mod time
-                auto listOrig = state ? database->getRefObjects(cdsObject->getID()) : std::vector<std::pair<int, std::chrono::seconds>> {};
-                layout->processCdsObject(cdsObject, parent, rootPath, contentType, autoscanDir ? autoscanDir->getContainerTypes() : AutoscanDirectory::ContainerTypesDefaults);
-                // compare ref'd objects last mod time
-                auto listResult = state ? database->getRefObjects(cdsObject->getID()) : std::vector<std::pair<int, std::chrono::seconds>> {};
-                for (auto&& origEntry : listOrig) {
-                    auto newEntry = std::find_if(listResult.begin(), listResult.end(), [&](auto& entry) { return origEntry.first == entry.first && origEntry.second > entry.second; });
-                    if (newEntry == listResult.end()) {
-                        log_debug("Deleting orphaned virtual item {}", origEntry.first);
-                        database->removeObject(origEntry.first, "", false);
-                    }
-                }
+                auto refObjects = state ? database->getRefObjects(cdsObject->getID()) : std::vector<int> {};
+                layout->processCdsObject(cdsObject, parent,
+                    rootPath, contentType,
+                    autoscanDir ? autoscanDir->getContainerTypes() : AutoscanDirectory::ContainerTypesDefaults,
+                    refObjects);
             } else {
                 log_debug("file ignored: {} autoscanDir={}, class={}, hasContent={}, mediaType={}", cdsObject->getLocation().string(), !!autoscanDir, cdsObject->getClass(), autoscanDir ? autoscanDir->hasContent(cdsObject->getClass()) : false, autoscanDir ? autoscanDir->getMediaType() : -2);
             }
@@ -574,7 +636,7 @@ void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state,
 #ifdef HAVE_JS
             try {
                 if (playlistParserScript && contentType == CONTENT_TYPE_PLAYLIST)
-                    playlistParserScript->processPlaylistObject(cdsObject, std::move(task), rootPath);
+                    playlistParserScript->processPlaylistObject(cdsObject, task, rootPath);
             } catch (const std::runtime_error& e) {
                 log_error("{}", e.what());
             }
@@ -588,10 +650,24 @@ void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state,
     }
 }
 
+void ImportService::parseMetafile(const std::shared_ptr<CdsObject>& obj, const fs::path& path) const
+{
+#ifdef HAVE_JS
+    try {
+        if (metafileParserScript)
+            metafileParserScript->processObject(obj, path);
+    } catch (const std::runtime_error& e) {
+        log_error("{}: {}", path.string(), e.what());
+    }
+#else
+    log_warning("Metadata file {} will not be parsed: Gerbera was compiled without JS support!", path.string());
+#endif // HAVE_JS
+}
+
 void ImportService::updateItemData(const std::shared_ptr<CdsItem>& item, const std::string& mimetype)
 {
-    if (hasDefaultDate && item->getMetaData(M_DATE).empty())
-        item->addMetaData(M_DATE, fmt::format("{:%FT%T%z}", fmt::localtime(item->getMTime().count())));
+    if (hasDefaultDate && item->getMetaData(MetadataFields::M_DATE).empty())
+        item->addMetaData(MetadataFields::M_DATE, fmt::format("{:%FT%T%z}", fmt::localtime(item->getMTime().count())));
     for (auto&& upnpPattern : upnpMap) {
         if (upnpPattern.isMatch(item, mimetype)) {
             item->setClass(upnpPattern.upnpClass);
@@ -632,10 +708,10 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
         return;
     }
 
-    auto fanart = container->getResource(CdsResource::Purpose::Thumbnail);
+    auto fanart = container->getResource(ResourcePurpose::Thumbnail);
     if (fanart && fanart->getHandlerType() != ContentHandler::CONTAINERART) {
         // remove stale references
-        auto fanartObjId = stoiString(fanart->getAttribute(CdsResource::Attribute::FANART_OBJ_ID));
+        auto fanartObjId = stoiString(fanart->getAttribute(ResourceAttribute::FANART_OBJ_ID));
         try {
             if (fanartObjId > CDS_ID_ROOT) {
                 database->loadObject(fanartObjId);
@@ -646,53 +722,54 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
         }
     }
     if (!fanart || fanart->getHandlerType() != ContentHandler::CONTAINERART) {
-        MetadataHandler::createHandler(context, nullptr, ContentHandler::CONTAINERART)->fillMetadata(container);
-        auto containerart = container->getResource(CdsResource::Purpose::Thumbnail);
-        if (containerart) {
+        if (fanart) {
             container->clearResources();
-            container->addResource(containerart); // overwrite all other resources by container art
-            database->updateObject(container, nullptr);
+        }
+        metadataService->getHandler(ContentHandler::CONTAINERART)->fillMetadata(container);
+        auto containerart = container->getResource(ResourcePurpose::Thumbnail);
+        if (containerart) {
             fanart = containerart;
         }
         log_debug("fanart from dir {}", containerart != nullptr);
     }
 
-    auto location = container->getLocation();
-    if (refObj) {
-        if (!fanart && (refObj->isContainer() || (count < containerImageParentCount && container->getParentID() != CDS_ID_ROOT && std::distance(location.begin(), location.end()) > containerImageMinDepth))) {
-            fanart = refObj->getResource(CdsResource::Purpose::Thumbnail);
-            if (fanart) {
+    if (refObj && !fanart) {
+        auto location = container->getLocation();
+        if (refObj->isContainer() || (count < containerImageParentCount && container->getParentID() != CDS_ID_ROOT && std::distance(location.begin(), location.end()) > containerImageMinDepth)) {
+            auto refFanArt = refObj->getResource(ResourcePurpose::Thumbnail);
+            if (refFanArt) {
                 auto fanArtObj = refObj;
                 auto fanArtObjId = refObj->getID();
-                auto fanArtResId = fanart->getResId();
+                auto fanArtResId = refFanArt->getResId();
                 if (refObj->getID() <= CDS_ID_ROOT) {
                     fanArtObj = database->loadObject(refObj->getRefID());
-                    auto fanartRef = fanArtObj->getResource(CdsResource::Purpose::Thumbnail);
+                    auto fanartRef = fanArtObj->getResource(ResourcePurpose::Thumbnail);
                     fanArtResId = fanartRef->getResId();
                 }
                 if (fanArtObj->isItem() && fanArtResId == 0) {
                     for (auto&& res : fanArtObj->getResources()) {
-                        if (res->getPurpose() == CdsResource::Purpose::Thumbnail)
+                        if (res->getPurpose() == ResourcePurpose::Thumbnail)
                             break;
                         fanArtResId++;
                     }
                 }
-                if (fanart->getAttribute(CdsResource::Attribute::RESOURCE_FILE).empty()) {
+                if (refFanArt->getAttribute(ResourceAttribute::RESOURCE_FILE).empty()) {
                     if (fanArtObjId > CDS_ID_ROOT) {
-                        fanart->addAttribute(CdsResource::Attribute::FANART_OBJ_ID, fmt::to_string(fanArtObjId));
-                        fanart->addAttribute(CdsResource::Attribute::FANART_RES_ID, fmt::to_string(fanArtResId));
-                    } else {
-                        fanart = nullptr;
+                        refFanArt->addAttribute(ResourceAttribute::FANART_OBJ_ID, fmt::to_string(fanArtObjId));
+                        refFanArt->addAttribute(ResourceAttribute::FANART_RES_ID, fmt::to_string(fanArtResId));
+                        fanart = refFanArt;
                     }
+                } else {
+                    fanart = refFanArt;
                 }
-                if (fanart)
-                    container->addResource(fanart);
                 log_debug("fanart from ref {}", fanart != nullptr);
-                database->updateObject(container, nullptr);
             }
         }
     }
     if (fanart) {
+        container->clearResources();
+        container->addResource(fanart); // overwrite all other resources
+        database->updateObject(container, nullptr);
         containersWithFanArt[container->getID()] = container;
     }
 }
@@ -746,7 +823,7 @@ std::pair<int, bool> ImportService::addContainerTree(
                 tree = std::regex_replace(tree, std::regex(key), val);
             }
             auto dirKeyValues = std::vector<std::string>();
-            for (auto&& field : virutalDirKeys) {
+            for (auto&& field : virtualDirKeys) {
                 if (field == "LOCATION") {
                     std::string location = item->getLocation().c_str();
                     if (!location.empty()) {
@@ -754,7 +831,7 @@ std::pair<int, bool> ImportService::addContainerTree(
                         item->setLocation("");
                     }
                 } else {
-                    auto metaField = MetadataHandler::remapMetaDataField(field);
+                    auto metaField = MetaEnumMapper::remapMetaDataField(field);
                     auto keyValue = item->getMetaData(metaField);
                     if (!keyValue.empty())
                         dirKeyValues.push_back(keyValue);
@@ -765,6 +842,9 @@ std::pair<int, bool> ImportService::addContainerTree(
             } else {
                 subTree = tree;
             }
+            if (!hasCaseSensitiveNames) {
+                subTree = toLower(subTree);
+            }
             if (containerMap.find(subTree) == containerMap.end() || !containerMap[subTree]) {
                 auto cont = database->findObjectByPath(subTree, DbFileType::Virtual);
                 if (cont && cont->isContainer())
@@ -777,8 +857,8 @@ std::pair<int, bool> ImportService::addContainerTree(
             subTree = tree;
         }
         if (containerMap.find(subTree) == containerMap.end() || !containerMap[subTree]) {
-            item->removeMetaData(M_TITLE);
-            item->addMetaData(M_TITLE, item->getTitle());
+            item->removeMetaData(MetadataFields::M_TITLE);
+            item->addMetaData(MetadataFields::M_TITLE, item->getTitle());
             item->setParentID(result);
             auto cont = std::dynamic_pointer_cast<CdsContainer>(item);
             cont->setVirtual(isVirtual);
