@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,16 +32,17 @@
 /// \file atrailers_content_handler.cc
 
 #ifdef ATRAILERS
-#define LOG_FAC log_facility_t::online
+#define GRB_LOG_FAC GrbLogFacility::online
 #include "atrailers_content_handler.h" // API
 
-#include <array>
-
 #include "cds/cds_item.h"
 #include "config/config.h"
-#include "metadata/metadata_handler.h"
+#include "config/config_val.h"
+#include "exceptions.h"
 #include "util/tools.h"
 
+#include <array>
+
 void ATrailersContentHandler::setServiceContent(std::unique_ptr<pugi::xml_document> service)
 {
     service_xml = std::move(service);
@@ -52,7 +53,7 @@ void ATrailersContentHandler::setServiceContent(std::unique_ptr<pugi::xml_docume
 
     trailer_it = root.begin();
 
-    auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST);
+    auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST);
     trailer_mimetype = getValueOrDefault(mappings, "mov", "video/quicktime");
 }
 
@@ -85,7 +86,7 @@ std::shared_ptr<CdsObject> ATrailersContentHandler::getNextObject()
 std::shared_ptr<CdsObject> ATrailersContentHandler::getObject(const pugi::xml_node& trailer) const
 {
     auto item = std::make_shared<CdsItemExternalURL>();
-    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, CdsResource::Purpose::Content);
+    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
     item->addResource(resource);
 
     auto info = trailer.child("info");
@@ -99,7 +100,7 @@ std::shared_ptr<CdsObject> ATrailersContentHandler::getObject(const pugi::xml_no
         item->setTitle(temp);
 
     item->setMimeType(trailer_mimetype);
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(trailer_mimetype));
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(trailer_mimetype));
 
     item->setAuxData(ONLINE_SERVICE_AUX_ID, fmt::to_string(to_underlying(OnlineServiceType::OS_ATrailers)));
 
@@ -135,12 +136,12 @@ std::shared_ptr<CdsObject> ATrailersContentHandler::getObject(const pugi::xml_no
     item->setClass("object.item.videoItem");
 
     auto propertyMap = std::array {
-        std::pair("rating", M_RATING),
-        std::pair("studio", M_PRODUCER),
-        std::pair("director", M_DIRECTOR),
-        std::pair("releasedate", M_DATE),
+        std::pair("rating", MetadataFields::M_RATING),
+        std::pair("studio", MetadataFields::M_PRODUCER),
+        std::pair("director", MetadataFields::M_DIRECTOR),
+        std::pair("releasedate", MetadataFields::M_DATE),
         /// \todo cut out a small part for the usual description
-        std::pair("description", M_LONGDESCRIPTION),
+        std::pair("description", MetadataFields::M_LONGDESCRIPTION),
     };
     for (auto&& [key, tag] : propertyMap) {
         temp = info.child(key).text().as_string();
@@ -163,7 +164,7 @@ std::shared_ptr<CdsObject> ATrailersContentHandler::getObject(const pugi::xml_no
 
             temp = actor.text().as_string();
             if (!temp.empty()) {
-                item->addMetaData(M_ARTIST, temp);
+                item->addMetaData(MetadataFields::M_ARTIST, temp);
             }
         }
     }
@@ -178,7 +179,7 @@ std::shared_ptr<CdsObject> ATrailersContentHandler::getObject(const pugi::xml_no
 
             temp = gn.text().as_string();
             if (!temp.empty()) {
-                item->addMetaData(M_GENRE, temp);
+                item->addMetaData(MetadataFields::M_GENRE, temp);
             }
         }
     }
