@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,30 +32,34 @@
 /// \file import_script.cc
 
 #ifdef HAVE_JS
-#define LOG_FAC log_facility_t::script
+#define GRB_LOG_FAC GrbLogFacility::script
 #include "import_script.h" // API
 
 #include "cds/cds_objects.h"
-#include "content/autoscan.h"
-#include "content/content_manager.h"
+#include "common.h"
+#include "config/config_val.h"
+#include "config/result/autoscan.h"
+#include "content/content.h"
+#include "context.h"
 #include "js_functions.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 
-ImportScript::ImportScript(const std::shared_ptr<ContentManager>& content, const std::string& parent)
+ImportScript::ImportScript(const std::shared_ptr<Content>& content, const std::string& parent)
     : Script(content, parent, "import", "orig", StringConverter::i2i(content->getContext()->getConfig()))
 {
-    std::string scriptPath = config->getOption(CFG_IMPORT_SCRIPTING_IMPORT_SCRIPT);
+    std::string scriptPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_SCRIPT);
     if (!scriptPath.empty()) {
         load(scriptPath);
     }
 }
 
-void ImportScript::processCdsObject(const std::shared_ptr<CdsObject>& obj, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> ImportScript::processCdsObject(const std::shared_ptr<CdsObject>& obj, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
+    std::vector<int> result;
     processed = obj;
     try {
-        execute(obj, scriptPath);
+        result = execute(obj, scriptPath);
     } catch (const std::runtime_error&) {
         cleanup();
         processed = nullptr;
@@ -69,10 +73,12 @@ void ImportScript::processCdsObject(const std::shared_ptr<CdsObject>& obj, const
         duk_gc(ctx, 0);
         gc_counter = 0;
     }
+    return result;
 }
 
-void ImportScript::callFunction(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const std::string& function, const fs::path& scriptPath, AutoscanMediaMode mediaMode, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> ImportScript::callFunction(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const std::string& function, const fs::path& scriptPath, AutoscanMediaMode mediaMode, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
+    std::vector<int> result;
     processed = obj;
     try {
         auto autoScan = content->getAutoscanDirectory(scriptPath);
@@ -84,7 +90,7 @@ void ImportScript::callFunction(const std::shared_ptr<CdsObject>& obj, const std
         }
 
         log_debug("{}, path = {}, containerType = {}", function, scriptPath.c_str(), containerType);
-        call(obj, cont, function, scriptPath, containerType);
+        result = call(obj, cont, function, scriptPath, containerType);
     } catch (const std::runtime_error&) {
         processed = nullptr;
         throw;
@@ -97,33 +103,34 @@ void ImportScript::callFunction(const std::shared_ptr<CdsObject>& obj, const std
         duk_gc(ctx, 0);
         gc_counter = 0;
     }
+    return result;
 }
 
-void ImportScript::addAudio(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> ImportScript::addAudio(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    callFunction(obj, cont, config->getOption(CFG_IMPORT_SCRIPTING_IMPORT_FUNCTION_AUDIOFILE), scriptPath, AutoscanMediaMode::Audio, containerMap);
+    return callFunction(obj, cont, config->getOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_FUNCTION_AUDIOFILE), scriptPath, AutoscanMediaMode::Audio, containerMap);
 }
 
-void ImportScript::addVideo(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> ImportScript::addVideo(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    callFunction(obj, cont, config->getOption(CFG_IMPORT_SCRIPTING_IMPORT_FUNCTION_VIDEOFILE), scriptPath, AutoscanMediaMode::Video, containerMap);
+    return callFunction(obj, cont, config->getOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_FUNCTION_VIDEOFILE), scriptPath, AutoscanMediaMode::Video, containerMap);
 }
 
-void ImportScript::addImage(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> ImportScript::addImage(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    callFunction(obj, cont, config->getOption(CFG_IMPORT_SCRIPTING_IMPORT_FUNCTION_IMAGEFILE), scriptPath, AutoscanMediaMode::Video, containerMap);
+    return callFunction(obj, cont, config->getOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_FUNCTION_IMAGEFILE), scriptPath, AutoscanMediaMode::Video, containerMap);
 }
 
 #ifdef ONLINE_SERVICES
-void ImportScript::addTrailer(const std::shared_ptr<CdsObject>& obj, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> ImportScript::addTrailer(const std::shared_ptr<CdsObject>& obj, const fs::path& scriptPath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    callFunction(obj, nullptr, config->getOption(CFG_IMPORT_SCRIPTING_IMPORT_FUNCTION_TRAILER), scriptPath, AutoscanMediaMode::Video, containerMap);
+    return callFunction(obj, nullptr, config->getOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_FUNCTION_TRAILER), scriptPath, AutoscanMediaMode::Video, containerMap);
 }
 #endif
 
 bool ImportScript::hasImportFunctions() const
 {
-    return config->getOption(CFG_IMPORT_SCRIPTING_IMPORT_SCRIPT).empty();
+    return config->getOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_SCRIPT).empty();
 }
 
 bool ImportScript::setRefId(const std::shared_ptr<CdsObject>& cdsObj, const std::shared_ptr<CdsObject>& origObject, int pcdId)
