@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,25 +32,24 @@
 /// \file script.cc
 
 #ifdef HAVE_JS
-#define LOG_FAC log_facility_t::script
+#define GRB_LOG_FAC GrbLogFacility::script
 #include "script.h" // API
 
-#include <array>
-#include <fmt/chrono.h>
-
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
 #include "config/config_definition.h"
-#include "config/config_setup.h"
+#include "config/result/autoscan.h"
+#include "config/result/box_layout.h"
+#include "config/setup/config_setup_array.h"
 #include "config/setup/config_setup_autoscan.h"
 #include "config/setup/config_setup_boxlayout.h"
-#include "content/autoscan.h"
-#include "content/content_manager.h"
-#include "content/layout/box_layout.h"
+#include "config/setup/config_setup_dictionary.h"
+#include "content/content.h"
+#include "context.h"
 #include "database/database.h"
 #include "js_functions.h"
-#include "metadata/metadata_handler.h"
 #include "script_names.h"
+#include "script_property.h"
 #include "scripting_runtime.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
@@ -63,7 +62,11 @@
 #include "content/onlineservice/atrailers_content_handler.h"
 #endif
 
-static constexpr auto jsGlobalFunctions = std::array {
+#include <array>
+#include <fmt/chrono.h>
+
+static constexpr std::array jsGlobalFunctions {
+    duk_function_list_entry { "print2", js_print2, DUK_VARARGS },
     duk_function_list_entry { "print", js_print, DUK_VARARGS },
     duk_function_list_entry { "addCdsObject", js_addCdsObject, 3 },
     duk_function_list_entry { "addContainerTree", js_addContainerTree, 1 },
@@ -75,90 +78,6 @@ static constexpr auto jsGlobalFunctions = std::array {
     duk_function_list_entry { nullptr, nullptr, 0 },
 };
 
-std::string Script::getProperty(const std::string& name) const
-{
-    std::string ret;
-    if (!duk_is_object_coercible(ctx, -1))
-        return ret;
-    duk_get_prop_string(ctx, -1, name.c_str());
-    if (duk_is_null_or_undefined(ctx, -1) || !duk_to_string(ctx, -1)) {
-        duk_pop(ctx);
-        return ret;
-    }
-    ret = duk_get_string(ctx, -1);
-    duk_pop(ctx);
-    return ret;
-}
-
-int Script::getBoolProperty(const std::string& name) const
-{
-    if (!duk_is_object_coercible(ctx, -1))
-        return -1;
-    duk_get_prop_string(ctx, -1, name.c_str());
-    if (duk_is_null_or_undefined(ctx, -1)) {
-        duk_pop(ctx);
-        return -1;
-    }
-    auto ret = duk_to_boolean(ctx, -1);
-    duk_pop(ctx);
-    return ret;
-}
-
-int Script::getIntProperty(const std::string& name, int def) const
-{
-    if (!duk_is_object_coercible(ctx, -1))
-        return def;
-    duk_get_prop_string(ctx, -1, name.c_str());
-    if (duk_is_null_or_undefined(ctx, -1)) {
-        duk_pop(ctx);
-        return def;
-    }
-    auto ret = duk_to_int32(ctx, -1);
-    duk_pop(ctx);
-    return ret;
-}
-
-std::vector<std::string> Script::getArrayProperty(const std::string& name) const
-{
-    std::vector<std::string> result;
-    if (!duk_is_object_coercible(ctx, -1))
-        return result;
-    duk_get_prop_string(ctx, -1, name.c_str());
-    if (duk_is_null_or_undefined(ctx, -1) || !duk_is_array(ctx, -1)) {
-        duk_pop(ctx);
-        return result;
-    }
-    duk_enum(ctx, -1, 0);
-    while (duk_next(ctx, -1, 1 /* get_value */)) {
-        duk_get_string(ctx, -1);
-        if (duk_is_null_or_undefined(ctx, -1)) {
-            duk_pop_2(ctx);
-            continue;
-        }
-        // [key = duk_to_string(ctx, -2)]
-        auto val = std::string(duk_to_string(ctx, -1));
-        result.push_back(std::move(val));
-        duk_pop_2(ctx); /* pop_key */
-    }
-    duk_pop(ctx); // duk_enum
-    duk_pop(ctx); // property
-    return result;
-}
-
-std::vector<std::string> Script::getPropertyNames() const
-{
-    std::vector<std::string> keys;
-    duk_enum(ctx, -1, 0);
-    while (duk_next(ctx, -1, 0 /*get_key*/)) {
-        /* [ ... enum key ] */
-        auto sym = std::string(duk_get_string(ctx, -1));
-        keys.push_back(std::move(sym));
-        duk_pop(ctx); /* pop_key */
-    }
-    duk_pop(ctx); // duk_enum
-    return keys;
-}
-
 void Script::setProperty(const std::string& name, const std::string& value)
 {
     duk_push_string(ctx, value.c_str());
@@ -173,7 +92,7 @@ void Script::setIntProperty(const std::string& name, int value)
 
 /* **************** */
 
-Script::Script(const std::shared_ptr<ContentManager>& content, const std::string& parent,
+Script::Script(const std::shared_ptr<Content>& content, const std::string& parent,
     const std::string& name, std::string objName, std::unique_ptr<StringConverter> sc)
     : config(content->getContext()->getConfig())
     , database(content->getContext()->getDatabase())
@@ -183,7 +102,8 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
     , contextName(fmt::format("{}_{}", name, parent))
     , objectName(std::move(objName))
 {
-    entrySeparator = config->getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
+    hasCaseSensitiveNames = config->getBoolOption(ConfigVal::IMPORT_CASE_SENSITIVE_TAGS);
+    entrySeparator = config->getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
     /* create a context and associate it with the JS run time */
     ScriptingRuntime::AutoLock lock(runtime->getMutex());
     replaceAllString(contextName, "/", "_");
@@ -193,7 +113,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
 
     _p2i = StringConverter::p2i(config);
     _j2i = StringConverter::j2i(config);
-    _m2i = StringConverter::m2i(CFG_IMPORT_METADATA_CHARSET, "", config);
+    _m2i = StringConverter::m2i(ConfigVal::IMPORT_METADATA_CHARSET, "", config);
     _f2i = StringConverter::f2i(config);
     _i2i = StringConverter::i2i(config);
 
@@ -238,7 +158,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
 #endif // ONLINE_SERVICES
 
     // M_TITLE = "dc:title"
-    for (auto&& [field, sym] : MetadataHandler::mt_keys) {
+    for (auto&& [field, sym] : MetaEnumMapper::mt_keys) {
         auto s = getValueOrDefault(mt_names, field, { "" });
         if (!s.empty()) {
             duk_push_lstring(ctx, sym.data(), sym.size());
@@ -248,7 +168,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
 
     // R_SIZE = "size"
     for (auto&& [field, sym] : res_names) {
-        duk_push_string(ctx, CdsResource::getAttributeName(field).c_str());
+        duk_push_string(ctx, EnumMapper::getAttributeName(field).c_str());
         duk_put_global_lstring(ctx, sym.data(), sym.length());
     }
 
@@ -258,6 +178,11 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
         duk_put_global_lstring(ctx, sym.data(), sym.size());
     }
 
+    for (auto&& [field, sym] : boxKeyNames) {
+        duk_push_lstring(ctx, sym.data(), sym.length());
+        duk_put_global_lstring(ctx, field.data(), field.length());
+    }
+
     duk_push_object(ctx); // config
     for (auto&& i : ConfigOptionIterator()) {
         auto scs = ConfigDefinition::findConfigSetup(i, true);
@@ -265,7 +190,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
             continue;
         auto value = scs->getCurrentValue();
         if (!value.empty()) {
-            setProperty(scs->getItemPath(-1), value);
+            setProperty(scs->getItemPath(ITEM_PATH_ROOT, {}), value);
         }
     }
 
@@ -275,7 +200,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
         for (auto&& [key, val] : dictionary) {
             setProperty(key.substr(5), val);
         }
-        duk_put_prop_string(ctx, -2, dcs->getItemPath(-1).c_str());
+        duk_put_prop_string(ctx, -2, dcs->getItemPath(ITEM_PATH_ROOT, {}).c_str());
     }
 
     for (auto&& acs : ConfigDefinition::getConfigSetupList<ConfigArraySetup>()) {
@@ -286,7 +211,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
             duk_push_string(ctx, entry.c_str());
             duk_put_prop_index(ctx, dukArray, i);
         }
-        duk_put_prop_string(ctx, -2, acs->getItemPath(-1).c_str());
+        duk_put_prop_string(ctx, -2, acs->getItemPath(ITEM_PATH_ROOT, {}).c_str());
     }
 
     duk_push_object(ctx); // autoscan
@@ -295,19 +220,19 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
         auto autoscan = ascs->getValue()->getAutoscanListOption();
         for (const auto& adir : content->getAutoscanDirectories()) {
             duk_push_object(ctx);
-            setProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_LOCATION), adir->getLocation());
-            setProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_MODE), AutoscanDirectory::mapScanmode(adir->getScanMode()));
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_INTERVAL), adir->getInterval().count());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_RECURSIVE), adir->getRecursive());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_MEDIATYPE), adir->getMediaType());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES), adir->getHidden());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_SCANCOUNT), adir->getActiveScanCount());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_TASKCOUNT), adir->getTaskCount());
-            setProperty(ConfigDefinition::removeAttribute(ATTR_AUTOSCAN_DIRECTORY_LMT), fmt::format("{:%Y-%m-%d %H:%M:%S}", fmt::localtime(adir->getPreviousLMT().count())));
+            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_LOCATION), adir->getLocation());
+            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_MODE), AutoscanDirectory::mapScanmode(adir->getScanMode()));
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_INTERVAL), adir->getInterval().count());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_RECURSIVE), adir->getRecursive());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_MEDIATYPE), adir->getMediaType());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_HIDDENFILES), adir->getHidden());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_SCANCOUNT), adir->getActiveScanCount());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_TASKCOUNT), adir->getTaskCount());
+            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_LMT), fmt::format("{:%Y-%m-%d %H:%M:%S}", fmt::localtime(adir->getPreviousLMT().count())));
 
             duk_put_prop_string(ctx, -2, fmt::to_string(adir->getScanID()).c_str());
         }
-        autoscanItemPath = ascs->getItemPath(ITEM_PATH_PREFIX); // prefix
+        autoscanItemPath = ascs->getItemPath(ITEM_PATH_PREFIX, {}); // prefix
     }
     duk_put_prop_string(ctx, -2, autoscanItemPath.c_str()); // autoscan
 
@@ -319,13 +244,13 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
             duk_push_object(ctx);
             auto boxLayout = boxLayoutList->get(i);
             setIntProperty("id", boxLayout->getId());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_SIZE), boxLayout->getSize());
-            setIntProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_ENABLED), boxLayout->getEnabled());
-            setProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_TITLE), boxLayout->getTitle());
-            setProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_CLASS), boxLayout->getClass());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_SIZE), boxLayout->getSize());
+            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_ENABLED), boxLayout->getEnabled());
+            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_TITLE), boxLayout->getTitle());
+            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_CLASS), boxLayout->getClass());
             duk_put_prop_string(ctx, -2, boxLayout->getKey().c_str());
         }
-        boxLayoutItemPath = bcs->getItemPath(ITEM_PATH_PREFIX); // prefix
+        boxLayoutItemPath = bcs->getItemPath(ITEM_PATH_PREFIX, {}); // prefix
         duk_put_prop_string(ctx, -2, boxLayoutItemPath.c_str()); // box-layout
     }
 
@@ -333,11 +258,11 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
 
     defineFunctions(jsGlobalFunctions.data());
 
-    std::string commonScrPath = config->getOption(CFG_IMPORT_SCRIPTING_COMMON_SCRIPT);
-    std::string commonFdrPath = config->getOption(CFG_IMPORT_SCRIPTING_COMMON_FOLDER);
+    std::string commonScrPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_COMMON_SCRIPT);
+    std::string commonFdrPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_COMMON_FOLDER);
 
     if (commonScrPath.empty() && commonFdrPath.empty()) {
-        log_js("Common script disabled in configuration");
+        log_warning("Common script disabled in configuration");
     } else if (commonScrPath.empty()) {
         loadFolder(commonFdrPath);
     } else {
@@ -345,17 +270,17 @@ Script::Script(const std::shared_ptr<ContentManager>& content, const std::string
             _load(commonScrPath);
             _execute();
         } catch (const std::runtime_error& e) {
-            log_js("Unable to load {}: {}", commonScrPath, e.what());
+            log_error("Unable to load {}: {}", commonScrPath, e.what());
         }
     }
-    std::string customScrPath = config->getOption(CFG_IMPORT_SCRIPTING_CUSTOM_SCRIPT);
-    std::string customFdrPath = config->getOption(CFG_IMPORT_SCRIPTING_CUSTOM_FOLDER);
+    std::string customScrPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_CUSTOM_SCRIPT);
+    std::string customFdrPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_CUSTOM_FOLDER);
     if (!customScrPath.empty()) {
         try {
             _load(customScrPath);
             _execute();
         } catch (const std::runtime_error& e) {
-            log_js("Unable to load {}: {}", customScrPath, e.what());
+            log_error("Unable to load {}: {}", customScrPath, e.what());
         }
     } else if (!customFdrPath.empty()) {
         loadFolder(customFdrPath);
@@ -409,7 +334,11 @@ void Script::_load(const fs::path& scriptPath)
 
     duk_push_string(ctx, scriptPath.c_str());
     if (duk_pcompile_lstring_filename(ctx, 0, scriptText.c_str(), scriptText.length()) != 0) {
-        log_error("Failed to load script: {}", duk_safe_to_string(ctx, -1));
+#if DUK_VERSION > 20399
+        log_error("Failed to load script {}: {}", scriptPath.c_str(), duk_safe_to_stacktrace(ctx, -1));
+#else
+        log_error("Failed to load script {}: {}", scriptPath.c_str(), duk_safe_to_string(ctx, -1));
+#endif
         throw_std_runtime_error("Scripting: failed to compile {}", scriptPath.c_str());
     }
 }
@@ -418,6 +347,7 @@ void Script::load(const fs::path& scriptPath)
 {
     ScriptingRuntime::AutoLock lock(runtime->getMutex());
     duk_push_thread_stash(ctx, ctx);
+    log_debug("Loading file {}", scriptPath.c_str());
     _load(scriptPath);
     duk_put_prop_string(ctx, -2, "script");
     duk_pop(ctx);
@@ -427,9 +357,9 @@ void Script::_execute()
 {
     if (duk_pcall(ctx, 0) != DUK_EXEC_SUCCESS) {
 #if DUK_VERSION > 20399
-        log_error("Failed to execute script {}: {}", scriptPath, duk_safe_to_stacktrace(ctx, -1));
+        log_error("Failed to execute script {}: {}", scriptPath.c_str(), duk_safe_to_stacktrace(ctx, -1));
 #else
-        log_error("Failed to execute script {}: {}", scriptPath, duk_safe_to_string(ctx, -1));
+        log_error("Failed to execute script {}: {}", scriptPath.c_str(), duk_safe_to_string(ctx, -1));
 #endif
         throw_std_runtime_error("Script: failed to execute script");
     }
@@ -469,6 +399,7 @@ void Script::loadFolder(const fs::path& scriptFolder)
 #define GRB_CONTAINERTYPE_VIDEO "grb_container_type_video"
 #define OBJECT_SCRIPT_PATH "object_script_path"
 #define OBJECT_AUTOSCAN_ID "object_autoscan_id"
+#define OBJECT_REF_LIST "object_ref_list"
 #define CONT_NAME "cont"
 
 void Script::cleanup()
@@ -478,12 +409,13 @@ void Script::cleanup()
     duk_del_prop_string(ctx, -1, CONT_NAME);
     duk_del_prop_string(ctx, -1, OBJECT_SCRIPT_PATH);
     duk_del_prop_string(ctx, -1, OBJECT_AUTOSCAN_ID);
+    duk_del_prop_string(ctx, -1, OBJECT_REF_LIST);
     duk_del_prop_string(ctx, -1, GRB_CONTAINERTYPE_AUDIO);
     duk_del_prop_string(ctx, -1, GRB_CONTAINERTYPE_VIDEO);
     duk_del_prop_string(ctx, -1, GRB_CONTAINERTYPE_IMAGE);
 }
 
-void Script::execute(const std::shared_ptr<CdsObject>& obj, const std::string& rootPath)
+std::vector<int> Script::execute(const std::shared_ptr<CdsObject>& obj, const std::string& rootPath)
 {
     cdsObject2dukObject(obj);
     duk_put_global_string(ctx, objectName.c_str());
@@ -495,6 +427,9 @@ void Script::execute(const std::shared_ptr<CdsObject>& obj, const std::string& r
     duk_push_string(ctx, rootPath.c_str());
     duk_put_global_string(ctx, OBJECT_SCRIPT_PATH);
 
+    duk_push_array(ctx);
+    duk_put_global_string(ctx, OBJECT_REF_LIST);
+
     auto autoScan = content->getAutoscanDirectory(rootPath);
     if (autoScan && !rootPath.empty()) {
         duk_push_sprintf(ctx, "%d", autoScan->getScanID());
@@ -523,11 +458,13 @@ void Script::execute(const std::shared_ptr<CdsObject>& obj, const std::string& r
     duk_remove(ctx, -2);
 
     _execute();
+    std::vector<int> result = ScriptGlobalProperty(ctx, OBJECT_REF_LIST).getIntArrayValue();
     cleanup();
     duk_pop(ctx);
+    return result;
 }
 
-void Script::call(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const std::string& functionName, const fs::path& rootPath, const std::string& containerType)
+std::vector<int> Script::call(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const std::string& functionName, const fs::path& rootPath, const std::string& containerType)
 {
     // write global object used in callback functionss
     cdsObject2dukObject(obj);
@@ -572,30 +509,33 @@ void Script::call(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<C
     duk_push_sprintf(ctx, "%s", containerType.c_str());
     narg++;
 
-    if (duk_pcall(ctx, (duk_idx_t)narg) != DUK_EXEC_SUCCESS) {
+    if (duk_pcall(ctx, static_cast<duk_idx_t>(narg)) != DUK_EXEC_SUCCESS) {
         // Note: The invoked function will be blamed for execution errors, not the actual offending line of code
         // https://github.com/svaarala/duktape/blob/master/doc/error-objects.rst
 #if DUK_VERSION > 20399
-        log_error("javascript runtime error: {}() - {}\n", functionName, duk_safe_to_stacktrace(ctx, -1));
+        log_error("javascript {} runtime error: {}() - {}\n", contextName, functionName, duk_safe_to_stacktrace(ctx, -1));
 #else
-        log_error("javascript runtime error: {}() - {}\n", functionName, duk_safe_to_string(ctx, -1));
+        log_error("javascript {} runtime error: {}() - {}\n", contextName, functionName, duk_safe_to_string(ctx, -1));
 #endif
         duk_pop(ctx);
         throw_std_runtime_error("javascript runtime error");
     }
+    std::vector<int> result = ScriptResultProperty(ctx).getIntArrayValue();
+
     duk_pop(ctx);
+    return result;
 }
 
 void Script::setMetaData(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsItem>& item, const std::string& sym, const std::string& val) const
 {
-    if (sym == MetadataHandler::getMetaFieldName(M_TRACKNUMBER) && item) {
+    if (sym == MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER) && item) {
         int j = stoiString(val, 0);
         if (j > 0) {
             item->addMetaData(sym, val);
             item->setTrackNumber(j);
         } else
             item->setTrackNumber(0);
-    } else if (sym == MetadataHandler::getMetaFieldName(M_PARTNUMBER) && item) {
+    } else if (sym == MetaEnumMapper::getMetaFieldName(MetadataFields::M_PARTNUMBER) && item) {
         int j = stoiString(val, 0);
         if (j > 0) {
             item->addMetaData(sym, val);
@@ -609,7 +549,7 @@ void Script::setMetaData(const std::shared_ptr<CdsObject>& obj, const std::share
 
 std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>& pcd)
 {
-    int objType = getIntProperty("objectType", -1);
+    int objType = ScriptNamedProperty(ctx, "objectType").getIntValue(-1);
     if (objType == -1) {
         log_error("missing objectType property");
         return nullptr;
@@ -620,27 +560,26 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
     // CdsObject
     obj->setVirtual(true); // JS creates only virtual objects
 
-    int id = getIntProperty("id", INVALID_OBJECT_ID);
-    if (id != INVALID_OBJECT_ID)
+    int id = ScriptNamedProperty(ctx, "id").getIntValue(INVALID_OBJECT_ID);
+    if (id != INVALID_OBJECT_ID) {
         obj->setID(id);
-    id = getIntProperty("refID", INVALID_OBJECT_ID);
-    if (id != INVALID_OBJECT_ID)
+    }
+    id = ScriptNamedProperty(ctx, "refID").getIntValue(INVALID_OBJECT_ID);
+    if (id != INVALID_OBJECT_ID) {
         obj->setRefID(id);
-    id = getIntProperty("parentID", INVALID_OBJECT_ID);
+    }
+    id = ScriptNamedProperty(ctx, "parentID").getIntValue(INVALID_OBJECT_ID);
     if (id != INVALID_OBJECT_ID) {
         obj->setParentID(id);
     }
 
-    duk_get_prop_string(ctx, -1, "parent");
-    if (duk_is_object(ctx, -1)) {
-        duk_to_object(ctx, -1);
+    ScriptNamedProperty(ctx, "parent").getObject([&]() {
         auto parent = dukObject2cdsObject(nullptr);
         if (parent) {
             obj->setParent(parent);
             log_debug("dukObject2cdsObject: Parent {}", parent->getClass());
         }
-    }
-    duk_pop(ctx);
+    });
 
     // stuff that has not been exported to js
     if (pcd) {
@@ -648,23 +587,21 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
         obj->setResources(pcd->getResources());
         obj->setAuxData(pcd->getAuxData());
     } else {
-        int flags = getIntProperty("flags", -1);
+        int flags = ScriptNamedProperty(ctx, "flags").getIntValue(-1);
         if (flags >= 0)
             obj->setFlags(flags);
 
-        duk_get_prop_string(ctx, -1, "res");
-        if (!duk_is_null_or_undefined(ctx, -1) && duk_is_object(ctx, -1)) {
-            duk_to_object(ctx, -1);
-            auto keys = getPropertyNames();
+        ScriptNamedProperty(ctx, "res").getObject([&]() {
+            auto keys = ScriptProperty(ctx).getPropertyNames();
 
             int resCount = 0;
             for (auto&& sym : keys) {
                 if (sym.find("handlerType") != std::string::npos) {
-                    int ht = getIntProperty(sym, -1);
+                    int ht = ScriptNamedProperty(ctx, sym).getIntValue(-1);
                     auto purpSym = fmt::format("{}:purpose", resCount);
-                    int purpose = getIntProperty(purpSym, -1);
+                    int purpose = ScriptNamedProperty(ctx, purpSym).getIntValue(-1);
                     if (ht >= 0 && purpose >= 0) {
-                        auto newRes = std::make_shared<CdsResource>(MetadataHandler::remapContentHandler(ht), CdsResource::remapPurpose(purpose));
+                        auto newRes = std::make_shared<CdsResource>(EnumMapper::remapContentHandler(ht), EnumMapper::remapPurpose(purpose));
                         obj->addResource(newRes);
                         newRes->setResId(resCount);
                     }
@@ -675,7 +612,7 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
                 resCount = res->getResId();
                 // only attributes enumerated in res_names are allowed
                 for (auto&& [key, upnp] : res_names) {
-                    auto val = getProperty(resCount == 0 ? CdsResource::getAttributeName(key) : fmt::format("{}-{}", resCount, CdsResource::getAttributeName(key)));
+                    auto val = ScriptNamedProperty(ctx, resCount == 0 ? EnumMapper::getAttributeName(key) : fmt::format("{}-{}", resCount, EnumMapper::getAttributeName(key))).getStringValue();
                     if (!val.empty()) {
                         val = sc->convert(val);
                         res->addAttribute(key, val);
@@ -685,7 +622,7 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
                 for (auto&& sym : keys) {
                     if (sym.find(head) != std::string::npos) {
                         auto key = sym.substr(head.size());
-                        auto val = getProperty(sym);
+                        auto val = ScriptNamedProperty(ctx, sym).getStringValue();
                         res->addParameter(key, val);
                     }
                 }
@@ -693,27 +630,23 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
                 for (auto&& sym : keys) {
                     if (sym.find(head) != std::string::npos) {
                         auto key = sym.substr(head.size());
-                        auto val = getProperty(sym);
+                        auto val = ScriptNamedProperty(ctx, sym).getStringValue();
                         res->addOption(key, val);
                     }
                 }
             }
-        }
-        duk_pop(ctx); // res
+        });
 
-        duk_get_prop_string(ctx, -1, "aux");
-        if (!duk_is_null_or_undefined(ctx, -1) && duk_is_object(ctx, -1)) {
-            duk_to_object(ctx, -1);
-            auto keys = getPropertyNames();
+        ScriptNamedProperty(ctx, "aux").getObject([&]() {
+            auto keys = ScriptProperty(ctx).getPropertyNames();
             for (auto&& sym : keys) {
-                auto val = getProperty(sym);
+                auto val = ScriptNamedProperty(ctx, sym).getStringValue();
                 if (!val.empty()) {
                     val = sc->convert(val);
                     obj->setAuxData(sym, val);
                 }
             }
-        }
-        duk_pop(ctx); // aux
+        });
     }
     return obj;
 }
@@ -722,13 +655,16 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
 {
     auto obj = createObject(pcd);
 
-    int mtime = getIntProperty("mtime", 0);
+    if (!obj)
+        return nullptr;
+
+    int mtime = ScriptNamedProperty(ctx, "mtime").getIntValue(0);
     if (mtime > 0) {
         obj->setMTime(std::chrono::seconds(mtime));
     }
 
     {
-        auto val = getProperty("title");
+        auto val = ScriptNamedProperty(ctx, "title").getStringValue();
         if (!val.empty()) {
             val = sc->convert(val);
             obj->setTitle(val);
@@ -738,7 +674,7 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
     }
 
     {
-        auto val = getProperty("upnpclass");
+        auto val = ScriptNamedProperty(ctx, "upnpclass").getStringValue();
         if (!val.empty()) {
             val = sc->convert(val);
             obj->setClass(val);
@@ -747,37 +683,34 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         }
     }
 
-    int restricted = getBoolProperty("restricted");
+    int restricted = ScriptNamedProperty(ctx, "restricted").getBoolValue();
     if (restricted >= 0)
         obj->setRestricted(restricted);
 
-    duk_get_prop_string(ctx, -1, "metaData");
-    if (!duk_is_null_or_undefined(ctx, -1) && duk_is_object(ctx, -1)) {
-        duk_to_object(ctx, -1);
+    ScriptNamedProperty(ctx, "metaData").getObject([&]() {
         auto item = std::static_pointer_cast<CdsItem>(obj);
-        auto keys = getPropertyNames();
+        auto keys = ScriptProperty(ctx).getPropertyNames();
         for (auto&& sym : keys) {
-            auto arrayVal = getArrayProperty(sym);
+            auto arrayVal = ScriptNamedProperty(ctx, sym).getStringArrayValue();
             for (auto&& val : arrayVal) {
                 if (!val.empty()) {
                     setMetaData(obj, item, sym, val);
                 }
             }
         }
-    }
-    duk_pop(ctx); // metaData
+    });
 
-    fs::path location = getProperty("location");
+    fs::path location = !hasCaseSensitiveNames && obj->isContainer() ? toLower(ScriptNamedProperty(ctx, "location").getStringValue()) : ScriptNamedProperty(ctx, "location").getStringValue();
     if (!location.empty()) {
         // location must not be touched by character conversion!
         obj->setLocation(location);
     }
 
-    auto description = getProperty("description");
+    auto description = ScriptNamedProperty(ctx, "description").getStringValue();
     if (!description.empty()) {
         description = sc->convert(description);
-        obj->removeMetaData(M_DESCRIPTION);
-        obj->addMetaData(M_DESCRIPTION, description);
+        obj->removeMetaData(MetadataFields::M_DESCRIPTION);
+        obj->addMetaData(MetadataFields::M_DESCRIPTION, description);
     }
 
     // CdsItem
@@ -788,7 +721,7 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         if (pcd)
             pcdItem = std::static_pointer_cast<CdsItem>(pcd);
 
-        auto mimetype = getProperty("mimetype");
+        auto mimetype = ScriptNamedProperty(ctx, "mimetype").getStringValue();
         if (!mimetype.empty()) {
             mimetype = sc->convert(mimetype);
             item->setMimeType(mimetype);
@@ -796,14 +729,21 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
             item->setMimeType(pcdItem->getMimeType());
         }
 
-        auto serviceID = getProperty("serviceID");
+        auto serviceID = ScriptNamedProperty(ctx, "serviceID").getStringValue();
         if (!serviceID.empty()) {
             serviceID = sc->convert(serviceID);
             item->setServiceID(serviceID);
         }
 
-        if (description.empty() && pcd && obj->getMetaData(M_DESCRIPTION).empty() && !obj->getMetaData(M_DESCRIPTION).empty()) {
-            obj->addMetaData(M_DESCRIPTION, obj->getMetaData(M_DESCRIPTION));
+        {
+            auto val = ScriptNamedProperty(ctx, "description").getStringValue();
+            if (!val.empty()) {
+                val = sc->convert(val);
+                item->removeMetaData(MetadataFields::M_DESCRIPTION);
+                item->addMetaData(MetadataFields::M_DESCRIPTION, val);
+            } else if (pcdItem && item->getMetaData(MetadataFields::M_DESCRIPTION).empty() && !pcdItem->getMetaData(MetadataFields::M_DESCRIPTION).empty()) {
+                item->addMetaData(MetadataFields::M_DESCRIPTION, pcdItem->getMetaData(MetadataFields::M_DESCRIPTION));
+            }
         }
 
         if (location.empty() && pcd) {
@@ -815,7 +755,7 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
 
             obj->setRestricted(true);
 
-            auto protocol = getProperty("protocol");
+            auto protocol = ScriptNamedProperty(ctx, "protocol").getStringValue();
             if (!protocol.empty()) {
                 protocol = sc->convert(protocol);
                 protocolInfo = renderProtocolInfo(item->getMimeType(), protocol);
@@ -825,15 +765,15 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
 
             std::shared_ptr<CdsResource> resource;
             if (item->getResourceCount() == 0) {
-                resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, CdsResource::Purpose::Content);
+                resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
                 item->addResource(resource);
             } else {
                 resource = item->getResource(ContentHandler::DEFAULT);
             }
-            resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, protocolInfo);
-            int size = getIntProperty("size", -1);
+            resource->addAttribute(ResourceAttribute::PROTOCOLINFO, protocolInfo);
+            int size = ScriptNamedProperty(ctx, "size").getIntValue(-1);
             if (size > -1) {
-                resource->addAttribute(CdsResource::Attribute::SIZE, fmt::to_string(size));
+                resource->addAttribute(ResourceAttribute::SIZE, fmt::to_string(size));
             }
         }
 
@@ -843,11 +783,12 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
     // CdsDirectory
     if (obj->isContainer()) {
         auto cont = std::static_pointer_cast<CdsContainer>(obj);
-        auto id = getIntProperty("updateID", -1);
+        auto id = ScriptNamedProperty(ctx, "updateID").getIntValue(-1);
         if (id >= CDS_ID_ROOT)
             cont->setUpdateID(id);
 
-        int searchable = getBoolProperty("searchable");
+        int searchable = ScriptNamedProperty(ctx, "searchable").getBoolValue();
+        log_debug("{} searchable {}", cont->getTitle(), searchable);
         if (searchable >= 0)
             cont->setSearchable(searchable);
 
@@ -887,9 +828,9 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
     if (!val.empty())
         setProperty("location", val);
 
-    setIntProperty("mtime", int(obj->getMTime().count()));
-    setIntProperty("utime", int(obj->getUTime().count()));
-    setIntProperty("sizeOnDisk", int(obj->getSizeOnDisk()));
+    setIntProperty("mtime", static_cast<int>(obj->getMTime().count()));
+    setIntProperty("utime", static_cast<int>(obj->getUTime().count()));
+    setIntProperty("sizeOnDisk", static_cast<int>(obj->getSizeOnDisk()));
     setIntProperty("flags", obj->getFlags());
 
     // TODO: boolean type
@@ -904,7 +845,7 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
 #ifdef ONLINE_SERVICES
     if (obj->getFlag(OBJECT_FLAG_ONLINE_SERVICE)) {
         auto service = OnlineServiceType(std::stoi(obj->getAuxData(ONLINE_SERVICE_AUX_ID)));
-        setIntProperty("onlineservice", int(service));
+        setIntProperty("onlineservice", static_cast<int>(service));
     } else
 #endif
         setIntProperty("onlineservice", 0);
@@ -921,10 +862,10 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
             setProperty(key, fmt::format("{}", fmt::join(attr, entrySeparator)));
         }
         if (item && item->getTrackNumber() > 0)
-            setProperty(MetadataHandler::getMetaFieldName(M_TRACKNUMBER), fmt::to_string(item->getTrackNumber()));
+            setProperty(MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER), fmt::to_string(item->getTrackNumber()));
 
         if (item && item->getPartNumber() > 0)
-            setProperty(MetadataHandler::getMetaFieldName(M_PARTNUMBER), fmt::to_string(item->getPartNumber()));
+            setProperty(MetaEnumMapper::getMetaFieldName(MetadataFields::M_PARTNUMBER), fmt::to_string(item->getPartNumber()));
 
         duk_put_prop_string(ctx, -2, "meta");
         // stack: js
@@ -934,10 +875,10 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
         duk_push_object(ctx);
         // stack: js meta_js
         if (item && item->getTrackNumber() > 0) {
-            metaGroups[MetadataHandler::getMetaFieldName(M_TRACKNUMBER)] = { fmt::to_string(item->getTrackNumber()) };
+            metaGroups[MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER)] = { fmt::to_string(item->getTrackNumber()) };
         }
         if (item && item->getPartNumber() > 0) {
-            metaGroups[MetadataHandler::getMetaFieldName(M_PARTNUMBER)] = { fmt::to_string(item->getPartNumber()) };
+            metaGroups[MetaEnumMapper::getMetaFieldName(MetadataFields::M_PARTNUMBER)] = { fmt::to_string(item->getPartNumber()) };
         }
         for (auto&& [key, array] : metaGroups) {
             auto dukArray = duk_push_array(ctx);
@@ -985,7 +926,7 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
                 setProperty(fmt::format("{}:purpose", resCount), fmt::to_string(to_underlying(res->getPurpose())));
                 auto attributes = res->getAttributes();
                 for (auto&& [key, attr] : attributes) {
-                    setProperty(resCount == 0 ? CdsResource::getAttributeName(key) : fmt::format("{}-{}", resCount, CdsResource::getAttributeName(key)), attr);
+                    setProperty(resCount == 0 ? EnumMapper::getAttributeName(key) : fmt::format("{}-{}", resCount, EnumMapper::getAttributeName(key)), attr);
                 }
                 auto parameters = res->getParameters();
                 for (auto&& [key, param] : parameters) {
@@ -1005,8 +946,8 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
 
     // CdsItem
     if (obj->isItem() && item) {
-        setIntProperty("trackNumber", int(item->getTrackNumber()));
-        setIntProperty("partNumber", int(item->getPartNumber()));
+        setIntProperty("trackNumber", item->getTrackNumber());
+        setIntProperty("partNumber", item->getPartNumber());
         val = item->getMimeType();
         if (!val.empty())
             setProperty("mimetype", val);
