@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -33,12 +33,13 @@
 
 #ifdef HAVE_MYSQL
 
-#ifndef __mysql_database_H__
-#define __mysql_database_H__
+#ifndef __MYSQL_DATABASE_H__
+#define __MYSQL_DATABASE_H__
 
-#include "common.h"
 #include "database/sql_database.h"
-#include <mutex>
+
+#include "config/config_val.h"
+
 #include <mysql.h>
 #include <vector>
 
