@@ -3,7 +3,7 @@
 
   search_handler.cc - this file is part of Gerbera.
 
-  Copyright (C) 2018-2023 Gerbera Contributors
+  Copyright (C) 2018-2024 Gerbera Contributors
 
   Gerbera is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
@@ -22,16 +22,19 @@
 
 /// \file search_handler.cc
 
+#define GRB_LOG_FAC GrbLogFacility::sqldatabase
 #include "search_handler.h" // API
 
+#include "config/config_val.h"
+#include "database/sql_database.h"
+#include "exceptions.h"
+#include "metadata/metadata_enums.h"
+#include "util/grb_time.h"
+#include "util/tools.h"
+
 #include <algorithm>
 #include <stack>
 
-#include "database/sql_database.h"
-#include "metadata/metadata_handler.h"
-#include "util/grb_time.h"
-#include "util/tools.h"
-
 static const std::unordered_map<std::string_view, TokenType> tokenTypes {
     { "(", TokenType::LPAREN },
     { ")", TokenType::RPAREN },
@@ -540,10 +543,11 @@ std::string ASTOrOperator::emit() const
     return sqlEmitter.emit(this, lhs->emit(), rhs->emit());
 }
 
-DefaultSQLEmitter::DefaultSQLEmitter(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> metaMapper, std::shared_ptr<ColumnMapper> resMapper)
+DefaultSQLEmitter::DefaultSQLEmitter(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> metaMapper, std::shared_ptr<ColumnMapper> resMapper, std::shared_ptr<ColumnMapper> plyMapper)
     : colMapper(std::move(colMapper))
     , metaMapper(std::move(metaMapper))
     , resMapper(std::move(resMapper))
+    , plyMapper(std::move(plyMapper))
 {
 }
 
@@ -587,13 +591,16 @@ std::pair<std::string, std::string> DefaultSQLEmitter::getPropertyStatement(cons
     if (resMapper && resMapper->hasEntry(property)) {
         return { resMapper->mapQuoted(property), resMapper->mapQuotedLower(property) };
     }
+    if (plyMapper && plyMapper->hasEntry(property)) {
+        return { plyMapper->mapQuoted(property), plyMapper->mapQuotedLower(property) };
+    }
     if (metaMapper) {
         return {
             fmt::format("{0}='{2}' AND {1}", metaMapper->mapQuoted(META_NAME), metaMapper->mapQuoted(META_VALUE), property),
             fmt::format("{0}='{2}' AND {1}", metaMapper->mapQuoted(META_NAME), metaMapper->mapQuotedLower(META_VALUE), property)
         };
     }
-    log_info("Property {} not yet supported. Search may return no result!", property);
+    log_warning("Property {} not yet supported. Search may return no result!", property);
     return {};
 }
 
@@ -608,6 +615,10 @@ std::string DefaultSQLEmitter::emit(const ASTCompareOperator* node, const std::s
         return fmt::format(logicOperator.at("newer"), "", fmt::format("{} {}", prpUpper, operatr), fmt::format("{} {}", prpLower, operatr), dateVal.count());
     }
 
+    if ((operatr == ">" || operatr == ">=") && stoiString(value) > 0 && property == UPNP_SEARCH_PLAY_COUNT) {
+        return fmt::format("{} {} {}", prpUpper, operatr, stoiString(value));
+    }
+
     if (operatr != "=")
         throw_std_runtime_error("Operator '{}' not yet supported", operatr);
 
@@ -654,8 +665,9 @@ std::string DefaultSQLEmitter::emit(const ASTOrOperator* node, const std::string
     return fmt::format("{} OR {}", lhs, rhs);
 }
 
-SortParser::SortParser(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> metaMapper, std::string sortCriteria)
+SortParser::SortParser(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> plyMapper, std::shared_ptr<ColumnMapper> metaMapper, std::string sortCriteria)
     : colMapper(std::move(colMapper))
+    , plyMapper(std::move(plyMapper))
     , metaMapper(std::move(metaMapper))
     , sortCrit(std::move(sortCriteria))
 {
@@ -678,10 +690,10 @@ std::string SortParser::parse(std::string& addColumns, std::string& addJoin)
         } else {
             log_warning("Unknown sort direction '{}' in '{}'", seg, sortCrit);
         }
-        if (!colMapper->mapQuotedList(sort, seg, (desc ? "DESC" : "ASC"))) {
+        if (!colMapper->mapQuotedList(sort, seg, (desc ? "DESC" : "ASC")) && !plyMapper->mapQuotedList(sort, seg, (desc ? "DESC" : "ASC"))) {
             std::string sortSql;
             for (auto&& metaId : MetadataIterator()) {
-                auto&& metaName = MetadataHandler::getMetaFieldName(metaId);
+                auto&& metaName = MetaEnumMapper::getMetaFieldName(metaId);
                 if (metaName == seg) {
                     sortSql = metaName;
                 }
