@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -34,15 +34,16 @@
 #ifndef __SQL_STORAGE_H__
 #define __SQL_STORAGE_H__
 
+#include "config/config.h"
+#include "config/config_val.h"
+#include "database.h"
+#include "sql_format.h"
+
 #include <array>
 #include <mutex>
 #include <unordered_set>
 #include <utility>
 
-#include "config/config.h"
-#include "database.h"
-#include "sql_format.h"
-
 // forward declarations
 class CdsContainer;
 class CdsResource;
@@ -138,7 +139,7 @@ public:
     std::map<int, int> getChildCounts(const std::vector<int>& contId, bool containers, bool items, bool hideFsRoot) override;
 
     std::size_t getObjects(int parentID, bool withoutContainer, std::unordered_set<int>& ret, bool full) override;
-    std::vector<std::pair<int, std::chrono::seconds>> getRefObjects(int objectId) override;
+    std::vector<int> getRefObjects(int objectId) override;
     std::unordered_set<int> getUnreferencedObjects() override;
 
     std::unique_ptr<ChangedContainers> removeObject(int objectID, const fs::path& path, bool all) override;
@@ -238,7 +239,7 @@ protected:
     using SqlAutoLock = std::scoped_lock<decltype(sqlMutex)>;
     std::map<int, std::shared_ptr<CdsContainer>> dynamicContainers;
 
-    void upgradeDatabase(unsigned int dbVersion, const std::array<unsigned int, DBVERSION>& hashies, config_option_t upgradeOption, std::string_view updateVersionCommand, std::string_view addResourceColumnCmd);
+    void upgradeDatabase(unsigned int dbVersion, const std::array<unsigned int, DBVERSION>& hashies, ConfigVal upgradeOption, std::string_view updateVersionCommand, std::string_view addResourceColumnCmd);
     virtual void _exec(const std::string& query) = 0;
 
 private:
@@ -342,7 +343,7 @@ void SQLDatabase::deleteRow(std::string_view tableName, std::string_view key, co
 class SqlWithTransactions {
 protected:
     explicit SqlWithTransactions(const std::shared_ptr<Config>& config)
-        : use_transaction(config->getBoolOption(CFG_SERVER_STORAGE_USE_TRANSACTIONS))
+        : use_transaction(config->getBoolOption(ConfigVal::SERVER_STORAGE_USE_TRANSACTIONS))
     {
     }
 
