@@ -3,7 +3,7 @@
 
   sql_format.h - this file is part of Gerbera.
 
-  Copyright (C) 2021-2023 Gerbera Contributors
+  Copyright (C) 2021-2024 Gerbera Contributors
 
   Gerbera is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
@@ -26,10 +26,13 @@
 #define __SQL_FORMAT_H__
 
 #include <fmt/format.h>
+#if FMT_VERSION >= 100202
+#include <fmt/ranges.h>
+#endif
 
 struct SQLIdentifier {
-    SQLIdentifier(const std::string& name, char quote_begin, char quote_end)
-        : name(name)
+    SQLIdentifier(std::string name, char quote_begin, char quote_end)
+        : name(std::move(name))
         , quote_begin(quote_begin)
         , quote_end(quote_end)
     {
@@ -49,8 +52,8 @@ struct fmt::formatter<SQLIdentifier> : formatter<std::string> {
 };
 
 struct ColumnUpdate {
-    ColumnUpdate(const SQLIdentifier& column, std::string value)
-        : column(column)
+    ColumnUpdate(SQLIdentifier column, std::string value)
+        : column(std::move(column))
         , value(std::move(value))
     {
     }
