@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,11 +30,13 @@
 */
 
 /// \file sqlite3_database.cc
-#define LOG_FAC log_facility_t::sqlite3
+#define GRB_LOG_FAC GrbLogFacility::sqlite3
 
 #include "sqlite_database.h" // API
 
-#include "config/config_manager.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "exceptions.h"
 #include "sl_task.h"
 
 static constexpr auto sqlite3UpdateVersion = std::string_view(R"(UPDATE "mt_internal_setting" SET "value"='{}' WHERE "key"='db_version' AND "value"='{}')");
@@ -61,8 +63,8 @@ void Sqlite3Database::prepare()
 {
     _exec("PRAGMA locking_mode = EXCLUSIVE");
     _exec("PRAGMA foreign_keys = ON");
-    _exec(fmt::format("PRAGMA journal_mode = {}", config->getOption(CFG_SERVER_STORAGE_SQLITE_JOURNALMODE)));
-    exec(fmt::format("PRAGMA synchronous = {}", config->getIntOption(CFG_SERVER_STORAGE_SQLITE_SYNCHRONOUS)));
+    _exec(fmt::format("PRAGMA journal_mode = {}", config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_JOURNALMODE)));
+    exec(fmt::format("PRAGMA synchronous = {}", config->getIntOption(ConfigVal::SERVER_STORAGE_SQLITE_SYNCHRONOUS)));
 }
 
 void Sqlite3Database::init()
@@ -70,7 +72,7 @@ void Sqlite3Database::init()
     dbInitDone = false;
     SQLDatabase::init();
 
-    fs::path dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
+    fs::path dbFilePath = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_DATABASE_FILE);
     auto dbFile = GrbFile(dbFilePath);
     log_debug("SQLite path: {}", dbFilePath.c_str());
 
@@ -109,7 +111,7 @@ void Sqlite3Database::init()
     } catch (const std::runtime_error&) {
         log_warning("Sqlite3 database seems to be corrupt or doesn't exist yet.");
         // database seems to be corrupt or nonexistent
-        if (config->getBoolOption(CFG_SERVER_STORAGE_SQLITE_RESTORE) && sqliteStatus == SQLITE_OK) {
+        if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_RESTORE) && sqliteStatus == SQLITE_OK) {
             // try to restore database
 
             // checking for backup file
@@ -153,15 +155,15 @@ void Sqlite3Database::init()
     }
 
     try {
-        upgradeDatabase(std::stoul(dbVersion), hashies, CFG_SERVER_STORAGE_SQLITE_UPGRADE_FILE, sqlite3UpdateVersion, sqlite3AddResourceAttr);
-        if (config->getBoolOption(CFG_SERVER_STORAGE_SQLITE_BACKUP_ENABLED) && timer) {
+        upgradeDatabase(std::stoul(dbVersion), hashies, ConfigVal::SERVER_STORAGE_SQLITE_UPGRADE_FILE, sqlite3UpdateVersion, sqlite3AddResourceAttr);
+        if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_ENABLED) && timer) {
             // do a backup now
             auto btask = std::make_shared<SLBackupTask>(config, false);
             this->addTask(btask);
             btask->waitForTask();
 
             // add timer for backups
-            auto backupInterval = std::chrono::seconds(config->getIntOption(CFG_SERVER_STORAGE_SQLITE_BACKUP_INTERVAL));
+            auto backupInterval = std::chrono::seconds(config->getIntOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_INTERVAL));
             timer->addTimerSubscriber(this, backupInterval, nullptr);
             hasBackupTimer = true;
         }
@@ -245,20 +247,15 @@ std::shared_ptr<SQLResult> Sqlite3Database::select(const std::string& query)
         stask->waitForTask();
         return stask->getResult();
     } catch (const std::runtime_error& e) {
-        if (!shutdownFlag) {
-            if (dbInitDone) {
-                log_error("Prematurely shutting down.\n{}", e.what());
-                rollback("");
-                shutdown();
-            }
+        if (!dbInitDone)
             throw_std_runtime_error(e.what());
-        } else {
-            if (dbInitDone) {
-                log_error("Already shutting down.\n{}", e.what());
-                return {};
-            } else
-                throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
+            rollback("");
+            shutdown();
         }
+        log_error("Already shutting down.\n{}", e.what());
+        return {};
     }
 }
 
@@ -269,28 +266,26 @@ void Sqlite3Database::del(std::string_view tableName, const std::string& clause,
         : fmt::format("DELETE FROM {} WHERE {}", identifier(std::string(tableName)), clause);
     try {
         log_debug("Adding delete to Queue: {}", query);
-        maxDeleteCount = maxDeleteCount > ids.size() * 10 ? maxDeleteCount : ids.size() * 10;
-        for (auto&& id : ids) {
-            deletedEntries.push_back(fmt::format("{}_{}", tableName, id));
+        {
+            DelAutoLock del_lock(del_mutex);
+            maxDeleteCount = maxDeleteCount > ids.size() * 10 ? maxDeleteCount : ids.size() * 10;
+            for (auto&& id : ids) {
+                deletedEntries.push_back(fmt::format("{}_{}", tableName, id));
+            }
+            lastDelete = currentTime();
         }
-        lastDelete = currentTime();
         auto etask = std::make_shared<SLExecTask>(query, false);
         addTask(etask);
         etask->waitForTask();
     } catch (const std::runtime_error& e) {
-        if (!shutdownFlag) {
-            if (dbInitDone) {
-                log_error("Prematurely shutting down.\n{}", e.what());
-                rollback("");
-                shutdown();
-            }
+        if (!dbInitDone)
             throw_std_runtime_error(e.what());
-        } else {
-            if (dbInitDone)
-                log_error("Already shutting down.\n{}", e.what());
-            else
-                throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
+            rollback("");
+            shutdown();
         }
+        log_error("Already shutting down.\n{}", e.what());
     }
 }
 
@@ -303,19 +298,14 @@ void Sqlite3Database::exec(std::string_view tableName, const std::string& query,
         addTask(etask);
         etask->waitForTask();
     } catch (const std::runtime_error& e) {
-        if (!shutdownFlag) {
-            if (dbInitDone) {
-                log_error("Prematurely shutting down.\n{}", e.what());
-                rollback("");
-                shutdown();
-            }
+        if (!dbInitDone)
             throw_std_runtime_error(e.what());
-        } else {
-            if (dbInitDone)
-                log_error("Already shutting down.\n{}", e.what());
-            else
-                throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
+            rollback("");
+            shutdown();
         }
+        log_error("Already shutting down.\n{}", e.what());
     }
 }
 
@@ -328,20 +318,15 @@ int Sqlite3Database::exec(const std::string& query, bool getLastInsertId)
         etask->waitForTask();
         return getLastInsertId ? etask->getLastInsertId() : -1;
     } catch (const std::runtime_error& e) {
-        if (!shutdownFlag) {
-            if (dbInitDone) {
-                log_error("Prematurely shutting down.\n{}", e.what());
-                rollback("");
-                shutdown();
-            }
+        if (!dbInitDone)
             throw_std_runtime_error(e.what());
-        } else {
-            if (dbInitDone) {
-                log_error("Already shutting down.\n{}", e.what());
-                return -1;
-            } else
-                throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
+            rollback("");
+            shutdown();
         }
+        log_error("Already shutting down.\n{}", e.what());
+        return -1;
     }
 }
 
@@ -363,7 +348,7 @@ void Sqlite3Database::threadProc()
     try {
         sqlite3* db;
 
-        std::string dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
+        std::string dbFilePath = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_DATABASE_FILE);
 
         int res = sqlite3_open(dbFilePath.c_str(), &db);
         if (res != SQLITE_OK) {
@@ -406,11 +391,13 @@ void Sqlite3Database::threadProc()
             /* if nothing to do, sleep until awakened */
             auto now = currentTime();
             if (now.count() - lastDelete.count() > DELETE_CACHE_MAX_TIME) { // drop cache if last delete was more than 60 secs ago
+                DelAutoLock del_lock(del_mutex);
                 maxDeleteCount = DELETE_CACHE_MAX_SIZE;
                 deletedEntries.clear();
-            } else if (deletedEntries.size() > maxDeleteCount) // dynamically increase if large DELETESs happen
+            } else if (deletedEntries.size() > maxDeleteCount) { // dynamically increase if large DELETESs happen
+                DelAutoLock del_lock(del_mutex);
                 deletedEntries.erase(deletedEntries.begin(), deletedEntries.begin() + maxDeleteCount * DELETE_CACHE_RED_SIZE);
-
+            }
             threadRunner->wait(lock);
         }
         log_debug("Exiting");
@@ -431,8 +418,8 @@ void Sqlite3Database::threadProc()
             }
             db = nullptr;
         }
-    } catch (const std::runtime_error&) {
-        log_error("Aborting thread");
+    } catch (const std::runtime_error& e) {
+        log_error("Aborting thread {}", e.what());
     }
 }
 
