@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -35,6 +35,7 @@
 #ifndef __SQLITE3_STORAGE_H__
 #define __SQLITE3_STORAGE_H__
 
+#include <mutex>
 #include <queue>
 #include <sqlite3.h>
 
@@ -90,7 +91,10 @@ private:
     /// \brief the tasks to be done by the sqlite3 thread
     std::queue<std::shared_ptr<SLTask>> taskQueue;
     bool taskQueueOpen {};
-    std::vector<std::string> deletedEntries {};
+
+    mutable std::mutex del_mutex;
+    using DelAutoLock = std::scoped_lock<std::mutex>;
+    mutable std::vector<std::string> deletedEntries {};
     size_t maxDeleteCount { DELETE_CACHE_MAX_SIZE };
     std::chrono::seconds lastDelete;
 
