@@ -4,7 +4,7 @@
 
     device_description_handler.h - this file is part of Gerbera.
 
-    Copyright (C) 2020-2023 Gerbera Contributors
+    Copyright (C) 2020-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -25,18 +25,28 @@
 #define GERBERA_DEVICE_DESCRIPTION_HANDLER_H
 
 #include "request_handler.h"
-#include "upnp_xml.h"
+
 #include <memory>
+#include <netinet/in.h>
+#include <string>
+
+class UpnpXMLBuilder;
 
 class DeviceDescriptionHandler : public RequestHandler {
 public:
-    explicit DeviceDescriptionHandler(const std::shared_ptr<ContentManager>& content, const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder);
+    explicit DeviceDescriptionHandler(const std::shared_ptr<Content>& content, const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder, std::string ip, in_port_t port);
 
-    void getInfo(const char* filename, UpnpFileInfo* info) override;
-    std::unique_ptr<IOHandler> open(const char* filename, enum UpnpOpenFileMode mode) override;
+    const struct ClientInfo* getInfo(const char* filename, UpnpFileInfo* info) override;
+    std::unique_ptr<IOHandler> open(const char* filename, const std::shared_ptr<Quirks>& quirks, enum UpnpOpenFileMode mode) override;
+    std::string renderDeviceDescription(std::string ip, in_port_t port, const std::shared_ptr<Quirks>& quirks) const;
+
+private:
+    std::string getPresentationUrl(std::string ip, in_port_t port) const;
+
+    std::string ip;
+    in_port_t port;
+    bool useDynamicDescription { false };
 
-protected:
-    std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
     std::string deviceDescription;
 };
 
