@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,37 +30,40 @@
 */
 
 /// \file file_request_handler.cc
-#define LOG_FAC log_facility_t::requests
+#define GRB_LOG_FAC GrbLogFacility::requests
 
 #include "file_request_handler.h" // API
 
+#include "cds/cds_item.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "config/result/transcoding.h"
+#include "content/content.h"
+#include "database/database.h"
+#include "exceptions.h"
+#include "iohandler/file_io_handler.h"
+#include "metadata/metadata_handler.h"
+#include "metadata/metadata_service.h"
+#include "transcoding/transcode_dispatcher.h"
+#include "upnp/headers.h"
+#include "upnp/quirks.h"
+#include "upnp/upnp_common.h"
+#include "upnp/xml_builder.h"
+#include "util/grb_net.h"
+#include "util/tools.h"
+#include "util/url_utils.h"
+#include "web/session_manager.h"
+
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "cds/cds_item.h"
-#include "config/config_manager.h"
-#include "content/content_manager.h"
-#include "database/database.h"
-#include "iohandler/file_io_handler.h"
-#include "metadata/metadata_handler.h"
-#include "transcoding/transcode_dispatcher.h"
-#include "transcoding/transcoding.h"
-#include "upnp_common.h"
-#include "upnp_xml.h"
-#include "util/grb_net.h"
-#include "util/tools.h"
-#include "util/upnp_headers.h"
-#include "util/upnp_quirks.h"
-#include "util/url_utils.h"
-#include "web/session_manager.h"
-
-FileRequestHandler::FileRequestHandler(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder)
-    : RequestHandler(content)
-    , xmlBuilder(std::move(xmlBuilder))
+FileRequestHandler::FileRequestHandler(const std::shared_ptr<Content>& content, const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder, std::shared_ptr<MetadataService> metadataService)
+    : RequestHandler(content, xmlBuilder)
+    , metadataService(std::move(metadataService))
 {
 }
 
-void FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
+const struct ClientInfo* FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 {
     log_debug("Start: {}", filename);
 
@@ -80,14 +83,14 @@ void FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
     if (resourceId >= obj->getResourceCount() && trProfile.empty()) {
         throw_std_runtime_error("Requested resource {} does not exist", resourceId);
     }
-    auto resource = trProfile.empty() ? obj->getResource(resourceId) : std::make_shared<CdsResource>(ContentHandler::TRANSCODE, CdsResource::Purpose::Transcode);
+    auto resource = trProfile.empty() ? obj->getResource(resourceId) : std::make_shared<CdsResource>(ContentHandler::TRANSCODE, ResourcePurpose::Transcode);
 
     fs::path path = obj->getLocation();
 
     // Check if the resource is actually another external file, and if it exists
     bool isResourceFile = false;
 
-    auto resPath = resource->getAttribute(CdsResource::Attribute::RESOURCE_FILE);
+    auto resPath = resource->getAttribute(ResourceAttribute::RESOURCE_FILE);
     isResourceFile = !resPath.empty() && resPath != obj->getLocation();
     if (isResourceFile) {
         log_debug("Resource is file: {}", path.string());
@@ -117,7 +120,7 @@ void FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
     if (resource->getHandlerType() != ContentHandler::DEFAULT && resource->getHandlerType() != ContentHandler::TRANSCODE) {
         auto metadataHandler = getResourceMetadataHandler(obj, resource);
 
-        std::string protocolInfo = resource->getAttribute(CdsResource::Attribute::PROTOCOLINFO);
+        std::string protocolInfo = resource->getAttribute(ResourceAttribute::PROTOCOLINFO);
         if (!protocolInfo.empty()) {
             mimeType = getMTFromProtocolInfo(protocolInfo);
         }
@@ -138,18 +141,18 @@ void FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 
     } else if (!isResourceFile && !trProfile.empty()) {
 
-        auto transcodingProfile = config->getTranscodingProfileListOption(CFG_TRANSCODING_PROFILE_LIST)->getByName(trProfile);
+        auto transcodingProfile = config->getTranscodingProfileListOption(ConfigVal::TRANSCODING_PROFILE_LIST)->getByName(trProfile);
         if (!transcodingProfile)
             throw_std_runtime_error("Transcoding of file {} but no profile matching the name {} found", path.c_str(), trProfile);
 
         mimeType = transcodingProfile->getTargetMimeType();
 
         // TODO: this WAV specific logic should be more generic
-        auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+        auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
         if (getValueOrDefault(mappings, mimeType) == CONTENT_TYPE_PCM) {
             auto res = obj->getResource(ContentHandler::DEFAULT);
-            std::string freq = res->getAttribute(CdsResource::Attribute::SAMPLEFREQUENCY);
-            std::string nrch = res->getAttribute(CdsResource::Attribute::NRAUDIOCHANNELS);
+            std::string freq = res->getAttribute(ResourceAttribute::SAMPLEFREQUENCY);
+            std::string nrch = res->getAttribute(ResourceAttribute::NRAUDIOCHANNELS);
             if (!freq.empty())
                 mimeType += fmt::format(";rate={}", freq);
             if (!nrch.empty())
@@ -167,7 +170,7 @@ void FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
         resource = item->getResource(resourceId);
 
         // Generate DNLA Headers
-        auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+        auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
         std::string dlnaContentHeader = xmlBuilder->getDLNAContentHeader(getValueOrDefault(mappings, mimeType), resource);
         if (!dlnaContentHeader.empty()) {
             headers.addHeader(UPNP_DLNA_CONTENT_FEATURES_HEADER, dlnaContentHeader);
@@ -188,20 +191,23 @@ void FileRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
     UpnpFileInfo_set_ContentType(info, mimeType.c_str());
 #endif
 
+    if (quirks)
+        quirks->updateHeaders(headers);
     headers.writeHeaders(info);
 
     // log_debug("getInfo: Requested {}, ObjectID: {}, Location: {}, MimeType: {}",
     //      filename, object_id.c_str(), path.c_str(), info->content_type);
 
     log_debug("end: {}", filename);
+    return quirks ? quirks->getInfo() : nullptr;
 }
 
-std::unique_ptr<MetadataHandler> FileRequestHandler::getResourceMetadataHandler(std::shared_ptr<CdsObject>& obj, std::shared_ptr<CdsResource>& resource) const
+std::shared_ptr<MetadataHandler> FileRequestHandler::getResourceMetadataHandler(std::shared_ptr<CdsObject>& obj, std::shared_ptr<CdsResource>& resource) const
 {
     auto resHandler = resource->getHandlerType();
-    if (resource->getAttribute(CdsResource::Attribute::RESOURCE_FILE).empty()) {
-        auto objID = stoiString(resource->getAttribute(CdsResource::Attribute::FANART_OBJ_ID));
-        auto resID = stoiString(resource->getAttribute(CdsResource::Attribute::FANART_RES_ID));
+    if (resource->getAttribute(ResourceAttribute::RESOURCE_FILE).empty()) {
+        auto objID = stoiString(resource->getAttribute(ResourceAttribute::FANART_OBJ_ID));
+        auto resID = stoiString(resource->getAttribute(ResourceAttribute::FANART_RES_ID));
         try {
             auto resObj = (objID > 0 && objID != obj->getID()) ? database->loadObject(objID) : nullptr;
             if (resObj) {
@@ -216,10 +222,10 @@ std::unique_ptr<MetadataHandler> FileRequestHandler::getResourceMetadataHandler(
             log_error(ex.what());
         }
     }
-    return MetadataHandler::createHandler(context, content, resHandler);
+    return metadataService->getHandler(resHandler);
 }
 
-std::unique_ptr<IOHandler> FileRequestHandler::open(const char* filename, enum UpnpOpenFileMode mode)
+std::unique_ptr<IOHandler> FileRequestHandler::open(const char* filename, const std::shared_ptr<Quirks>& quirks, enum UpnpOpenFileMode mode)
 {
     log_debug("start: {}", filename);
 
@@ -250,7 +256,7 @@ std::unique_ptr<IOHandler> FileRequestHandler::open(const char* filename, enum U
     }
 
     if (!trProfile.empty()) {
-        auto transcodingProfile = config->getTranscodingProfileListOption(CFG_TRANSCODING_PROFILE_LIST)->getByName(trProfile);
+        auto transcodingProfile = config->getTranscodingProfileListOption(ConfigVal::TRANSCODING_PROFILE_LIST)->getByName(trProfile);
         if (!transcodingProfile)
             throw_std_runtime_error("Transcoding of file {} but no profile matching the name {} found", path.c_str(), trProfile);
 
@@ -266,7 +272,7 @@ std::unique_ptr<IOHandler> FileRequestHandler::open(const char* filename, enum U
     return std::make_unique<FileIOHandler>(path);
 }
 
-std::size_t FileRequestHandler::parseResourceInfo(std::map<std::string, std::string>& params)
+std::size_t FileRequestHandler::parseResourceInfo(const std::map<std::string, std::string>& params)
 {
     std::size_t resourceId = 0;
     try {
@@ -281,11 +287,3 @@ std::size_t FileRequestHandler::parseResourceInfo(std::map<std::string, std::str
 
     return resourceId;
 }
-
-std::unique_ptr<Quirks> FileRequestHandler::getQuirks(const UpnpFileInfo* info) const
-{
-    auto ctrlPtIPAddr = std::make_shared<GrbNet>(UpnpFileInfo_get_CtrlPtIPAddr(info));
-    // HINT: most clients do not report exactly the same User-Agent for UPnP services and file request.
-    std::string userAgent = UpnpFileInfo_get_Os_cstr(info);
-    return std::make_unique<Quirks>(xmlBuilder, context->getClients(), ctrlPtIPAddr, std::move(userAgent));
-}
