@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,32 +32,37 @@
 /// \file exiv2_handler.cc
 
 #ifdef HAVE_EXIV2
-#define LOG_FAC log_facility_t::exif
+#define GRB_LOG_FAC GrbLogFacility::exiv2
 #include "exiv2_handler.h" // API
 
-#include <exiv2/exiv2.hpp>
-
-#include "cds/cds_objects.h"
+#include "cds/cds_item.h"
+#include "config/config_val.h"
 #include "iohandler/io_handler.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 
+#include <exiv2/exiv2.hpp>
+
 #if EXIV2_TEST_VERSION(0, 28, 0)
 #define AnyError Error
 #endif
 
 Exiv2Handler::Exiv2Handler(const std::shared_ptr<Context>& context)
-    : MetadataHandler(context)
+    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_EXIV2_ENABLED, ConfigVal::IMPORT_LIBOPTS_EXIV2_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_EXIV2_AUXDATA_TAGS_LIST)
 {
     // silence exiv2 messages without debug
     Exiv2::LogMsg::setHandler([](auto, auto s) { log_debug("Exiv2: {}", s); });
 }
 
-void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& item)
+void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
+    auto item = std::dynamic_pointer_cast<CdsItem>(obj);
+    if (!item || !isEnabled)
+        return;
+
     try {
         std::string value;
-        const auto sc = StringConverter::m2i(CFG_IMPORT_LIBOPTS_EXIV2_CHARSET, item->getLocation(), config);
+        const auto sc = StringConverter::m2i(ConfigVal::IMPORT_LIBOPTS_EXIV2_CHARSET, item->getLocation(), config);
 
         const auto image = Exiv2::ImageFactory::open(item->getLocation());
         image->readMetadata();
@@ -82,11 +87,11 @@ void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& item)
             if (value.length() >= 19) {
                 value = fmt::format("{}-{}-{}T{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2), value.substr(11, 8));
                 log_debug("date: {}", value);
-                item->addMetaData(M_DATE, value);
+                item->addMetaData(MetadataFields::M_DATE, value);
             } else if (value.length() >= 11) {
                 value = fmt::format("{}-{}-{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2));
                 log_debug("date: {}", value);
-                item->addMetaData(M_DATE, value);
+                item->addMetaData(MetadataFields::M_DATE, value);
             }
         }
 
@@ -149,13 +154,12 @@ void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& item)
         }  */
 
         if (!comment.empty()) {
-            item->addMetaData(M_DESCRIPTION, sc->convert(comment));
+            item->addMetaData(MetadataFields::M_DESCRIPTION, sc->convert(comment));
         }
 
         // if there are any metadata tags that the user wants - add them
-        const auto meta = config->getDictionaryOption(CFG_IMPORT_LIBOPTS_EXIV2_METADATA_TAGS_LIST);
-        if (!meta.empty()) {
-            for (auto&& [metatag, metakey] : meta) {
+        if (!metaTags.empty()) {
+            for (auto&& [metatag, metakey] : metaTags) {
                 std::string metaval;
                 log_debug("metatag: {} ", metatag.c_str());
                 if (startswith(metatag, "Exif")) {
@@ -167,8 +171,8 @@ void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& item)
                     if (xmpMd != xmpData.end())
                         metaval = trimString(xmpMd->toString());
                 } else {
-                    log_debug("Invalid meta Tag {}", metatag.c_str());
-                    break;
+                    log_warning("Invalid meta Tag {}", metatag.c_str());
+                    continue;
                 }
                 if (!metaval.empty()) {
                     metaval = sc->convert(metaval);
@@ -181,9 +185,8 @@ void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& item)
         }
 
         // if there are any auxilary tags that the user wants - add them
-        const auto aux = config->getArrayOption(CFG_IMPORT_LIBOPTS_EXIV2_AUXDATA_TAGS_LIST);
-        if (!aux.empty()) {
-            for (auto&& auxtag : aux) {
+        if (!auxTags.empty()) {
+            for (auto&& auxtag : auxTags) {
                 std::string auxval;
                 log_debug("auxtag: {} ", auxtag.c_str());
                 if (startswith(auxtag, "Exif")) {
@@ -195,8 +198,8 @@ void Exiv2Handler::fillMetadata(const std::shared_ptr<CdsObject>& item)
                     if (xmpMd != xmpData.end())
                         auxval = trimString(xmpMd->toString());
                 } else {
-                    log_debug("Invalid Aux Tag {}", auxtag.c_str());
-                    break;
+                    log_warning("Invalid Aux Tag {}", auxtag.c_str());
+                    continue;
                 }
                 if (!auxval.empty()) {
                     auxval = sc->convert(auxval);
