@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -42,9 +42,18 @@
 // in the ffmpeg sources
 
 #ifdef HAVE_FFMPEG
-#define LOG_FAC log_facility_t::ffmpeg
+#define GRB_LOG_FAC GrbLogFacility::ffmpeg
 #include "ffmpeg_handler.h"
 
+#include "cds/cds_item.h"
+#include "config/config_val.h"
+#include "iohandler/io_handler.h"
+#include "upnp/upnp_common.h"
+#include "util/grb_time.h"
+#include "util/string_converter.h"
+#include "util/tools.h"
+
+#include <algorithm>
 #include <fmt/chrono.h>
 
 extern "C" {
@@ -54,35 +63,88 @@ extern "C" {
 
 } // extern "C"
 
-#include "cds/cds_item.h"
-#include "iohandler/io_handler.h"
-#include "upnp_common.h"
-#include "util/grb_time.h"
-#include "util/string_converter.h"
-#include "util/tools.h"
-
 #ifdef HAVE_AVSTREAM_CODECPAR
 #define as_codecpar(s) s->codecpar
 #else
 #define as_codecpar(s) s->codec
 #endif
 
+class FfmpegLogger {
+public:
+    FfmpegLogger()
+    {
+        av_log_set_level(logLevel);
+        av_log_set_callback(&FfmpegLogger::LogFfmpegMessage);
+    }
+    ~FfmpegLogger()
+    {
+        av_log_set_callback(nullptr);
+    }
+
+private:
+    FfmpegLogger(const FfmpegLogger&) = delete;
+    FfmpegLogger& operator=(const FfmpegLogger&) = delete;
+    static int printPrefix;
+    static int logLevel;
+
+    static void LogFfmpegMessage(void* ptr, int level, const char* fmt, va_list vargs)
+    {
+        int len = av_log_format_line2(ptr, level, fmt, vargs, nullptr, 0, &printPrefix);
+        std::vector<char> buf(len + 1); // need space for NUL
+        std::string message;
+        if (len > 0) {
+            av_log_format_line2(ptr, level, fmt, vargs, buf.data(), len, &printPrefix);
+            message = buf.data();
+        } else {
+            message = "Failed to format message";
+        }
+
+        if (level <= logLevel) { // some ffmpeg functions use log_level_offset
+            switch (level) {
+            case AV_LOG_PANIC: // ffmpeg will crash now
+            case AV_LOG_FATAL: // fatal as in can't decode, not crash
+                log_error("FFMpeg {} {}", level, message);
+                break;
+            case AV_LOG_ERROR:
+            case AV_LOG_WARNING:
+                log_debug("FFMpeg {}: {}", level, message);
+                break;
+            case AV_LOG_INFO:
+                log_debug("FFMpeg {}: {}", level, message);
+                break;
+            case AV_LOG_VERBOSE:
+            case AV_LOG_DEBUG:
+            case AV_LOG_TRACE:
+                log_debug("FFMpeg {}: {}", level, message);
+                break;
+            case AV_LOG_QUIET:
+                break;
+            default:
+                log_warning("FFMpeg unhandled {}: {}", level, message);
+                break;
+            }
+        }
+    }
+};
+
+static FfmpegLogger globalFfmpegLogger = FfmpegLogger();
+int FfmpegLogger::printPrefix = 1;
+int FfmpegLogger::logLevel = AV_LOG_INFO;
+
 FfmpegHandler::FfmpegHandler(const std::shared_ptr<Context>& context)
-    : MetadataHandler(context)
+    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_FFMPEG_ENABLED, ConfigVal::IMPORT_LIBOPTS_FFMPEG_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_FFMPEG_AUXDATA_TAGS_LIST)
 {
-    specialPropertyMap = this->config->getDictionaryOption(CFG_IMPORT_LIBOPTS_FFMPEG_METADATA_TAGS_LIST);
+    mappings = this->config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
 }
 
-void FfmpegHandler::addFfmpegAuxdataFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx) const
+void FfmpegHandler::addFfmpegAuxdataFields(const std::shared_ptr<CdsItem>& item, const std::unique_ptr<StringConverter>& sc, const AVFormatContext* pFormatCtx) const
 {
     if (!pFormatCtx->metadata) {
         log_debug("no metadata");
         return;
     }
 
-    auto sc = StringConverter::m2i(CFG_IMPORT_LIBOPTS_FFMPEG_CHARSET, item->getLocation(), config);
-    std::vector<std::string> aux = config->getArrayOption(CFG_IMPORT_LIBOPTS_FFMPEG_AUXDATA_TAGS_LIST);
-    for (auto&& desiredTag : aux) {
+    for (auto&& desiredTag : auxTags) {
         if (!desiredTag.empty()) {
             auto tag = av_dict_get(pFormatCtx->metadata, desiredTag.c_str(), nullptr, AV_DICT_IGNORE_SUFFIX);
             if (tag && tag->value && tag->value[0]) {
@@ -93,18 +155,17 @@ void FfmpegHandler::addFfmpegAuxdataFields(const std::shared_ptr<CdsItem>& item,
     }
 } // addFfmpegAuxdataFields
 
-void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx) const
+void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item, const std::unique_ptr<StringConverter>& sc, const AVFormatContext* pFormatCtx) const
 {
     AVDictionaryEntry* e = nullptr;
-    auto sc = StringConverter::m2i(CFG_IMPORT_LIBOPTS_FFMPEG_CHARSET, item->getLocation(), config);
 
     // only use ffmpeg meta data if not found by other handler
-    auto emptyProperties = std::map<metadata_fields_t, bool>();
+    auto emptyProperties = std::map<MetadataFields, bool>();
     for (auto&& prop : propertyMap) {
         emptyProperties[prop.first] = item->getMetaData(prop.first).empty();
     }
     auto emptySpecProperties = std::map<std::string, bool>();
-    for (auto&& prop : specialPropertyMap) {
+    for (auto&& prop : metaTags) {
         emptySpecProperties[prop.second] = item->getMetaData(prop.second).empty();
     }
 
@@ -112,8 +173,8 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
         std::string key = toLower(e->key);
         std::string value = e->value;
         log_debug("FFMpeg tag: {}: {}", key, value);
-        auto it = specialPropertyMap.find(e->key);
-        if (it != specialPropertyMap.end() && emptySpecProperties[it->second]) {
+        auto it = metaTags.find(e->key);
+        if (it != metaTags.end() && emptySpecProperties[it->second]) {
             log_debug("Identified special metadata '{}' as '{}': '{}'", it->first, it->second, value);
             item->addMetaData(it->second, sc->convert(trimString(value)));
             continue; // iterate while loop
@@ -126,7 +187,7 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
             log_debug("Identified default metadata '{}': {}", pIt->second, value);
             const auto field = pIt->first;
             if (emptyProperties[field]) {
-                if (field == M_DATE || field == M_CREATION_DATE) {
+                if (field == MetadataFields::M_DATE || field == MetadataFields::M_CREATION_DATE) {
                     std::tm tmWork {};
                     if (strptime(e->value, "%Y-%m-%dT%T.000000%Z", &tmWork)) {
                         // convert creation_time to local time
@@ -148,9 +209,9 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
                 } else
                     item->addMetaData(field, sc->convert(trimString(value)));
             }
-            if (field == M_TRACKNUMBER) {
+            if (field == MetadataFields::M_TRACKNUMBER) {
                 item->setTrackNumber(stoiString(value));
-            } else if (field == M_PARTNUMBER) {
+            } else if (field == MetadataFields::M_PARTNUMBER) {
                 item->setPartNumber(stoiString(value));
             }
         } else {
@@ -163,14 +224,14 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
 void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx)
 {
     auto resource = item->getResource(ContentHandler::DEFAULT);
-    bool isAudioFile = item->isSubClass(UPNP_CLASS_AUDIO_ITEM) && item->getResource(CdsResource::Purpose::Thumbnail);
-    auto resource2 = isAudioFile ? item->getResource(CdsResource::Purpose::Thumbnail) : item->getResource(ContentHandler::DEFAULT);
+    bool isAudioFile = item->isSubClass(UPNP_CLASS_AUDIO_ITEM) && item->getResource(ResourcePurpose::Thumbnail);
+    auto resource2 = isAudioFile ? item->getResource(ResourcePurpose::Thumbnail) : item->getResource(ContentHandler::DEFAULT);
 
     // duration
     if (pFormatCtx->duration > 0) {
         auto duration = millisecondsToHMSF(pFormatCtx->duration / (AV_TIME_BASE / 1000));
         log_debug("Added duration: {}", duration);
-        resource->addAttribute(CdsResource::Attribute::DURATION, duration);
+        resource->addAttribute(ResourceAttribute::DURATION, duration);
     }
 
     // bitrate
@@ -179,7 +240,7 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
         // See http://www.upnp.org/schemas/av/didl-lite-v3.xsd
         auto bitrate = pFormatCtx->bit_rate / 8;
         log_debug("Added bitrate: {} kb/s", bitrate);
-        resource->addAttribute(CdsResource::Attribute::BITRATE, fmt::to_string(bitrate));
+        resource->addAttribute(ResourceAttribute::BITRATE, fmt::to_string(bitrate));
     }
 
     // video resolution, audio sampling rate, nr of audio channels
@@ -190,7 +251,7 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
 
         if (st && !videoSet && as_codecpar(st)->codec_type == AVMEDIA_TYPE_VIDEO) {
             auto codecId = as_codecpar(st)->codec_id;
-            resource2->addAttribute(CdsResource::Attribute::VIDEOCODEC, avcodec_get_name(codecId));
+            resource2->addAttribute(ResourceAttribute::VIDEOCODEC, avcodec_get_name(codecId));
 
             if (as_codecpar(st)->codec_tag > 0) {
                 char fourcc[5];
@@ -210,13 +271,13 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
                 auto resolution = fmt::format("{}x{}", as_codecpar(st)->width, as_codecpar(st)->height);
 
                 log_debug("Added resolution: {} pixel from stream {}", resolution, i);
-                resource2->addAttribute(CdsResource::Attribute::RESOLUTION, resolution);
+                resource2->addAttribute(ResourceAttribute::RESOLUTION, resolution);
                 videoSet = true;
             }
         }
         if (st && !audioSet && as_codecpar(st)->codec_type == AVMEDIA_TYPE_AUDIO) {
             auto codecId = as_codecpar(st)->codec_id;
-            resource->addAttribute(CdsResource::Attribute::AUDIOCODEC, avcodec_get_name(codecId));
+            resource->addAttribute(ResourceAttribute::AUDIOCODEC, avcodec_get_name(codecId));
             // find the first stream that has a valid sample rate
 
             if (as_codecpar(st)->sample_rate > 0) {
@@ -227,7 +288,7 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
                 log_debug("Bits per coded sample: {}", bitsPerSample);
 
                 if (bitsPerSample > 0) {
-                    resource->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(bitsPerSample));
+                    resource->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(bitsPerSample));
 
                     // Fix up Sample rate reporting
                     // FFMpeg will tell us DSD is 16bit PCM, but its actually 1bit, so we have to multiply this out
@@ -237,7 +298,7 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
                 }
 
                 log_debug("Added sample frequency: {} Hz from stream {}", sampleFreq, i);
-                resource->addAttribute(CdsResource::Attribute::SAMPLEFREQUENCY, fmt::to_string(sampleFreq));
+                resource->addAttribute(ResourceAttribute::SAMPLEFREQUENCY, fmt::to_string(sampleFreq));
                 audioSet = true;
 // FF_API_OLD_CHANNEL_LAYOUT
 #if LIBAVUTIL_VERSION_MAJOR < 57
@@ -247,7 +308,7 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
 #endif
                 if (audioCh > 0) {
                     log_debug("Added number of audio channels: {} from stream {}", audioCh, i);
-                    resource->addAttribute(CdsResource::Attribute::NRAUDIOCHANNELS, fmt::to_string(audioCh));
+                    resource->addAttribute(ResourceAttribute::NRAUDIOCHANNELS, fmt::to_string(audioCh));
                 }
             }
         }
@@ -257,16 +318,13 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
 void FfmpegHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
-    if (!item)
+    if (!item || !isEnabled)
         return;
 
     log_debug("Running ffmpeg handler on {}", item->getLocation().c_str());
 
     AVFormatContext* pFormatCtx = nullptr;
 
-    // Suppress all log messages
-    av_log_set_callback([](auto...) {});
-
 #if (LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58, 9, 100))
     // Register all formats and codecs
     av_register_all();
@@ -280,10 +338,11 @@ void FfmpegHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
         avformat_close_input(&pFormatCtx);
         return; // Couldn't find stream information
     }
+    auto sc = StringConverter::m2i(ConfigVal::IMPORT_LIBOPTS_FFMPEG_CHARSET, item->getLocation(), config);
     // Add metadata using ffmpeg library calls
-    addFfmpegMetadataFields(item, pFormatCtx);
+    addFfmpegMetadataFields(item, sc, pFormatCtx);
     // Add auxdata
-    addFfmpegAuxdataFields(item, pFormatCtx);
+    addFfmpegAuxdataFields(item, sc, pFormatCtx);
     // Add resources using ffmpeg library calls
     addFfmpegResourceFields(item, pFormatCtx);
     // Close the video file
@@ -297,7 +356,6 @@ std::unique_ptr<IOHandler> FfmpegHandler::serveContent(const std::shared_ptr<Cds
 
 std::string FfmpegHandler::getMimeType() const
 {
-    auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     return getValueOrDefault(mappings, CONTENT_TYPE_JPG, "image/jpeg");
 }
 #endif // HAVE_FFMPEG
