@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -37,21 +37,23 @@
 /// \brief Definition of the FfmpegHandler class - getting metadata via
 /// ffmpeg library calls.
 
-#ifdef HAVE_FFMPEG
 #ifndef __FFMPEG_HANDLER_H__
 #define __FFMPEG_HANDLER_H__
+#ifdef HAVE_FFMPEG
 
 #include <array>
 #include <optional>
 
+#include "metadata_enums.h"
 #include "metadata_handler.h"
 
 // forward declaration
 class IOHandler;
 struct AVFormatContext;
+class StringConverter;
 
 /// \brief This class is responsible for reading id3 tags metadata
-class FfmpegHandler : public MetadataHandler {
+class FfmpegHandler : public MediaMetadataHandler {
 public:
     explicit FfmpegHandler(const std::shared_ptr<Context>& context);
     void fillMetadata(const std::shared_ptr<CdsObject>& obj) override;
@@ -59,26 +61,26 @@ public:
     std::string getMimeType() const override;
 
 private:
-    std::map<std::string, std::string> specialPropertyMap;
+    std::map<std::string, std::string> mappings;
 
-    void addFfmpegAuxdataFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx) const;
-    void addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx) const;
+    void addFfmpegAuxdataFields(const std::shared_ptr<CdsItem>& item, const std::unique_ptr<StringConverter>& sc, const AVFormatContext* pFormatCtx) const;
+    void addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item, const std::unique_ptr<StringConverter>& sc, const AVFormatContext* pFormatCtx) const;
     static void addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx);
 
-    static constexpr auto propertyMap = std::array {
-        std::pair(M_TITLE, "title"),
-        std::pair(M_ARTIST, "artist"),
-        std::pair(M_ALBUM, "album"),
-        std::pair(M_GENRE, "genre"),
-        std::pair(M_DESCRIPTION, "description"),
-        std::pair(M_TRACKNUMBER, "track"),
-        std::pair(M_PARTNUMBER, "discnumber"),
-        std::pair(M_ALBUMARTIST, "album_artist"),
-        std::pair(M_COMPOSER, "composer"),
-        std::pair(M_DATE, "date"),
-        std::pair(M_CREATION_DATE, "creation_time"),
+    static constexpr std::array propertyMap {
+        std::pair(MetadataFields::M_TITLE, "title"),
+        std::pair(MetadataFields::M_ARTIST, "artist"),
+        std::pair(MetadataFields::M_ALBUM, "album"),
+        std::pair(MetadataFields::M_GENRE, "genre"),
+        std::pair(MetadataFields::M_DESCRIPTION, "description"),
+        std::pair(MetadataFields::M_TRACKNUMBER, "track"),
+        std::pair(MetadataFields::M_PARTNUMBER, "discnumber"),
+        std::pair(MetadataFields::M_ALBUMARTIST, "album_artist"),
+        std::pair(MetadataFields::M_COMPOSER, "composer"),
+        std::pair(MetadataFields::M_DATE, "date"),
+        std::pair(MetadataFields::M_CREATION_DATE, "creation_time"),
     };
 };
 
-#endif //__FFMPEG_HANDLER_H__
 #endif // HAVE_FFMPEG
+#endif //__FFMPEG_HANDLER_H__
