@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,18 +32,20 @@
 /// \file libexif_handler.cc
 
 #ifdef HAVE_LIBEXIF
-#define LOG_FAC log_facility_t::exif
+#define GRB_LOG_FAC GrbLogFacility::exif
 #include "libexif_handler.h" // API
 
-#include <array>
-
 #include "cds/cds_item.h"
+#include "config/config_val.h"
+#include "exceptions.h"
 #include "iohandler/file_io_handler.h"
 #include "iohandler/mem_io_handler.h"
 #include "util/jpeg_resolution.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 
+#include <array>
+
 /// \brief Sets resolution for a given resource index, item must be a JPEG image
 static void setJpegResolutionResource(const std::shared_ptr<CdsItem>& item, std::size_t resNum = 0)
 {
@@ -56,7 +58,7 @@ static void setJpegResolutionResource(const std::shared_ptr<CdsItem>& item, std:
         fioH.open(UPNP_READ);
         auto resolution = getJpegResolution(fioH);
 
-        item->getResource(resNum)->addAttribute(CdsResource::Attribute::RESOLUTION, resolution.string());
+        item->getResource(resNum)->addAttribute(ResourceAttribute::RESOLUTION, resolution.string());
 
     } catch (const std::runtime_error& e) {
         log_error("Failed to parse EXIF {} resolution: {}", item->getLocation().c_str(), e.what());
@@ -178,8 +180,33 @@ static int getTagFromString(const std::string& tag)
     return result;
 }
 
+static void logfunc(ExifLog* log, ExifLogCode code, const char* domain, const char* format, va_list args, void* data)
+{
+    va_list args_copy;
+    va_copy(args_copy, args);
+    size_t len = std::vsnprintf(nullptr, 0, format, args_copy);
+    std::vector<char> buf(len + 1); // need space for NUL
+    std::vsnprintf(buf.data(), len + 1, format, args);
+    std::string message = buf.data();
+    log_debug("Exif: {}-{}, domain {}: {}", exif_log_code_get_title(code), exif_log_code_get_message(code), domain, message);
+}
+
+LibExifHandler::LibExifHandler(const std::shared_ptr<Context>& context)
+    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_EXIF_ENABLED, ConfigVal::IMPORT_LIBOPTS_EXIF_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_EXIF_AUXDATA_TAGS_LIST)
+{
+    log = exif_log_new();
+    exif_log_set_func(log, logfunc, nullptr);
+}
+
+LibExifHandler::~LibExifHandler()
+{
+    exif_log_free(log);
+}
+
 void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_ptr<CdsItem>& item,
-    const std::unique_ptr<StringConverter>& sc, const std::vector<std::string>& auxtags, const std::map<std::string, std::string>& metatags)
+    const std::unique_ptr<StringConverter>& sc,
+    std::string& imageX, std::string& imageY,
+    const std::vector<std::string>& auxtags, const std::map<std::string, std::string>& metatags)
 {
     constexpr auto BUFLEN = 4096;
     std::array<char, BUFLEN> exif_entry_buffer;
@@ -197,9 +224,9 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
                 // convert date to ISO 8601 as required in the UPnP spec
                 // from YYYY:MM:DD to YYYY-MM-DD
                 if (value.length() >= 19) {
-                    item->addMetaData(M_DATE, fmt::format("{}-{}-{}T{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2), value.substr(11, 8)));
+                    item->addMetaData(MetadataFields::M_DATE, fmt::format("{}-{}-{}T{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2), value.substr(11, 8)));
                 } else if (value.length() >= 11) {
-                    item->addMetaData(M_DATE, fmt::format("{}-{}-{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2)));
+                    item->addMetaData(MetadataFields::M_DATE, fmt::format("{}-{}-{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2)));
                 }
             }
             break;
@@ -208,7 +235,7 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
             auto value = trimString(exif_egv(entry));
             log_debug("Found exif comment: {}", value);
             if (!value.empty()) {
-                item->addMetaData(M_DESCRIPTION, sc->convert(value));
+                item->addMetaData(MetadataFields::M_DESCRIPTION, sc->convert(value));
             }
             break;
         }
@@ -260,44 +287,50 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
 void LibExifHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
-    if (!item)
+    if (!item || !isEnabled)
         return;
 
-    auto sc = StringConverter::m2i(CFG_IMPORT_LIBOPTS_EXIF_CHARSET, item->getLocation(), config);
+    auto sc = StringConverter::m2i(ConfigVal::IMPORT_LIBOPTS_EXIF_CHARSET, item->getLocation(), config);
     ExifData* exifData = exif_data_new_from_file(item->getLocation().c_str());
 
-    if (exifData == nullptr) {
+    if (!exifData) {
         log_debug("Exif data not found, attempting to set resolution internally...");
         setJpegResolutionResource(item);
         return;
     }
 
-    auto aux = config->getArrayOption(CFG_IMPORT_LIBOPTS_EXIF_AUXDATA_TAGS_LIST);
-    auto meta = config->getDictionaryOption(CFG_IMPORT_LIBOPTS_EXIF_METADATA_TAGS_LIST);
+    // image resolution in pixels
+    // the problem is that I do not know when I encounter the
+    // tags for X and Y, so I have to save the information
+    // and in the very end - when I have both values - add the
+    // appropriate resource
+    std::string imageX;
+    std::string imageY;
+
     for (auto* exifContent : exifData->ifd) {
         if (exifContent != nullptr) {
-            process_ifd(exifContent, item, sc, aux, meta);
+            process_ifd(exifContent, item, sc, imageX, imageY, auxTags, metaTags);
         }
     }
 
     // we got the image resolution so we can add our resource
     if (!imageX.empty() && !imageY.empty()) {
-        item->getResource(ContentHandler::DEFAULT)->addAttribute(CdsResource::Attribute::RESOLUTION, fmt::format("{}x{}", imageX, imageY));
+        item->getResource(ContentHandler::DEFAULT)->addAttribute(ResourceAttribute::RESOLUTION, fmt::format("{}x{}", imageX, imageY));
     } else {
         log_debug("EXIF resolution not found, falling back to parsing directly: {}", item->getLocation().c_str());
         setJpegResolutionResource(item);
     }
 
     if (exifData->size != 0U) {
-        auto resource = std::make_shared<CdsResource>(ContentHandler::LIBEXIF, CdsResource::Purpose::Thumbnail);
-        resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(item->getMimeType()));
+        auto resource = std::make_shared<CdsResource>(ContentHandler::LIBEXIF, ResourcePurpose::Thumbnail);
+        resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(item->getMimeType()));
         item->addResource(resource);
         try {
             auto ioH = MemIOHandler(exifData->data, exifData->size);
             ioH.open(UPNP_READ);
             auto thResolution = getJpegResolution(ioH);
             log_debug("EXIF Thumb Resolution: {}", thResolution.string());
-            resource->addAttribute(CdsResource::Attribute::RESOLUTION, thResolution.string());
+            resource->addAttribute(ResourceAttribute::RESOLUTION, thResolution.string());
         } catch (const std::runtime_error& e) {
             log_error("Failed to parse EXIF Thumbnail {} details: {}", item->getLocation().c_str(), e.what());
         }
@@ -311,18 +344,20 @@ std::unique_ptr<IOHandler> LibExifHandler::serveContent(const std::shared_ptr<Cd
     if (!item)
         return nullptr;
 
-    if (resource->getPurpose() != CdsResource::Purpose::Thumbnail)
+    if (resource->getPurpose() != ResourcePurpose::Thumbnail)
         throw_std_runtime_error("Resource {} is not a Thumbnail", resource->getPurpose());
 
-    ExifData* ed = exif_data_new_from_file(item->getLocation().c_str());
-    if (!ed)
+    ExifData* exifData = exif_data_new_from_file(item->getLocation().c_str());
+    if (!exifData)
         throw_std_runtime_error("Resource {} has no exif information", item->getLocation().string());
 
-    if (!(ed->size))
+    if (exifData->size == 0U) {
+        exif_data_unref(exifData);
         throw_std_runtime_error("Resource {} has no exif thumbnail", item->getLocation().string());
+    }
 
-    auto ioHandler = std::make_unique<MemIOHandler>(ed->data, ed->size);
-    exif_data_unref(ed);
+    auto ioHandler = std::make_unique<MemIOHandler>(exifData->data, exifData->size);
+    exif_data_unref(exifData);
     return ioHandler;
 }
 #endif // HAVE_LIBEXIF
