@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,10 +32,21 @@
 /// \file taglib_handler.cc
 
 #ifdef HAVE_TAGLIB
-#define LOG_FAC log_facility_t::taglib
+#define GRB_LOG_FAC GrbLogFacility::taglib
 
 #include "taglib_handler.h" // API
 
+#include "cds/cds_item.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "exceptions.h"
+#include "iohandler/mem_io_handler.h"
+#include "metadata_enums.h"
+#include "util/grb_time.h"
+#include "util/mime.h"
+#include "util/string_converter.h"
+#include "util/tools.h"
+
 #include <aifffile.h>
 #include <apefile.h>
 #include <asffile.h>
@@ -47,6 +58,7 @@
 #include <oggflacfile.h>
 #include <opusfile.h>
 #include <speexfile.h>
+#include <tdebuglistener.h>
 #include <textidentificationframe.h>
 #include <tfilestream.h>
 #include <tiostream.h>
@@ -54,26 +66,39 @@
 #include <vorbisfile.h>
 #include <wavpackfile.h>
 
-#include "cds/cds_item.h"
-#include "config/config_manager.h"
-#include "iohandler/mem_io_handler.h"
-#include "util/grb_time.h"
-#include "util/mime.h"
-#include "util/string_converter.h"
-#include "util/tools.h"
+#if TAGLIB_MAJOR_VERSION >= 2
+#include <mp4itemfactory.h>
+#endif
+
+class GerberaTagLibDebugListener : public TagLib::DebugListener {
+private:
+    GerberaTagLibDebugListener()
+    {
+        // Install listener.
+        TagLib::setDebugListener(this);
+    }
+
+    virtual void printMessage(const TagLib::String& msg) override
+    {
+        // Remove trailing newlines.
+        log_debug("{}", msg.stripWhiteSpace().to8Bit(true));
+    }
+    static GerberaTagLibDebugListener grbListener;
+};
+
+GerberaTagLibDebugListener GerberaTagLibDebugListener::grbListener;
 
 TagLibHandler::TagLibHandler(const std::shared_ptr<Context>& context)
-    : MetadataHandler(context)
+    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_ID3_ENABLED, ConfigVal::IMPORT_LIBOPTS_ID3_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST)
 {
-    entrySeparator = this->config->getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
-    legacyEntrySeparator = this->config->getOption(CFG_IMPORT_LIBOPTS_ENTRY_LEGACY_SEP);
+    entrySeparator = this->config->getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
+    legacyEntrySeparator = this->config->getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_LEGACY_SEP);
     trimStringInPlace(legacyEntrySeparator);
     if (legacyEntrySeparator.empty())
         legacyEntrySeparator = "\n";
-    specialPropertyMap = this->config->getDictionaryOption(CFG_IMPORT_LIBOPTS_ID3_METADATA_TAGS_LIST);
 }
 
-void TagLibHandler::addField(metadata_fields_t field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const
+void TagLibHandler::addField(MetadataFields field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const
 {
     if (!tag || tag->isEmpty())
         return;
@@ -82,42 +107,42 @@ void TagLibHandler::addField(metadata_fields_t field, const TagLib::File& file,
 
     std::vector<std::string> value;
 
-    auto propertyMap = std::map<metadata_fields_t, std::string> {
-        { M_ARTIST, "ARTIST" },
-        { M_ALBUMARTIST, "ALBUMARTIST" },
-        { M_COMPOSER, "COMPOSER" },
-        { M_CONDUCTOR, "CONDUCTOR" },
-        { M_GENRE, "GENRE" },
-        { M_ORCHESTRA, "ORCHESTRA" },
+    auto propertyMap = std::map<MetadataFields, std::string> {
+        { MetadataFields::M_ARTIST, "ARTIST" },
+        { MetadataFields::M_ALBUMARTIST, "ALBUMARTIST" },
+        { MetadataFields::M_COMPOSER, "COMPOSER" },
+        { MetadataFields::M_CONDUCTOR, "CONDUCTOR" },
+        { MetadataFields::M_GENRE, "GENRE" },
+        { MetadataFields::M_ORCHESTRA, "ORCHESTRA" },
     };
 
     switch (field) {
-    case M_TITLE:
+    case MetadataFields::M_TITLE:
         value.push_back(tag->title().to8Bit(true));
         break;
-    case M_ALBUM:
+    case MetadataFields::M_ALBUM:
         value.push_back(tag->album().to8Bit(true));
         break;
-    case M_DATE:
-    case M_UPNP_DATE: {
+    case MetadataFields::M_DATE:
+    case MetadataFields::M_UPNP_DATE: {
         unsigned int i = tag->year();
         if (i == 0)
             return;
         value.push_back(fmt::format("{}-01-01", i));
         break;
     }
-    case M_DESCRIPTION:
+    case MetadataFields::M_DESCRIPTION:
         value.push_back(tag->comment().to8Bit(true));
         break;
-    case M_TRACKNUMBER: {
+    case MetadataFields::M_TRACKNUMBER: {
         std::uint32_t i = tag->track();
-        if (i == 0 || i > std::uint32_t(std::numeric_limits<int>::max()))
+        if (i == 0 || i > static_cast<std::uint32_t>(std::numeric_limits<int>::max()))
             return;
         value.push_back(fmt::to_string(i));
         item->setTrackNumber(i);
         break;
     }
-    case M_PARTNUMBER: {
+    case MetadataFields::M_PARTNUMBER: {
         auto list = file.properties()["DISCNUMBER"];
         if (!list.isEmpty()) {
             value.push_back(list[0].to8Bit(true));
@@ -164,7 +189,7 @@ void TagLibHandler::addSpecialFields(const TagLib::File& file, const TagLib::Tag
 
     auto sc = StringConverter::i2i(config); // sure is sure
 
-    for (auto&& [key, meta] : specialPropertyMap) {
+    for (auto&& [key, meta] : metaTags) {
         auto list = file.properties()[key];
         if (list.isEmpty())
             return;
@@ -185,7 +210,7 @@ void TagLibHandler::populateGenericTags(const std::shared_ptr<CdsItem>& item, co
         return;
 
     const TagLib::Tag* tag = file.tag();
-    for (auto&& [field, key] : mt_keys)
+    for (auto&& [field, key] : MetaEnumMapper::mt_keys)
         addField(field, file, tag, item);
 
     addSpecialFields(file, tag, item);
@@ -199,29 +224,28 @@ void TagLibHandler::populateGenericTags(const std::shared_ptr<CdsItem>& item, co
     // UPnP bitrate is in bytes/second
     int temp = audioProps->bitrate() * 1024 / 8; // kbit/second -> byte/second
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITRATE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITRATE, fmt::to_string(temp));
     }
 
     temp = audioProps->lengthInMilliseconds();
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::DURATION, millisecondsToHMSF(temp));
+        res->addAttribute(ResourceAttribute::DURATION, millisecondsToHMSF(temp));
     }
 
     temp = audioProps->sampleRate();
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::SAMPLEFREQUENCY, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::SAMPLEFREQUENCY, fmt::to_string(temp));
     }
 
     temp = audioProps->channels();
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::NRAUDIOCHANNELS, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::NRAUDIOCHANNELS, fmt::to_string(temp));
     }
 }
 
 void TagLibHandler::populateAuxTags(const std::shared_ptr<CdsItem>& item, const TagLib::PropertyMap& propertyMap, const std::unique_ptr<StringConverter>& sc) const
 {
-    auto auxTagsList = config->getArrayOption(CFG_IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST);
-    for (auto&& desiredTag : auxTagsList) {
+    for (auto&& desiredTag : auxTags) {
         if (desiredTag.empty()) {
             continue;
         }
@@ -247,10 +271,10 @@ void TagLibHandler::populateAuxTags(const std::shared_ptr<CdsItem>& item, const
 void TagLibHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
-    if (!item)
+    if (!item || !isEnabled)
         return;
 
-    auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+    auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     std::string contentType = getValueOrDefault(mappings, item->getMimeType());
 
     auto fs = TagLib::FileStream(item->getLocation().c_str(), true); // true = Read only
@@ -301,8 +325,8 @@ void TagLibHandler::addArtworkResource(const std::shared_ptr<CdsItem>& item, con
     log_debug("Found artwork of type {} in file {}", artMimetype.c_str(), item->getLocation().c_str());
 
     if (artMimetype != MIMETYPE_DEFAULT) {
-        auto resource = std::make_shared<CdsResource>(ContentHandler::ID3, CdsResource::Purpose::Thumbnail);
-        resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(artMimetype));
+        auto resource = std::make_shared<CdsResource>(ContentHandler::ID3, ResourcePurpose::Thumbnail);
+        resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(artMimetype));
         item->addResource(resource);
     }
 }
@@ -317,7 +341,7 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
     if (!item) // not streamable
         return nullptr;
 
-    auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+    auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     std::string contentType = getValueOrDefault(mappings, item->getMimeType());
     auto itemLocation = item->getLocation().c_str();
 
@@ -325,7 +349,11 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
 
     if (contentType == CONTENT_TYPE_MP3) {
         // stream album art from MP3 file
+#if TAGLIB_MAJOR_VERSION >= 2
+        auto f = TagLib::MPEG::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
         auto f = TagLib::MPEG::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
 
         if (!f.isValid())
             throw_std_runtime_error("Could not open file {} as MP3", itemLocation);
@@ -346,7 +374,11 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
     }
     if (contentType == CONTENT_TYPE_FLAC) {
         // stream album art from FLAC file
+#if TAGLIB_MAJOR_VERSION >= 2
+        auto f = TagLib::FLAC::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
         auto f = TagLib::FLAC::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
 
         if (!f.isValid())
             throw_std_runtime_error("Could not open file {} as flac", itemLocation);
@@ -418,7 +450,7 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
         if (!f || !f->isValid() || !f->tag())
             throw_std_runtime_error("Could not open file {} as OGG", itemLocation);
 
-        const TagLib::List<TagLib::FLAC::Picture*> picList = reinterpret_cast<TagLib::Ogg::XiphComment*>(f->tag())->pictureList();
+        const auto picList = dynamic_cast<TagLib::Ogg::XiphComment*>(f->tag())->pictureList();
         if (picList.isEmpty())
             throw_std_runtime_error("OGG resource {} has empty picture information", itemLocation);
 
@@ -433,7 +465,11 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
 
 void TagLibHandler::extractMP3(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const
 {
+#if TAGLIB_MAJOR_VERSION >= 2
+    auto mp3 = TagLib::MPEG::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
     auto mp3 = TagLib::MPEG::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
     if (!mp3.isValid()) {
         log_info("TagLibHandler {}: does not appear to be a valid mp3 file", item->getLocation().c_str());
         return;
@@ -451,7 +487,7 @@ void TagLibHandler::extractMP3(TagLib::IOStream& roStream, const std::shared_ptr
     // http://id3.org/id3v2.4.0-frames "4.2.6. User defined text information frame"
     bool hasTXXXFrames = frameListMap.contains("TXXX");
 
-    std::vector<std::string> auxTagsList = config->getArrayOption(CFG_IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST);
+    std::vector<std::string> auxTagsList = config->getArrayOption(ConfigVal::IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST);
     for (auto&& desiredFrame : auxTagsList) {
         if (desiredFrame.empty()) {
             continue;
@@ -577,7 +613,7 @@ void TagLibHandler::extractOgg(TagLib::IOStream& roStream, const std::shared_ptr
     // Vorbis uses the FLAC binary picture structure...
     // https://wiki.xiph.org/VorbisComment#Cover_art
     // The unofficial COVERART field is not supported.
-    const TagLib::List<TagLib::FLAC::Picture*> picList = reinterpret_cast<TagLib::Ogg::XiphComment*>(oggFile->tag())->pictureList();
+    const auto picList = dynamic_cast<TagLib::Ogg::XiphComment*>(oggFile->tag())->pictureList();
     if (picList.isEmpty())
         return;
 
@@ -609,7 +645,7 @@ void TagLibHandler::extractASF(TagLib::IOStream& roStream, const std::shared_ptr
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 
     const TagLib::ASF::AttributeListMap& attrListMap = asf.tag()->attributeListMap();
@@ -632,7 +668,11 @@ void TagLibHandler::extractASF(TagLib::IOStream& roStream, const std::shared_ptr
 
 void TagLibHandler::extractFLAC(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const
 {
+#if TAGLIB_MAJOR_VERSION >= 2
+    auto flac = TagLib::FLAC::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
     auto flac = TagLib::FLAC::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
 
     if (!flac.isValid()) {
         log_info("TagLibHandler {}: does not appear to be a valid flac file", item->getLocation().c_str());
@@ -648,7 +688,7 @@ void TagLibHandler::extractFLAC(TagLib::IOStream& roStream, const std::shared_pt
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 
     if (flac.pictureList().isEmpty()) {
@@ -683,7 +723,7 @@ void TagLibHandler::extractAPE(TagLib::IOStream& roStream, const std::shared_ptr
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 }
 
@@ -705,13 +745,17 @@ void TagLibHandler::extractWavPack(TagLib::IOStream& roStream, const std::shared
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 }
 
 void TagLibHandler::extractMP4(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const
 {
+#if TAGLIB_MAJOR_VERSION >= 2
+    auto mp4 = TagLib::MP4::File(&roStream, true, TagLib::MP4::Properties::Average, TagLib::MP4::ItemFactory::instance());
+#else
     auto mp4 = TagLib::MP4::File(&roStream);
+#endif
 
     if (!mp4.isValid()) {
         log_info("TagLibHandler {}: does not appear to be a valid mp4 file", item->getLocation().c_str());
@@ -733,7 +777,7 @@ void TagLibHandler::extractMP4(TagLib::IOStream& roStream, const std::shared_ptr
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 
     if (mp4.tag()->contains("covr")) {
@@ -774,7 +818,7 @@ void TagLibHandler::extractAiff(TagLib::IOStream& roStream, const std::shared_pt
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 }
 
