@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -46,7 +46,7 @@
 class StringConverter;
 
 /// \brief This class is responsible for reading id3 or ogg tags metadata
-class TagLibHandler : public MetadataHandler {
+class TagLibHandler : public MediaMetadataHandler {
 public:
     explicit TagLibHandler(const std::shared_ptr<Context>& context);
 
@@ -63,9 +63,8 @@ public:
 private:
     std::string entrySeparator;
     std::string legacyEntrySeparator;
-    std::map<std::string, std::string> specialPropertyMap;
 
-    void addField(metadata_fields_t field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const;
+    void addField(MetadataFields field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const;
     void addSpecialFields(const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const;
 
     static std::unique_ptr<TagLib::File> getOggFile(TagLib::IOStream& ioStream);
