@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,23 +30,28 @@
 */
 
 /// \file request_handler.cc
-#define LOG_FAC log_facility_t::requests
+#define GRB_LOG_FAC GrbLogFacility::requests
 
 #include "request_handler.h" // API
 
-#include "content/content_manager.h"
+#include "common.h"
+#include "content/content.h"
+#include "context.h"
 #include "database/database.h"
+#include "exceptions.h"
+#include "upnp/quirks.h"
 #include "util/grb_net.h"
 #include "util/tools.h"
 
 #include <fmt/core.h>
 
-RequestHandler::RequestHandler(std::shared_ptr<ContentManager> content)
+RequestHandler::RequestHandler(std::shared_ptr<Content> content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder)
     : content(std::move(content))
     , context(this->content->getContext())
     , config(this->context->getConfig())
     , mime(this->context->getMime())
     , database(this->context->getDatabase())
+    , xmlBuilder(std::move(xmlBuilder))
 {
 }
 
@@ -65,3 +70,11 @@ std::shared_ptr<CdsObject> RequestHandler::loadObject(const std::map<std::string
 
     return database->loadObject(group, objectID);
 }
+
+std::shared_ptr<Quirks> RequestHandler::getQuirks(const UpnpFileInfo* info) const
+{
+    auto ctrlPtIPAddr = std::make_shared<GrbNet>(UpnpFileInfo_get_CtrlPtIPAddr(info));
+    // HINT: most clients do not report exactly the same User-Agent for UPnP services and file request.
+    std::string userAgent = UpnpFileInfo_get_Os_cstr(info);
+    return std::make_shared<Quirks>(xmlBuilder, context->getClients(), ctrlPtIPAddr, std::move(userAgent));
+}
