@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -37,46 +37,62 @@
 
 #include <map>
 #include <memory>
+#include <upnp.h>
 #include <utility>
 #include <vector>
 
-#include <upnp.h>
-
-#include "common.h"
-#include "context.h"
-
 // forward declaration
 class CdsObject;
+struct ClientInfo;
 class Config;
 class Context;
-class ContentManager;
+class Content;
 class Database;
 class IOHandler;
 class Mime;
+class Quirks;
 class Server;
+class UpnpXMLBuilder;
+
+#define LINK_FILE_REQUEST_HANDLER "/" CONTENT_MEDIA_HANDLER "/"
+#define LINK_URL_REQUEST_HANDLER "/" CONTENT_ONLINE_HANDLER "/"
+
+#define URL_PARAM_TRANSCODE_PROFILE_NAME "pr_name"
+#define URL_PARAM_TRANSCODE "tr"
+#define URL_VALUE_TRANSCODE_NO_RES_ID "none"
+#define URL_VALUE_TRANSCODE "1"
+#define URL_OBJECT_ID "object_id"
+#define URL_REQUEST_TYPE "req_type"
+#define URL_RESOURCE_ID "res_id"
 
 class RequestHandler {
 public:
-    explicit RequestHandler(std::shared_ptr<ContentManager> content);
+    explicit RequestHandler(std::shared_ptr<Content> content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     virtual ~RequestHandler() = default;
 
     /// \brief Returns information about the requested content.
     /// \param filename Requested URL
-    /// \param info File_Info structure, quite similar to statbuf.
-    virtual void getInfo(const char* filename, UpnpFileInfo* info) = 0;
+    /// \param info UpnpFileInfo structure, quite similar to statbuf.
+    /// \return the ClientInfo details to be provided to quirks
+    virtual const struct ClientInfo* getInfo(const char* filename, UpnpFileInfo* info) = 0;
 
     /// \brief Prepares the output buffer and calls the process function.
-    /// \return IOHandler
-    virtual std::unique_ptr<IOHandler> open(const char* filename, enum UpnpOpenFileMode mode) = 0;
+    /// \param quirks allows modifying the content of the response based on the client
+    /// \param mode either UPNP_READ or UPNP_WRITE
+    /// \return the appropriate IOHandler for the request.
+    virtual std::unique_ptr<IOHandler> open(const char* filename, const std::shared_ptr<Quirks>& quirks, enum UpnpOpenFileMode mode) = 0;
 
     std::shared_ptr<CdsObject> loadObject(const std::map<std::string, std::string>& params) const;
 
 protected:
-    std::shared_ptr<ContentManager> content;
+    std::shared_ptr<Content> content;
     std::shared_ptr<Context> context;
     std::shared_ptr<Config> config;
     std::shared_ptr<Mime> mime;
     std::shared_ptr<Database> database;
+    std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
+
+    std::shared_ptr<Quirks> getQuirks(const UpnpFileInfo* info) const;
 };
 
 #endif // __REQUEST_HANDLER_H__
