@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,10 +30,52 @@
 */
 
 /// \file server.cc
-#define LOG_FAC log_facility_t::server
+#define GRB_LOG_FAC GrbLogFacility::server
 
 #include "server.h" // API
 
+#include "action_request.h"
+#include "config/config.h"
+#include "config/config_option_enum.h"
+#include "config/config_val.h"
+#include "content/content_manager.h"
+#include "context.h"
+#include "database/database.h"
+#include "device_description_handler.h"
+#include "exceptions.h"
+#include "file_request_handler.h"
+#include "iohandler/io_handler.h"
+#include "metadata/metadata_service.h"
+#include "request_handler.h"
+#include "subscription_request.h"
+#include "ui_handler.h"
+#include "upnp/clients.h"
+#include "upnp/conn_mgr_service.h"
+#include "upnp/cont_dir_service.h"
+#include "upnp/mr_reg_service.h"
+#include "upnp/upnp_common.h"
+#include "upnp/xml_builder.h"
+#include "upnp_desc_handler.h"
+#include "util/grb_net.h"
+#include "util/mime.h"
+#include "util/tools.h"
+#include "util/url_utils.h"
+#include "util/xml_to_json.h"
+#include "web/pages.h"
+#include "web/session_manager.h"
+
+#ifdef HAVE_CURL
+#include "url_request_handler.h"
+#endif
+
+#ifdef HAVE_INOTIFY
+#include "content/inotify/autoscan_inotify.h"
+#endif
+
+#ifdef ONLINE_SERVICES
+#include "content/onlineservice/online_service.h"
+#endif
+
 #ifdef HAVE_CURL
 #include <curl/curl.h>
 #endif
@@ -46,46 +88,7 @@
 #include <sstream>
 #include <thread>
 
-#include "action_request.h"
-#include "config/config_manager.h"
-#include "config/config_option_enum.h"
-#include "content/autoscan.h"
-#include "content/content_manager.h"
-#include "database/database.h"
-#include "device_description_handler.h"
-#include "file_request_handler.h"
-#include "iohandler/io_handler.h"
-#include "subscription_request.h"
-#include "upnp_cds.h"
-#include "upnp_cm.h"
-#include "upnp_common.h"
-#include "upnp_mrreg.h"
-#include "util/grb_net.h"
-#include "util/mime.h"
-#include "util/tools.h"
-#include "util/upnp_clients.h"
-#include "util/url_utils.h"
-#include "util/xml_to_json.h"
-#include "web/pages.h"
-#include "web/session_manager.h"
-
-#ifdef HAVE_JS
-#include "content/scripting/playlist_parser_script.h"
-#endif
-
-#ifdef HAVE_CURL
-#include "url_request_handler.h"
-#endif
-
-#ifdef HAVE_INOTIFY
-#include "content/autoscan_inotify.h"
-#endif
-
-#ifdef ONLINE_SERVICES
-#include "content/onlineservice/online_service.h"
-#endif
-
-constexpr auto DEVICE_DESCRIPTION_PATH = std::string_view("description.xml");
+constexpr auto DEVICE_DESCRIPTION_PATH = std::string_view(UPNP_DESC_SCPD_URL UPNP_DESC_DEVICE_DESCRIPTION);
 
 Server::Server(std::shared_ptr<Config> config)
     : config(std::move(config))
@@ -94,11 +97,10 @@ Server::Server(std::shared_ptr<Config> config)
 
 void Server::init(bool offln)
 {
-    virtual_directory = SERVER_VIRTUAL_DIR;
     offline = offln;
 
-    serverUDN = config->getOption(CFG_SERVER_UDN);
-    aliveAdvertisementInterval = config->getIntOption(CFG_SERVER_ALIVE_INTERVAL);
+    serverUDN = config->getOption(ConfigVal::SERVER_UDN);
+    aliveAdvertisementInterval = config->getIntOption(ConfigVal::SERVER_ALIVE_INTERVAL);
 
 #ifdef HAVE_CURL
     curl_global_init(CURL_GLOBAL_ALL);
@@ -114,12 +116,19 @@ void Server::init(bool offln)
     config->updateConfigFromDatabase(database);
 
     clientManager = std::make_shared<ClientManager>(config, database);
-    session_manager = std::make_shared<Web::SessionManager>(config, timer);
-    context = std::make_shared<Context>(config, clientManager, mime, database, session_manager);
+    sessionManager = std::make_shared<Web::SessionManager>(config, timer);
+    context = std::make_shared<Context>(config, clientManager, mime, database, sessionManager);
 
     content = std::make_shared<ContentManager>(context, self, timer);
+    metadataService = std::make_shared<MetadataService>(context, content);
 }
 
+struct UpnpDesc {
+    enum Upnp_DescType_e mode;
+    std::string desc;
+    int len;
+};
+
 void Server::run()
 {
     log_debug("Starting...");
@@ -132,8 +141,8 @@ void Server::run()
         throw UpnpException(500, fmt::format("run: Activating ContentService failed {}", ex.what()));
     }
 
-    std::string iface = config->getOption(CFG_SERVER_NETWORK_INTERFACE);
-    ip = config->getOption(CFG_SERVER_IP);
+    std::string iface = config->getOption(ConfigVal::SERVER_NETWORK_INTERFACE);
+    ip = config->getOption(ConfigVal::SERVER_IP);
 
     if (!ip.empty() && !iface.empty())
         throw_std_runtime_error("You cannot specify interface {} and IP {} at the same time", iface, ip);
@@ -145,12 +154,12 @@ void Server::run()
         throw_std_runtime_error("Could not find IP: {}", ip);
 
     // check webroot directory
-    std::string webRoot = config->getOption(CFG_SERVER_WEBROOT);
+    std::string webRoot = config->getOption(ConfigVal::SERVER_WEBROOT);
     if (webRoot.empty()) {
         throw_std_runtime_error("Invalid web server root directory {}", webRoot);
     }
 
-    auto configPort = in_port_t(config->getIntOption(CFG_SERVER_PORT));
+    auto configPort = static_cast<in_port_t>(config->getUIntOption(ConfigVal::SERVER_PORT));
     auto ret = startupInterface(iface, configPort);
     if (ret != UPNP_E_SUCCESS) {
         throw UpnpException(ret, fmt::format("UpnpInit failed {} {}", iface, port));
@@ -160,44 +169,35 @@ void Server::run()
         throw UpnpException(ret, fmt::format("LibUPnP failed to bind to request port! Bound to {}, requested: {}", port, configPort));
     }
 
-    ret = UpnpSetWebServerRootDir(webRoot.c_str());
-    if (ret != UPNP_E_SUCCESS) {
-        throw UpnpException(ret, fmt::format("UpnpSetWebServerRootDir failed {}", webRoot));
-    }
-
     log_debug("webroot: {}", webRoot);
-    log_debug("Setting virtual dir to: {}", virtual_directory);
-
-    ret = UpnpAddVirtualDir(virtual_directory.c_str(), this, nullptr);
+    log_debug("Adding root virtual dir");
+    ret = UpnpAddVirtualDir("/", this, nullptr);
     if (ret != UPNP_E_SUCCESS) {
-        throw UpnpException(ret, fmt::format("run: UpnpAddVirtualDir failed {}", virtual_directory));
+        throw UpnpException(ret, "run: UpnpAddVirtualDir failed!");
     }
 
     ret = registerVirtualDirCallbacks();
-
     if (ret != UPNP_E_SUCCESS) {
         throw UpnpException(ret, "run: UpnpSetVirtualDirCallbacks failed");
     }
 
-    std::string presentationURL = getPresentationUrl();
-
-    log_debug("Creating UpnpXMLBuilder");
-    upnpXmlBuilder = std::make_shared<UpnpXMLBuilder>(context, getVirtualUrl(), presentationURL);
-    webXmlBuilder = std::make_shared<UpnpXMLBuilder>(context, getExternalUrl(), presentationURL);
+    log_debug("Creating UpnpXMLBuilders");
+    upnpXmlBuilder = std::make_shared<UpnpXMLBuilder>(context, getVirtualUrl());
+    webXmlBuilder = std::make_shared<UpnpXMLBuilder>(context, getExternalUrl());
+    auto devDescHdl = std::make_shared<DeviceDescriptionHandler>(content, webXmlBuilder, ip, port);
 
+    int activeUpnpDescription = config->getBoolOption(ConfigVal::UPNP_DYNAMIC_DESCRIPTION) ? 0 : 1;
     // register root device with the library
-    auto desc = upnpXmlBuilder->renderDeviceDescription();
-    std::ostringstream buf;
-    desc->print(buf, "", 0);
-    std::string deviceDescription = buf.str();
-    // log_debug("Device Description: {}", deviceDescription.c_str());
-
-    log_debug("Registering with UPnP...");
+    auto upnpDesc = std::vector<UpnpDesc> {
+        { UPNPREG_URL_DESC, fmt::format("http://{}{}", GrbNet::renderWebUri(ip, port), DEVICE_DESCRIPTION_PATH), -1 },
+        { UPNPREG_BUF_DESC, devDescHdl->renderDeviceDescription(ip, port, nullptr), 0 }
+    };
+    log_debug("Registering with UPnP... ({})", upnpDesc[activeUpnpDescription].desc);
     ret = UpnpRegisterRootDevice2(
-        UPNPREG_BUF_DESC,
-        deviceDescription.c_str(),
-        deviceDescription.length() + 1,
-        true,
+        upnpDesc[activeUpnpDescription].mode,
+        upnpDesc[activeUpnpDescription].desc.c_str(),
+        upnpDesc[activeUpnpDescription].len >= 0 ? upnpDesc[activeUpnpDescription].desc.length() + 1 : upnpDesc[activeUpnpDescription].len,
+        false,
         [](auto eventType, auto event, auto cookie) { return static_cast<Server*>(cookie)->handleUpnpRootDeviceEvent(eventType, event); },
         this,
         &rootDeviceHandle);
@@ -205,6 +205,7 @@ void Server::run()
         throw UpnpException(ret, "run: UpnpRegisterRootDevice2 failed");
     }
 
+    log_debug("Registering UPnP client...");
     ret = UpnpRegisterClient(
         [](auto eventType, auto event, auto cookie) { return static_cast<Server*>(cookie)->handleUpnpClientEvent(eventType, event); },
         this,
@@ -215,7 +216,7 @@ void Server::run()
 
     log_debug("Creating ContentDirectoryService");
     cds = std::make_unique<ContentDirectoryService>(context, upnpXmlBuilder, rootDeviceHandle,
-        config->getIntOption(CFG_SERVER_UPNP_TITLE_AND_DESC_STRING_LIMIT));
+        config->getIntOption(ConfigVal::SERVER_UPNP_TITLE_AND_DESC_STRING_LIMIT));
 
     log_debug("Creating ConnectionManagerService");
     cmgr = std::make_unique<ConnectionManagerService>(context, upnpXmlBuilder, rootDeviceHandle);
@@ -231,27 +232,10 @@ void Server::run()
         throw UpnpException(ret, fmt::format("run: UpnpSendAdvertisement {} failed", aliveAdvertisementInterval));
     }
 
-    if (config->getBoolOption(CFG_UPNP_LITERAL_HOST_REDIRECTION))
+#if !defined(USING_NPUPNP)
+    if (config->getBoolOption(ConfigVal::UPNP_LITERAL_HOST_REDIRECTION))
         UpnpSetAllowLiteralHostRedirection(1);
-
-    UpnpSetHostValidateCallback(
-        [](auto host, auto cookie) -> int {
-            auto hostStr = std::string(host);
-            for (auto&& name : static_cast<Server*>(cookie)->validHosts) {
-                if (hostStr.find(name) != std::string::npos) {
-                    return UPNP_E_SUCCESS;
-                } else if (name.find(hostStr) != std::string::npos) {
-                    return UPNP_E_SUCCESS;
-                }
-            }
-
-            log_warning("Rejected attempt to load host '{}' as it does not match configured virtualURL/externalURL: '{}'/'{}'. "
-                        "See https://docs.gerbera.io/en/stable/config-server.html#virtualurl",
-                host, static_cast<Server*>(cookie)->config->getOption(CFG_VIRTUAL_URL),
-                static_cast<Server*>(cookie)->config->getOption(CFG_EXTERNAL_URL));
-            return UPNP_E_BAD_HTTPMSG;
-        },
-        this);
+#endif
 
     std::string url = getVirtualUrl();
     writeBookmark(url);
@@ -269,6 +253,26 @@ void Server::run()
     if (!url.empty()) {
         validHosts.push_back(url);
     }
+
+    UpnpSetHostValidateCallback(
+        [](auto host, auto cookie) -> int {
+            auto hostStr = std::string(host);
+            for (auto&& name : static_cast<Server*>(cookie)->validHosts) {
+                if (hostStr.find(name) != std::string::npos) {
+                    return UPNP_E_SUCCESS;
+                }
+                if (name.find(hostStr) != std::string::npos) {
+                    return UPNP_E_SUCCESS;
+                }
+            }
+
+            log_warning("Rejected attempt to load host '{}' as it does not match configured virtualURL/externalURL: '{}'/'{}'. "
+                        "See https://docs.gerbera.io/en/stable/config-server.html#virtualurl",
+                host, static_cast<Server*>(cookie)->config->getOption(ConfigVal::VIRTUAL_URL),
+                static_cast<Server*>(cookie)->config->getOption(ConfigVal::EXTERNAL_URL));
+            return UPNP_E_BAD_HTTPMSG;
+        },
+        this);
 }
 
 int Server::startupInterface(const std::string& iface, in_port_t inPort)
@@ -306,48 +310,29 @@ int Server::startupInterface(const std::string& iface, in_port_t inPort)
     return ret;
 }
 
-std::string Server::getPresentationUrl() const
-{
-    std::string presentationURL = config->getOption(CFG_SERVER_PRESENTATION_URL);
-    if (presentationURL.empty()) {
-        presentationURL = fmt::format("http://{}/", GrbNet::renderWebUri(ip, port));
-    } else {
-        auto appendto = EnumOption<UrlAppendMode>::getEnumOption(config, CFG_SERVER_APPEND_PRESENTATION_URL_TO);
-        if (appendto == UrlAppendMode::ip) {
-            presentationURL = fmt::format("{}/{}", GrbNet::renderWebUri(ip, 0), presentationURL);
-        } else if (appendto == UrlAppendMode::port) {
-            presentationURL = fmt::format("{}/{}", GrbNet::renderWebUri(ip, port), presentationURL);
-        } // else appendto is none and we take the URL as it entered by user
-    }
-    if (!startswith(presentationURL, "http")) { // url does not start with http
-        presentationURL = fmt::format("http://{}", presentationURL);
-    }
-    return presentationURL;
-}
-
 void Server::writeBookmark(const std::string& addr)
 {
-    const std::string data = config->getBoolOption(CFG_SERVER_UI_ENABLED)
+    const std::string data = config->getBoolOption(ConfigVal::SERVER_UI_ENABLED)
         ? httpRedirectTo(addr)
         : httpRedirectTo(addr, "disabled.html");
 
-    fs::path path = config->getOption(CFG_SERVER_BOOKMARK_FILE);
+    fs::path path = config->getOption(ConfigVal::SERVER_BOOKMARK_FILE);
     log_debug("Writing bookmark file to: {}", path.c_str());
-    GrbFile(std::move(path)).writeTextFile(data);
+    GrbFile(path).writeTextFile(data);
 }
 
 void Server::emptyBookmark()
 {
     const std::string_view data = "<html><body><h1>Gerbera Media Server is not running.</h1><p>Please start it and try again.</p></body></html>";
 
-    fs::path path = config->getOption(CFG_SERVER_BOOKMARK_FILE);
+    fs::path path = config->getOption(ConfigVal::SERVER_BOOKMARK_FILE);
     log_debug("Clearing bookmark file at: {}", path.c_str());
-    GrbFile(std::move(path)).writeTextFile(data);
+    GrbFile(path).writeTextFile(data);
 }
 
 std::string Server::getVirtualUrl() const
 {
-    auto virtUrl = config->getOption(CFG_VIRTUAL_URL);
+    auto virtUrl = config->getOption(ConfigVal::VIRTUAL_URL);
     if (virtUrl.empty()) {
         virtUrl = GrbNet::renderWebUri(ip, port);
     }
@@ -361,9 +346,9 @@ std::string Server::getVirtualUrl() const
 
 std::string Server::getExternalUrl() const
 {
-    auto virtUrl = config->getOption(CFG_EXTERNAL_URL);
+    auto virtUrl = config->getOption(ConfigVal::EXTERNAL_URL);
     if (virtUrl.empty()) {
-        virtUrl = config->getOption(CFG_VIRTUAL_URL);
+        virtUrl = config->getOption(ConfigVal::VIRTUAL_URL);
         if (virtUrl.empty()) {
             virtUrl = GrbNet::renderWebUri(ip, port);
         }
@@ -399,20 +384,24 @@ void Server::shutdown()
     if (ret != UPNP_E_SUCCESS) {
         log_error("UpnpUnRegisterRootDevice failed ({})", ret);
     }
+    UpnpSetHostValidateCallback(nullptr, nullptr);
 
 #ifdef HAVE_CURL
     curl_global_cleanup();
 #endif
 
     log_debug("now calling upnp finish");
-    UpnpFinish();
+    ret = UpnpFinish();
+    if (ret != UPNP_E_SUCCESS) {
+        log_error("UpnpFinish failed ({})", ret);
+    }
 
     if (content) {
         content->shutdown();
-        content = nullptr;
+        content.reset();
     }
 
-    session_manager = nullptr;
+    sessionManager.reset();
 
     if (database->threadCleanupRequired()) {
         try {
@@ -421,15 +410,21 @@ void Server::shutdown()
         }
     }
     database->shutdown();
-    database = nullptr;
+    database.reset();
 
     timer->shutdown();
-    timer = nullptr;
+    timer.reset();
+    context.reset();
+    mime.reset();
+    clientManager.reset();
+    metadataService.reset();
+    upnpXmlBuilder.reset();
+    webXmlBuilder.reset();
+    cds.reset();
+    cmgr.reset();
+    mrreg.reset();
 
-    context = nullptr;
-
-    mime = nullptr;
-    clientManager = nullptr;
+    log_debug("Server down");
 }
 
 int Server::handleUpnpRootDeviceEvent(Upnp_EventType eventType, const void* event)
@@ -449,12 +444,12 @@ int Server::handleUpnpRootDeviceEvent(Upnp_EventType eventType, const void* even
     case UPNP_CONTROL_ACTION_REQUEST:
         log_debug("UPNP_CONTROL_ACTION_REQUEST");
         try {
-            auto request = ActionRequest(upnpXmlBuilder, clientManager, static_cast<UpnpActionRequest*>(const_cast<void*>(event)));
+            auto request = ActionRequest(upnpXmlBuilder, clientManager, const_cast<UpnpActionRequest*>(static_cast<const UpnpActionRequest*>(event)));
             routeActionRequest(request);
             request.update();
         } catch (const UpnpException& upnpE) {
             ret = upnpE.getErrorCode();
-            UpnpActionRequest_set_ErrCode(static_cast<UpnpActionRequest*>(const_cast<void*>(event)), ret);
+            UpnpActionRequest_set_ErrCode(const_cast<UpnpActionRequest*>(static_cast<const UpnpActionRequest*>(event)), ret);
         } catch (const std::runtime_error& e) {
             log_info("Exception: {}", e.what());
         }
@@ -463,7 +458,7 @@ int Server::handleUpnpRootDeviceEvent(Upnp_EventType eventType, const void* even
     case UPNP_EVENT_SUBSCRIPTION_REQUEST:
         log_debug("UPNP_EVENT_SUBSCRIPTION_REQUEST");
         try {
-            auto request = SubscriptionRequest(static_cast<UpnpSubscriptionRequest*>(const_cast<void*>(event)));
+            auto request = SubscriptionRequest(const_cast<UpnpSubscriptionRequest*>(static_cast<const UpnpSubscriptionRequest*>(event)));
             routeSubscriptionRequest(request);
         } catch (const UpnpException& upnpE) {
             log_warning("Subscription exception: {}", upnpE.what());
@@ -521,21 +516,23 @@ void Server::routeActionRequest(ActionRequest& request)
     // we need to match the serviceID to one of our services
     if (request.getServiceID() == UPNP_DESC_CM_SERVICE_ID) {
         // this call is for the lifetime stats service
-        // log_debug("request for connection manager service");
+        log_debug("request for connection manager service");
         cmgr->processActionRequest(request);
     } else if (request.getServiceID() == UPNP_DESC_CDS_SERVICE_ID) {
         // this call is for the toaster control service
-        // log_debug("routeActionRequest: request for content directory service");
+        log_debug("request for content directory service");
         if (!offline)
             cds->processActionRequest(request);
         else
             request.setErrorCode(UPNP_E_INVALID_ACTION);
 
     } else if (request.getServiceID() == UPNP_DESC_MRREG_SERVICE_ID) {
+        log_debug("request for MR Registrar service");
         mrreg->processActionRequest(request);
     } else {
         // cp is asking for a nonexistent service, or for a service
         // that does not support any actions
+        log_debug("Service {} does not exist or action not supported", request.getServiceID());
         throw UpnpException(UPNP_E_BAD_REQUEST, "Service does not exist or action not supported");
     }
 }
@@ -545,25 +542,27 @@ void Server::routeSubscriptionRequest(const SubscriptionRequest& request) const
     // make sure that the request is for our device
     if (request.getUDN() != serverUDN) {
         // not for us
-        log_debug("routeSubscriptionRequest: request not for this device: {} vs {}", request.getUDN(), serverUDN);
+        log_debug("request not for this device: {} vs {}", request.getUDN(), serverUDN);
         throw UpnpException(UPNP_E_BAD_REQUEST, "routeSubscriptionRequest: request not for this device");
     }
 
     // we need to match the serviceID to one of our services
     if (request.getServiceID() == UPNP_DESC_CDS_SERVICE_ID) {
         // this call is for the content directory service
-        // log_debug("routeSubscriptionRequest: request for content directory service");
+        log_debug("request for content directory service");
         if (!offline)
             cds->processSubscriptionRequest(request);
     } else if (request.getServiceID() == UPNP_DESC_CM_SERVICE_ID) {
         // this call is for the connection manager service
-        // log_debug("routeSubscriptionRequest: request for connection manager service");
+        log_debug("request for connection manager service");
         cmgr->processSubscriptionRequest(request);
     } else if (request.getServiceID() == UPNP_DESC_MRREG_SERVICE_ID) {
+        log_debug("request for MRRegistrar service");
         mrreg->processSubscriptionRequest(request);
     } else {
         // cp asks for a nonexistent service or for a service that
         // does not support subscriptions
+        log_debug("Service {} does not exist or action not supported", request.getServiceID());
         throw UpnpException(UPNP_E_BAD_REQUEST, "Service does not exist or subscriptions not supported");
     }
 }
@@ -580,11 +579,11 @@ std::unique_ptr<RequestHandler> Server::createRequestHandler(const char* filenam
     std::string link = URLUtils::urlUnescape(filename);
     log_debug("Filename: {}", filename);
 
-    if (startswith(link, fmt::format("/{}/{}", SERVER_VIRTUAL_DIR, CONTENT_MEDIA_HANDLER))) {
-        return std::make_unique<FileRequestHandler>(content, upnpXmlBuilder);
+    if (startswith(link, fmt::format("/{}", CONTENT_MEDIA_HANDLER))) {
+        return std::make_unique<FileRequestHandler>(content, upnpXmlBuilder, metadataService);
     }
 
-    if (startswith(link, fmt::format("/{}/{}", SERVER_VIRTUAL_DIR, CONTENT_UI_HANDLER))) {
+    if (startswith(link, fmt::format("/{}", CONTENT_UI_HANDLER))) {
         auto&& [path, parameters] = URLUtils::splitUrl(filename, URL_UI_PARAM_SEPARATOR);
 
         auto params = URLUtils::dictDecode(parameters);
@@ -595,13 +594,28 @@ std::unique_ptr<RequestHandler> Server::createRequestHandler(const char* filenam
         return Web::createWebRequestHandler(context, content, webXmlBuilder, rType);
     }
 
-    if (startswith(link, fmt::format("/{}/{}", SERVER_VIRTUAL_DIR, DEVICE_DESCRIPTION_PATH))) {
-        return std::make_unique<DeviceDescriptionHandler>(content, upnpXmlBuilder);
+    if (startswith(link, DEVICE_DESCRIPTION_PATH) || endswith(link, UPNP_DESC_DEVICE_DESCRIPTION)) {
+        return std::make_unique<DeviceDescriptionHandler>(content, upnpXmlBuilder, ip, port);
+    }
+
+    if (startswith(link, UPNP_DESC_SCPD_URL) || endswith(link, UPNP_DESC_CDS_SCPD_URL) || endswith(link, UPNP_DESC_CM_SCPD_URL) || endswith(link, UPNP_DESC_MRREG_SCPD_URL)) {
+        return std::make_unique<UpnpDescHandler>(content, upnpXmlBuilder);
+    }
+
+    if (link == "/" || startswith(link, "/index.html")
+        || startswith(link, "/favicon.ico")
+        || startswith(link, "/assets")
+        || startswith(link, "/vendor")
+        || startswith(link, "/js")
+        || startswith(link, "/css")
+        || startswith(link, "/icons")
+        || startswith(link, "/gerbera-config-")) {
+        return std::make_unique<UIHandler>(content, upnpXmlBuilder);
     }
 
 #if defined(HAVE_CURL)
-    if (startswith(link, fmt::format("/{}/{}", SERVER_VIRTUAL_DIR, CONTENT_ONLINE_HANDLER))) {
-        return std::make_unique<URLRequestHandler>(content);
+    if (startswith(link, fmt::format("/{}", CONTENT_ONLINE_HANDLER))) {
+        return std::make_unique<URLRequestHandler>(content, upnpXmlBuilder);
     }
 #endif
 
@@ -615,15 +629,15 @@ int Server::registerVirtualDirCallbacks()
         try {
             auto reqHandler = static_cast<const Server*>(cookie)->createRequestHandler(filename);
             std::string link = URLUtils::urlUnescape(filename);
-            reqHandler->getInfo(startswith(link, fmt::format("/{}/{}", SERVER_VIRTUAL_DIR, CONTENT_UI_HANDLER)) ? filename : link.c_str(), info);
+            *requestCookie = reqHandler->getInfo(startswith(link, fmt::format("/{}", CONTENT_UI_HANDLER)) ? filename : link.c_str(), info);
             return 0;
         } catch (const ServerShutdownException&) {
             return -1;
-        } catch (const SubtitlesNotFoundException& sex) {
-            log_warning("{}", sex.what());
+        } catch (const SubtitlesNotFoundException& stex) {
+            log_warning("SubtitlesNotFoundException: {}", stex.what());
             return -1;
         } catch (const std::runtime_error& e) {
-            log_error("{}", e.what());
+            log_error("Exception: {}", e.what());
             return -1;
         }
     });
@@ -633,19 +647,21 @@ int Server::registerVirtualDirCallbacks()
     log_debug("Setting UpnpVirtualDir OpenCallback");
     ret = UpnpVirtualDir_set_OpenCallback([](const char* filename, enum UpnpOpenFileMode mode, const void* cookie, const void* requestCookie) -> UpnpWebFileHandle {
         try {
+            auto client = requestCookie ? static_cast<const ClientInfo*>(requestCookie) : nullptr;
+            auto quirks = client ? std::make_shared<Quirks>(client) : nullptr;
             auto reqHandler = static_cast<const Server*>(cookie)->createRequestHandler(filename);
             std::string link = URLUtils::urlUnescape(filename);
-            auto ioHandler = reqHandler->open(startswith(link, fmt::format("/{}/{}", SERVER_VIRTUAL_DIR, CONTENT_UI_HANDLER)) ? filename : link.c_str(), mode);
+            auto ioHandler = reqHandler->open(startswith(link, fmt::format("/{}", CONTENT_UI_HANDLER)) ? filename : link.c_str(), quirks, mode);
             if (ioHandler) {
                 ioHandler->open(mode);
                 return ioHandler.release();
             }
             log_warning("No Handler for {}", link);
             return nullptr;
-        } catch (const ServerShutdownException& se) {
+        } catch (const ServerShutdownException&) {
             return nullptr;
-        } catch (const SubtitlesNotFoundException& sex) {
-            log_info("SubtitlesNotFoundException: {}", sex.what());
+        } catch (const SubtitlesNotFoundException& stex) {
+            log_warning("SubtitlesNotFoundException: {}", stex.what());
             return nullptr;
         } catch (const std::runtime_error& ex) {
             log_error("Exception: {}", ex.what());
