@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,19 +30,20 @@
 */
 
 /// \file url_request_handler.cc
-#define LOG_FAC log_facility_t::requests
+#define GRB_LOG_FAC GrbLogFacility::requests
 
 #ifdef HAVE_CURL
 #include "url_request_handler.h" // API
 
 #include "cds/cds_item.h"
-#include "config/config_manager.h"
-#include "content/content_manager.h"
-#include "database/database.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "config/result/transcoding.h"
+#include "content/content.h"
+#include "exceptions.h"
 #include "iohandler/curl_io_handler.h"
 #include "transcoding/transcode_dispatcher.h"
-#include "transcoding/transcoding.h"
-#include "upnp_xml.h"
+#include "upnp/xml_builder.h"
 #include "util/tools.h"
 #include "util/url.h"
 #include "util/url_utils.h"
@@ -51,7 +52,7 @@
 #include "content/onlineservice/online_service_helper.h"
 #endif
 
-void URLRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
+const struct ClientInfo* URLRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 {
     log_debug("start");
 
@@ -68,7 +69,7 @@ void URLRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
     std::string mimeType;
 
     if (!trProfile.empty()) {
-        auto tp = config->getTranscodingProfileListOption(CFG_TRANSCODING_PROFILE_LIST)
+        auto tp = config->getTranscodingProfileListOption(ConfigVal::TRANSCODING_PROFILE_LIST)
                       ->getByName(trProfile);
         if (!tp)
             throw_std_runtime_error("Transcoding requested but no profile matching the name {} found", trProfile);
@@ -79,7 +80,7 @@ void URLRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 #ifdef ONLINE_SERVICES
         std::string url = item->getFlag(OBJECT_FLAG_ONLINE_SERVICE) ? OnlineServiceHelper::resolveURL(item) : item->getLocation().string();
 #else
-        std::string url = item->getLocation();
+        std::string url = item->getLocation().string();
 #endif
         log_debug("Online content url: {}", url);
         try {
@@ -105,10 +106,11 @@ void URLRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 #endif
     log_debug("web_get_info(): end");
 
-    /// \todo transcoding for get_info
+    auto quirks = getQuirks(info);
+    return quirks ? quirks->getInfo() : nullptr;
 }
 
-std::unique_ptr<IOHandler> URLRequestHandler::open(const char* filename, enum UpnpOpenFileMode mode)
+std::unique_ptr<IOHandler> URLRequestHandler::open(const char* filename, const std::shared_ptr<Quirks>& quirks, enum UpnpOpenFileMode mode)
 {
     log_debug("start");
 
@@ -127,7 +129,7 @@ std::unique_ptr<IOHandler> URLRequestHandler::open(const char* filename, enum Up
 #ifdef ONLINE_SERVICES
     std::string url = item->getFlag(OBJECT_FLAG_ONLINE_SERVICE) ? OnlineServiceHelper::resolveURL(item) : item->getLocation().string();
 #else
-    std::string url = item->getLocation();
+    std::string url = item->getLocation().string();
 #endif
     log_debug("Online content url: {}", url);
 
@@ -139,7 +141,7 @@ std::unique_ptr<IOHandler> URLRequestHandler::open(const char* filename, enum Up
 
     std::string trProfile = getValueOrDefault(params, URL_PARAM_TRANSCODE_PROFILE_NAME);
     if (!trProfile.empty()) {
-        auto tp = config->getTranscodingProfileListOption(CFG_TRANSCODING_PROFILE_LIST)
+        auto tp = config->getTranscodingProfileListOption(ConfigVal::TRANSCODING_PROFILE_LIST)
                       ->getByName(trProfile);
         if (!tp)
             throw_std_runtime_error("Transcoding of file {} but no profile matching the name {} found", url, trProfile);
