@@ -3,7 +3,7 @@ Gerbera - https://gerbera.io/
 
     enum_iterator.h - this file is part of Gerbera.
 
-    Copyright (C) 2022-2023 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -61,7 +61,7 @@ public:
     }
 };
 
-template <class En, class Under = typename std::underlying_type<En>::type>
+template <class En, class Under = std::underlying_type_t<En>>
 class FlagEnum {
 public:
     FlagEnum()
