@@ -3,7 +3,7 @@ Gerbera - https://gerbera.io/
 
     grb_net.cc - this file is part of Gerbera.
 
-    Copyright (C) 2022-2023 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -19,9 +19,13 @@ Gerbera - https://gerbera.io/
 */
 
 /// \file grb_net.cc
-
+#define GRB_LOG_FAC GrbLogFacility::clients
 #include "grb_net.h" // API
 
+#include "exceptions.h"
+#include "logger.h"
+#include "tools.h"
+
 #include <arpa/inet.h>
 #include <ifaddrs.h>
 #include <netdb.h>
@@ -30,9 +34,6 @@ Gerbera - https://gerbera.io/
 #include <sys/sockio.h>
 #endif
 
-#include "common.h"
-#include "tools.h"
-
 #define M_SOCK_ADDR_IN_PTR(sa) reinterpret_cast<struct sockaddr_in*>(sa)
 #define M_SOCK_ADDR_IN_ADDR(sa) M_SOCK_ADDR_IN_PTR(sa)->sin_addr
 #define M_SOCK_ADDR_IN6_PTR(sa) reinterpret_cast<struct sockaddr_in6*>(sa)
@@ -234,7 +235,8 @@ std::string GrbNet::renderWebUri(std::string_view ip, in_port_t port)
             return fmt::format("[{}]", ip);
         }
         return fmt::format("{}", ip);
-    } else if (ip.find(':') != std::string_view::npos) {
+    }
+    if (ip.find(':') != std::string_view::npos) {
         return fmt::format("[{}]:{}", ip, port);
     }
     return fmt::format("{}:{}", ip, port);
