@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -36,6 +36,9 @@
 
 #include <array>
 #include <fmt/format.h>
+#if FMT_VERSION >= 100202
+#include <fmt/ranges.h>
+#endif
 #include <map>
 #include <spdlog/spdlog.h>
 #include <type_traits>
@@ -46,13 +49,13 @@
 #define log_info SPDLOG_INFO
 #define log_warning SPDLOG_WARN
 #define log_error SPDLOG_ERROR
-#define log_js SPDLOG_INFO
 
-enum class log_facility_t {
+enum class GrbLogFacility {
     thread,
     sqlite3,
     cds,
     server,
+    config,
     content,
     update,
     mysql,
@@ -63,11 +66,13 @@ enum class log_facility_t {
     web,
     layout,
     exif,
+    exiv2,
     transcoding,
     taglib,
     ffmpeg,
     wavpack,
     requests,
+    device,
     connmgr,
     mrregistrar,
     xml,
@@ -76,6 +81,7 @@ enum class log_facility_t {
     online,
     metadata,
     matroska,
+    curl,
 
     log_MAX,
 };
@@ -84,7 +90,7 @@ enum class log_facility_t {
 template <typename E>
 constexpr auto to_underlying(E e) noexcept
 {
-    return std::underlying_type_t<E>(e);
+    return static_cast<std::underlying_type_t<E>>(e);
 }
 #else
 using std::to_underlying;
@@ -96,47 +102,60 @@ public:
     static GrbLogger Logger;
 
     void init(int debugMode);
-    bool isDebugging(log_facility_t facility)
+    bool isDebugging(GrbLogFacility facility)
     {
         return GrbLogger::Logger.hasDebugging[to_underlying(facility)];
     }
-    static std::string_view mapFacility(log_facility_t facility)
+    static std::string_view mapFacility(GrbLogFacility facility)
     {
         return GrbLogger::facilities[facility];
     }
     static std::string printFacility(int facility);
-    static int remapFacility(const std::string& optValue);
+    static int remapFacility(const std::string& flag);
     static int makeFacility(const std::string& optValue);
 
-private:
-    static std::map<log_facility_t, std::string_view> facilities;
+    bool isDebugLogging() { return debug; }
+    void setDebugLogging(bool deb) { debug = deb; }
 
-    std::array<bool, to_underlying(log_facility_t::log_MAX)> hasDebugging {};
+private:
+    static std::map<GrbLogFacility, std::string_view> facilities;
+    bool debug;
+
+    std::array<bool, to_underlying(GrbLogFacility::log_MAX)> hasDebugging {};
 };
 
 #define log_facility(fac, ...) GrbLogger::Logger.isDebugging((fac)) ? log_faci(__VA_ARGS__) : log_dbg(__VA_ARGS__)
 
-#ifdef LOG_FAC
+#ifdef GRB_LOG_FAC
 
 #undef log_debug
-#define log_debug(...) log_facility(LOG_FAC, __VA_ARGS__)
+#define log_debug(...) log_facility(GRB_LOG_FAC, __VA_ARGS__)
 
 #endif
 
 #else
+class GrbLogger {
+public:
+    static GrbLogger Logger;
 
+    bool isDebugLogging() { return debug; }
+    void setDebugLogging(bool deb) { debug = deb; }
+
+private:
+    bool debug;
+};
 #define log_facility(fac, ...) log_debug(__VA_ARGS__)
 
 #endif
 
-#define log_threading(...) log_facility(log_facility_t::thread, __VA_ARGS__)
+#define log_threading(...) log_facility(GrbLogFacility::thread, __VA_ARGS__)
 
 #if FMT_VERSION >= 80100
 template <typename T>
 struct fmt::formatter<T, std::enable_if_t<std::is_enum_v<T>, char>>
     : formatter<std::underlying_type_t<T>> {
     template <typename FormatContext>
-    auto format(const T& value, FormatContext& ctx) -> decltype(ctx.out())
+    auto format(const T& value, FormatContext& ctx) const -> decltype(ctx.out())
     {
         return fmt::formatter<std::underlying_type_t<T>>::format(
             static_cast<std::underlying_type_t<T>>(value), ctx);
