@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,14 +30,14 @@
 */
 
 /// \file add_object.cc
-#define LOG_FAC log_facility_t::web
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
-#include "content/content_manager.h"
-#include "metadata/metadata_handler.h"
+#include "content/content.h"
+#include "exceptions.h"
 #include "util/tools.h"
 #include "util/xml_to_json.h"
 
@@ -61,7 +61,7 @@ std::shared_ptr<CdsItem> Web::AddObject::addItem(int parentID)
 
     std::string tmp = param("description");
     if (!tmp.empty()) {
-        item->addMetaData(M_DESCRIPTION, tmp);
+        item->addMetaData(MetadataFields::M_DESCRIPTION, tmp);
     }
 
     /// \todo is there a default setting? autoscan? import settings?
@@ -92,7 +92,7 @@ std::shared_ptr<CdsItemExternalURL> Web::AddObject::addUrl(int parentID, bool ad
 
     std::string tmp = param("description");
     if (!tmp.empty()) {
-        item->addMetaData(M_DESCRIPTION, tmp);
+        item->addMetaData(MetadataFields::M_DESCRIPTION, tmp);
     }
 
     /// \todo is there a default setting? autoscan? import settings?
@@ -114,8 +114,8 @@ std::shared_ptr<CdsItemExternalURL> Web::AddObject::addUrl(int parentID, bool ad
     } else
         protocolInfo = renderProtocolInfo(tmp);
 
-    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, CdsResource::Purpose::Content);
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, protocolInfo);
+    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, protocolInfo);
     item->addResource(resource);
 
     return item;
