@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,19 +30,20 @@
 */
 
 /// \file directories.cc
-#define LOG_FAC log_facility_t::web
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include <array>
-
-#include "content/autoscan.h"
-#include "content/content_manager.h"
-#include "database/database.h"
+#include "config/config_val.h"
+#include "config/result/autoscan.h"
+#include "content/content.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 #include "util/xml_to_json.h"
 
+#include <algorithm>
+#include <array>
+
 using dirInfo = std::pair<fs::path, bool>;
 
 void Web::Directories::process()
@@ -58,17 +59,17 @@ void Web::Directories::process()
 
     auto containers = root.append_child("containers");
     xml2Json->setArrayName(containers, "container");
-    xml2Json->setFieldType("title", "string");
+    xml2Json->setFieldType("title", FieldType::STRING);
     containers.append_attribute("parent_id") = parentID.c_str();
     if (!param("select_it").empty())
         containers.append_attribute("select_it") = param("select_it").c_str();
     containers.append_attribute("type") = "filesystem";
 
     // don't bother users with system directorties
-    auto&& excludesFullpath = config->getArrayOption(CFG_IMPORT_SYSTEM_DIRECTORIES);
-    auto&& includesFullpath = config->getArrayOption(CFG_IMPORT_VISIBLE_DIRECTORIES);
+    auto&& excludesFullpath = config->getArrayOption(ConfigVal::IMPORT_SYSTEM_DIRECTORIES);
+    auto&& includesFullpath = config->getArrayOption(ConfigVal::IMPORT_VISIBLE_DIRECTORIES);
     // don't bother users with special or config directorties
-    constexpr auto excludesDirname = std::array {
+    constexpr std::array excludesDirname {
         "lost+found",
     };
     bool excludeConfigDirs = true;
