@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,23 +30,23 @@
 */
 
 /// \file edit_load.cc
-#define LOG_FAC log_facility_t::web
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include <fmt/chrono.h>
-
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
 #include "database/database.h"
-#include "metadata/metadata_handler.h"
+#include "exceptions.h"
 #include "server.h"
-#include "upnp_xml.h"
+#include "upnp/clients.h"
+#include "upnp/xml_builder.h"
 #include "util/tools.h"
-#include "util/upnp_clients.h"
 #include "util/xml_to_json.h"
 
-Web::EditLoad::EditLoad(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder)
+#include <fmt/chrono.h>
+
+Web::EditLoad::EditLoad(const std::shared_ptr<Content>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder)
     : WebRequestHandler(content)
     , xmlBuilder(std::move(xmlBuilder))
 {
@@ -66,8 +66,8 @@ void Web::EditLoad::process()
 
     // set handling of json properties
     auto root = xmlDoc->document_element();
-    xml2Json->setFieldType("value", "string");
-    xml2Json->setFieldType("title", "string");
+    xml2Json->setFieldType("value", FieldType::STRING);
+    xml2Json->setFieldType("title", FieldType::STRING);
 
     // write object core info
     auto item = root.append_child("item");
@@ -110,7 +110,7 @@ void Web::EditLoad::process()
     // write metadata
     auto metaData = item.append_child("metadata");
     xml2Json->setArrayName(metaData, "metadata");
-    xml2Json->setFieldType("metavalue", "string");
+    xml2Json->setFieldType("metavalue", FieldType::STRING);
 
     for (auto&& [key, val] : obj->getMetaData()) {
         auto metaEntry = metaData.append_child("metadata");
@@ -122,7 +122,7 @@ void Web::EditLoad::process()
     // write auxdata
     auto auxData = item.append_child("auxdata");
     xml2Json->setArrayName(auxData, "auxdata");
-    xml2Json->setFieldType("auxvalue", "string");
+    xml2Json->setFieldType("auxvalue", FieldType::STRING);
 
     for (auto&& [key, val] : obj->getAuxData()) {
         auto auxEntry = auxData.append_child("auxdata");
@@ -133,8 +133,8 @@ void Web::EditLoad::process()
 
     auto resources = item.append_child("resources");
     xml2Json->setArrayName(resources, "resources");
-    xml2Json->setFieldType("resvalue", "string");
-    xml2Json->setFieldType("rawvalue", "string");
+    xml2Json->setFieldType("resvalue", FieldType::ENCSTRING);
+    xml2Json->setFieldType("rawvalue", FieldType::STRING);
 
     auto objItem = std::dynamic_pointer_cast<CdsItem>(obj);
 
@@ -147,18 +147,18 @@ void Web::EditLoad::process()
 
         resEntry = resources.append_child("resources");
         resEntry.append_attribute("resname") = "handlerType";
-        resEntry.append_attribute("resvalue") = MetadataHandler::mapContentHandler2String(resItem->getHandlerType()).c_str();
+        resEntry.append_attribute("resvalue") = EnumMapper::mapContentHandler2String(resItem->getHandlerType()).c_str();
         resEntry.append_attribute("editable") = false;
 
         resEntry = resources.append_child("resources");
         resEntry.append_attribute("resname") = "purpose";
-        resEntry.append_attribute("resvalue") = CdsResource::getPurposeDisplay(resItem->getPurpose()).c_str();
+        resEntry.append_attribute("resvalue") = EnumMapper::getPurposeDisplay(resItem->getPurpose()).c_str();
         resEntry.append_attribute("editable") = false;
 
         // write resource content
         if (objItem) {
             std::string url = xmlBuilder->renderResourceURL(*objItem, *resItem, {});
-            if (resItem->getPurpose() == CdsResource::Purpose::Thumbnail) {
+            if (resItem->getPurpose() == ResourcePurpose::Thumbnail) {
                 auto image = resources.append_child("resources");
                 image.append_attribute("resname") = "image";
                 image.append_attribute("resvalue") = url.c_str();
@@ -183,7 +183,7 @@ void Web::EditLoad::process()
             auto val = resItem->getAttribute(attr);
             if (!val.empty()) {
                 auto resEntry = resources.append_child("resources");
-                resEntry.append_attribute("resname") = CdsResource::getAttributeDisplay(attr).c_str();
+                resEntry.append_attribute("resname") = EnumMapper::getAttributeDisplay(attr).c_str();
                 auto aVal = resItem->getAttributeValue(attr);
                 if (aVal != val)
                     resEntry.append_attribute("rawvalue") = val.c_str();
@@ -203,7 +203,7 @@ void Web::EditLoad::process()
     // write item meta info
     if (obj->isItem()) {
         auto description = item.append_child("description");
-        description.append_attribute("value") = objItem->getMetaData(M_DESCRIPTION).c_str();
+        description.append_attribute("value") = objItem->getMetaData(MetadataFields::M_DESCRIPTION).c_str();
         description.append_attribute("editable") = true;
 
         auto location = item.append_child("location");
@@ -249,7 +249,7 @@ void Web::EditLoad::process()
 
         if (obj->isExternalItem()) {
             auto protocol = item.append_child("protocol");
-            protocol.append_attribute("value") = getProtocol(objItem->getResource(ContentHandler::DEFAULT)->getAttribute(CdsResource::Attribute::PROTOCOLINFO)).data();
+            protocol.append_attribute("value") = getProtocol(objItem->getResource(ContentHandler::DEFAULT)->getAttribute(ResourceAttribute::PROTOCOLINFO)).data();
             protocol.append_attribute("editable") = true;
         }
     }
