@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,11 +30,13 @@
 */
 
 /// \file files.cc
-#define LOG_FAC log_facility_t::web
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "database/database.h"
+#include "common.h"
+#include "config/config_val.h"
+#include "exceptions.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 #include "util/xml_to_json.h"
@@ -50,7 +52,7 @@ void Web::Files::process()
 
     auto files = root.append_child("files");
     xml2Json->setArrayName(files, "file");
-    xml2Json->setFieldType("filename", "string");
+    xml2Json->setFieldType("filename", FieldType::STRING);
     files.append_attribute("parent_id") = parentID.c_str();
     files.append_attribute("location") = path.c_str();
 
@@ -58,7 +60,7 @@ void Web::Files::process()
 
     std::error_code ec;
     std::map<std::string, fs::path> filesMap;
-    auto&& includesFullpath = config->getArrayOption(CFG_IMPORT_VISIBLE_DIRECTORIES);
+    auto&& includesFullpath = config->getArrayOption(ConfigVal::IMPORT_VISIBLE_DIRECTORIES);
 
     for (auto&& it : fs::directory_iterator(path, ec)) {
         const fs::path& filepath = it.path();
