@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,18 +30,19 @@
 */
 
 /// \file pages.cc
-#define LOG_FAC log_facility_t::web
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
 #include "config/config_setup.h"
+#include "exceptions.h"
 #include "util/xml_to_json.h"
 
 namespace Web {
 
 std::unique_ptr<WebRequestHandler> createWebRequestHandler(
     const std::shared_ptr<Context>& context,
-    const std::shared_ptr<ContentManager>& content,
+    const std::shared_ptr<Content>& content,
     const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
     const std::string& page)
 {
