@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -34,17 +34,20 @@
 #ifndef __WEB_PAGES_H__
 #define __WEB_PAGES_H__
 
-#include "common.h"
-#include "request_handler.h"
 #include "web_request_handler.h"
 
+#include <chrono>
+
 // forward declaration
+class AutoscanDirectory;
 class CdsItemExternalURL;
 class CdsItem;
 class Config;
+class ConfigSetup;
 class ConfigValue;
 class Database;
 class UpnpXMLBuilder;
+enum class ConfigVal;
 
 namespace Web {
 
@@ -55,9 +58,10 @@ class SessionManager;
 class Auth : public WebRequestHandler {
 protected:
     std::chrono::seconds timeout = std::chrono::seconds::zero();
+    bool accountsEnabled { false };
 
 public:
-    explicit Auth(const std::shared_ptr<ContentManager>& content);
+    explicit Auth(const std::shared_ptr<Content>& content);
     void process() override;
 };
 
@@ -67,7 +71,7 @@ protected:
     std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
 
 public:
-    explicit Containers(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
+    explicit Containers(const std::shared_ptr<Content>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     void process() override;
 };
 
@@ -93,7 +97,7 @@ protected:
     std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
 
 public:
-    explicit Items(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
+    explicit Items(const std::shared_ptr<Content>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     void process() override;
 };
 
@@ -119,7 +123,7 @@ protected:
     std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
 
 public:
-    explicit EditLoad(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
+    explicit EditLoad(const std::shared_ptr<Content>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     void process() override;
 };
 
@@ -184,7 +188,7 @@ public:
 /// \return the appropriate request handler.
 std::unique_ptr<WebRequestHandler> createWebRequestHandler(
     const std::shared_ptr<Context>& context,
-    const std::shared_ptr<ContentManager>& content,
+    const std::shared_ptr<Content>& content,
     const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
     const std::string& page);
 
@@ -201,14 +205,28 @@ class ConfigLoad : public WebRequestHandler {
 protected:
     std::vector<ConfigValue> dbEntries;
     std::map<std::string, std::string> allItems;
-    void createItem(pugi::xml_node& item, const std::string& name, config_option_t id, config_option_t aid, const std::shared_ptr<ConfigSetup>& cs = nullptr);
+    void createItem(pugi::xml_node& item, const std::string& name, ConfigVal id, ConfigVal aid, const std::shared_ptr<ConfigSetup>& cs = nullptr);
+
+    void writeDatabaseStatus(pugi::xml_node& values);
+    void writeSimpleProperties(pugi::xml_node& values);
+    void writeClientConfig(pugi::xml_node& values);
+    void writeImportTweaks(pugi::xml_node& values);
+    void writeDynamicContent(pugi::xml_node& values);
+    void writeBoxLayout(pugi::xml_node& values);
+    void writeTranscoding(pugi::xml_node& values);
+    void writeAutoscan(pugi::xml_node& values);
+    void writeDictionaries(pugi::xml_node& values);
+    void writeVectors(pugi::xml_node& values);
+    void writeArrays(pugi::xml_node& values);
+    void updateEntriesFromDatabase(pugi::xml_node& root, pugi::xml_node& values);
+
     template <typename T>
     static void setValue(pugi::xml_node& item, const T& value);
 
     static void addTypeMeta(pugi::xml_node& meta, const std::shared_ptr<ConfigSetup>& cs);
 
 public:
-    explicit ConfigLoad(const std::shared_ptr<ContentManager>& content);
+    explicit ConfigLoad(const std::shared_ptr<Content>& content);
     void process() override;
 };
 
@@ -218,7 +236,7 @@ protected:
     std::shared_ptr<Context> context;
 
 public:
-    explicit ConfigSave(std::shared_ptr<Context> context, const std::shared_ptr<ContentManager>& content);
+    explicit ConfigSave(std::shared_ptr<Context> context, const std::shared_ptr<Content>& content);
     void process() override;
 };
 
