@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,11 +30,13 @@
 */
 
 /// \file tasks.cc
-#define LOG_FAC log_facility_t::web
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/content_manager.h"
+#include "content/content.h"
+#include "exceptions.h"
+#include "util/generic_task.h"
 #include "util/xml_to_json.h"
 
 void Web::Tasks::process()
@@ -55,7 +57,7 @@ void Web::Tasks::process()
         }
     } else if (action == "cancel") {
         int taskID = intParam("task_id");
-        content->invalidateTask(taskID);
+        content->invalidateTask(taskID, TaskOwner::ContentManagerTask);
     } else
         throw_std_runtime_error("called with illegal action");
 }
