@@ -3,7 +3,7 @@
 
     test_autoscan_list.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -21,8 +21,8 @@
 */
 #include <gtest/gtest.h>
 
+#include "config/result/autoscan.h"
 #include "content/autoscan_list.h"
-#include "content/autoscan.h"
 
 TEST(AutoscanListTest, create)
 {
