@@ -3,7 +3,7 @@
 
   test_searchhandler.cc - this file is part of Gerbera.
 
-  Copyright (C) 2018-2023 Gerbera Contributors
+  Copyright (C) 2018-2024 Gerbera Contributors
 
   Gerbera is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
@@ -20,12 +20,12 @@
   $Id$
 */
 
+#include "database/search_handler.h"
+#include "metadata/metadata_enums.h"
+
 #include <gtest/gtest.h>
 #include <regex>
 
-#include "database/search_handler.h"
-#include "metadata/metadata_handler.h"
-
 using upVecUpST = std::unique_ptr<std::vector<std::unique_ptr<SearchToken>>>;
 static decltype(auto) getAllTokens(const std::string& input)
 {
@@ -57,9 +57,9 @@ static decltype(auto) getAllTokens(const std::string& input)
 
         if (expectedTokens.at(i).second != tokens->at(i)->getType())
             return ::testing::AssertionFailure() << "Token type " << i << ": expected ["
-                                                 << int(expectedTokens.at(i).second)
+                                                 << static_cast<int>(expectedTokens.at(i).second)
                                                  << "], got ["
-                                                 << int(tokens->at(i)->getType()) << "]";
+                                                 << static_cast<int>(tokens->at(i)->getType()) << "]";
     }
     return ::testing::AssertionSuccess();
 }
@@ -259,7 +259,7 @@ enum class TestCol {
     LastUpdated,
 };
 
-std::string OTN = MetadataHandler::getMetaFieldName(M_TRACKNUMBER);
+std::string OTN = MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER);
 
 class ParserTest : public ::testing::Test {
 public:
@@ -273,13 +273,13 @@ public:
 
     void SetUp() override
     {
-        otn = MetadataHandler::getMetaFieldName(M_TRACKNUMBER);
+        otn = MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER);
 
         testSortMap = {
             { "id", TestCol::Id },
             { UPNP_SEARCH_ID, TestCol::ItemId },
-            { MetadataHandler::getMetaFieldName(M_TRACKNUMBER), TestCol::Number1 },
-            { MetadataHandler::getMetaFieldName(M_TRACKNUMBER), TestCol::Number2 },
+            { MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER), TestCol::Number1 },
+            { MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER), TestCol::Number2 },
             { "double_name", TestCol::PropertyName },
             { META_NAME, TestCol::PropertyName },
             { META_NAME, TestCol::PropertyName2 },
@@ -301,7 +301,6 @@ public:
             { TestCol::LastUpdated, { "t", "last_updated" } },
         };
         columnMapper = std::make_shared<EnumColumnMapper<TestCol>>('_', '_', "t", "TestTable", testSortMap, testColMap);
-
     }
 
     void TearDown() override { }
@@ -310,7 +309,7 @@ public:
         const std::string& expectedOutput, const std::string& expectedRe = "")
     {
         try {
-            DefaultSQLEmitter emitter(columnMapper, columnMapper, columnMapper);
+            DefaultSQLEmitter emitter(columnMapper, columnMapper, columnMapper, columnMapper);
             auto parser = SearchParser(emitter, input);
             auto rootNode = parser.parse();
             if (!rootNode)
@@ -330,7 +329,7 @@ public:
         const std::string& expectedOutput)
     {
         try {
-            auto parser = SortParser(columnMapper, columnMapper, input);
+            auto parser = SortParser(columnMapper, columnMapper, columnMapper, input);
             std::string addColumns;
             std::string addJoin;
             auto output = parser.parse(addColumns, addJoin);
@@ -507,7 +506,7 @@ TEST_F(ParserTest, SortTrackNumber)
 {
     EXPECT_TRUE(OTN == "unknown" || OTN == "upnp:originalTrackNumber");
     EXPECT_EQ(otn, "upnp:originalTrackNumber");
-    EXPECT_EQ(MetadataHandler::getMetaFieldName(M_TRACKNUMBER), "upnp:originalTrackNumber");
+    EXPECT_EQ(MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER), "upnp:originalTrackNumber");
     EXPECT_TRUE(executeSortParserTest("+upnp:originalTrackNumber",
         "_t_._number1_ ASC, _t_._number2_ ASC"));
 }
