@@ -4,7 +4,7 @@ Gerbera - https://gerbera.io/
 
     test_server.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -19,17 +19,22 @@ Gerbera - https://gerbera.io/
     along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "common.h"
+#include "exceptions.h"
+#include "util/tools.h"
+
 #include <array>
-#include <common.h>
 #include <filesystem>
+#include <fmt/format.h>
+#if FMT_VERSION >= 100202
+#include <fmt/ranges.h>
+#endif
 #include <fstream>
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include <memory>
 #include <regex>
 
-#include "util/tools.h"
-
 namespace fs = std::filesystem;
 
 using ::testing::HasSubstr;
