@@ -4,7 +4,7 @@ Gerbera - https://gerbera.io/
 
     test_upnp_xml.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -24,11 +24,11 @@ Gerbera - https://gerbera.io/
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
 #include "common.h"
-#include "config/client_config.h"
+#include "config/result/client_config.h"
+#include "config/result/transcoding.h"
 #include "config/config_setup.h"
 #include "metadata/metadata_handler.h"
-#include "transcoding/transcoding.h"
-#include "upnp_xml.h"
+#include "upnp/xml_builder.h"
 #include "util/grb_net.h"
 #include "util/tools.h"
 
@@ -41,7 +41,7 @@ using ::testing::Return;
 class MyConfigMock final : public ConfigMock {
 public:
     MyConfigMock() { list = std::make_shared<ClientConfigList>(); }
-    std::shared_ptr<ClientConfigList> getClientConfigListOption(config_option_t option) const override { return list; }
+    std::shared_ptr<ClientConfigList> getClientConfigListOption(ConfigVal option) const override { return list; }
     std::shared_ptr<ClientConfigList> list;
 };
 
@@ -57,7 +57,7 @@ public:
 
         std::string virtualDir = "http://server";
         std::string presentationURl = "http://someurl/";
-        subject = std::make_shared<UpnpXMLBuilder>(context, virtualDir, presentationURl);
+        subject = std::make_shared<UpnpXMLBuilder>(context, virtualDir);
     }
 
     void TearDown() override
@@ -82,17 +82,17 @@ TEST_F(UpnpXmlTest, RenderObjectContainer)
     obj->setRestricted(false);
     obj->setTitle("Title");
     obj->setClass(UPNP_CLASS_MUSIC_ALBUM);
-    obj->addMetaData(M_ALBUMARTIST, "Creator");
-    obj->addMetaData(M_COMPOSER, "Composer");
-    obj->addMetaData(M_CONDUCTOR, "Conductor");
-    obj->addMetaData(M_ORCHESTRA, "Orchestra");
-    obj->addMetaData(M_UPNP_DATE, "2001-01-01");
-    obj->addMetaData(M_DATE, "2022-04-01T00:00:00");
+    obj->addMetaData(MetadataFields::M_ALBUMARTIST, "Creator");
+    obj->addMetaData(MetadataFields::M_COMPOSER, "Composer");
+    obj->addMetaData(MetadataFields::M_CONDUCTOR, "Conductor");
+    obj->addMetaData(MetadataFields::M_ORCHESTRA, "Orchestra");
+    obj->addMetaData(MetadataFields::M_UPNP_DATE, "2001-01-01");
+    obj->addMetaData(MetadataFields::M_DATE, "2022-04-01T00:00:00");
 
     // albumArtURI
-    auto resource = std::make_shared<CdsResource>(ContentHandler::CONTAINERART, CdsResource::Purpose::Thumbnail);
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo("jpg"));
-    resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, "/home/resource/cover.jpg");
+    auto resource = std::make_shared<CdsResource>(ContentHandler::CONTAINERART, ResourcePurpose::Thumbnail);
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo("jpg"));
+    resource->addAttribute(ResourceAttribute::RESOURCE_FILE, "/home/resource/cover.jpg");
     obj->addResource(resource);
 
     std::ostringstream expectedXml;
@@ -108,7 +108,7 @@ TEST_F(UpnpXmlTest, RenderObjectContainer)
     expectedXml << "<upnp:conductor>Conductor</upnp:conductor>\n";
     expectedXml << "<upnp:date>2001-01-01</upnp:date>\n";
     expectedXml << "<upnp:orchestra>Orchestra</upnp:orchestra>\n";
-    expectedXml << "<upnp:albumArtURI>http://server/content/media/object_id/1/res_id/0/ext/file.jpg</upnp:albumArtURI>\n";
+    expectedXml << "<upnp:albumArtURI>http://server/media/object_id/1/res_id/0/ext/file.jpg</upnp:albumArtURI>\n";
     expectedXml << "</container>\n";
     expectedXml << "</DIDL-Lite>\n";
 
@@ -131,10 +131,10 @@ TEST_F(UpnpXmlTest, RenderObjectItem)
     obj->setRestricted(false);
     obj->setTitle("Title");
     obj->setClass(UPNP_CLASS_MUSIC_TRACK);
-    obj->addMetaData(M_DESCRIPTION, "Description");
-    obj->addMetaData(M_ALBUM, "Album");
-    obj->addMetaData(M_TRACKNUMBER, "10");
-    obj->addMetaData(M_DATE, "2022-04-01T00:00:00");
+    obj->addMetaData(MetadataFields::M_DESCRIPTION, "Description");
+    obj->addMetaData(MetadataFields::M_ALBUM, "Album");
+    obj->addMetaData(MetadataFields::M_TRACKNUMBER, "10");
+    obj->addMetaData(MetadataFields::M_DATE, "2022-04-01T00:00:00");
 
     std::ostringstream expectedXml;
     expectedXml << "<DIDL-Lite>\n";
@@ -148,9 +148,9 @@ TEST_F(UpnpXmlTest, RenderObjectItem)
     expectedXml << "</item>\n";
     expectedXml << "</DIDL-Lite>\n";
 
-    EXPECT_CALL(*config, getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP))
+    EXPECT_CALL(*config, getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP))
         .WillRepeatedly(Return(" / "));
-    log_warning(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
+    log_warning(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
     EXPECT_CALL(*config, getTranscodingProfileListOption(_))
         .WillRepeatedly(Return(std::make_shared<TranscodingProfileList>()));
 
@@ -173,10 +173,10 @@ TEST_F(UpnpXmlTest, RenderObjectItemWithEscapes)
     obj->setRestricted(false);
     obj->setTitle("Title 'n Ticks");
     obj->setClass(UPNP_CLASS_MUSIC_TRACK);
-    obj->addMetaData(M_DESCRIPTION, "Description < Input");
-    obj->addMetaData(M_ALBUM, "Album & Test");
-    obj->addMetaData(M_TRACKNUMBER, "10");
-    obj->addMetaData(M_DATE, "2022-04-01T00:00:00");
+    obj->addMetaData(MetadataFields::M_DESCRIPTION, "Description < Input");
+    obj->addMetaData(MetadataFields::M_ALBUM, "Album & Test");
+    obj->addMetaData(MetadataFields::M_TRACKNUMBER, "10");
+    obj->addMetaData(MetadataFields::M_DATE, "2022-04-01T00:00:00");
 
     auto clientConfig = std::make_shared<MyConfigMock>();
     auto clientManager = std::make_shared<ClientManager>(clientConfig, database);
@@ -195,9 +195,9 @@ TEST_F(UpnpXmlTest, RenderObjectItemWithEscapes)
     expectedXml << "</item>\n";
     expectedXml << "</DIDL-Lite>\n";
 
-    EXPECT_CALL(*config, getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP))
+    EXPECT_CALL(*config, getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP))
         .WillRepeatedly(Return(" / "));
-    log_warning(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
+    log_warning(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
     EXPECT_CALL(*config, getTranscodingProfileListOption(_))
         .WillRepeatedly(Return(std::make_shared<TranscodingProfileList>()));
 
@@ -220,10 +220,10 @@ TEST_F(UpnpXmlTest, RenderObjectItemWithStrictXmlQuirks)
     obj->setRestricted(false);
     obj->setTitle("Title 'n Ticks");
     obj->setClass(UPNP_CLASS_MUSIC_TRACK);
-    obj->addMetaData(M_DESCRIPTION, "Description < Input");
-    obj->addMetaData(M_ALBUM, "Album & Test");
-    obj->addMetaData(M_TRACKNUMBER, "10");
-    obj->addMetaData(M_DATE, "2022-04-01T00:00:00");
+    obj->addMetaData(MetadataFields::M_DESCRIPTION, "Description < Input");
+    obj->addMetaData(MetadataFields::M_ALBUM, "Album & Test");
+    obj->addMetaData(MetadataFields::M_TRACKNUMBER, "10");
+    obj->addMetaData(MetadataFields::M_DATE, "2022-04-01T00:00:00");
 
     auto clientConfig = std::make_shared<MyConfigMock>();
     auto clientManager = std::make_shared<ClientManager>(clientConfig, database);
@@ -242,9 +242,9 @@ TEST_F(UpnpXmlTest, RenderObjectItemWithStrictXmlQuirks)
     expectedXml << "</item>\n";
     expectedXml << "</DIDL-Lite>\n";
 
-    EXPECT_CALL(*config, getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP))
+    EXPECT_CALL(*config, getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP))
         .WillRepeatedly(Return(" / "));
-    log_warning(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
+    log_warning(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
     EXPECT_CALL(*config, getTranscodingProfileListOption(_))
         .WillRepeatedly(Return(std::make_shared<TranscodingProfileList>()));
 
@@ -267,38 +267,38 @@ TEST_F(UpnpXmlTest, RenderObjectItemWithResources)
     obj->setRestricted(false);
     obj->setTitle("Title");
     obj->setClass(UPNP_CLASS_MUSIC_TRACK);
-    obj->addMetaData(M_DESCRIPTION, "Description");
-    obj->addMetaData(M_ALBUM, "Album");
-    obj->addMetaData(M_TRACKNUMBER, "7");
-    obj->addMetaData(M_UPNP_DATE, "2002-01-01");
-    obj->addMetaData(M_DATE, "2022-04-01T00:00:00");
+    obj->addMetaData(MetadataFields::M_DESCRIPTION, "Description");
+    obj->addMetaData(MetadataFields::M_ALBUM, "Album");
+    obj->addMetaData(MetadataFields::M_TRACKNUMBER, "7");
+    obj->addMetaData(MetadataFields::M_UPNP_DATE, "2002-01-01");
+    obj->addMetaData(MetadataFields::M_DATE, "2022-04-01T00:00:00");
 
-    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, CdsResource::Purpose::Content);
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, "http-get:*:audio/mpeg:*");
-    resource->addAttribute(CdsResource::Attribute::BITRATE, "16044");
-    resource->addAttribute(CdsResource::Attribute::DURATION, "123456");
-    resource->addAttribute(CdsResource::Attribute::NRAUDIOCHANNELS, "2");
-    resource->addAttribute(CdsResource::Attribute::SIZE, "4711");
+    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, "http-get:*:audio/mpeg:*");
+    resource->addAttribute(ResourceAttribute::BITRATE, "16044");
+    resource->addAttribute(ResourceAttribute::DURATION, "123456");
+    resource->addAttribute(ResourceAttribute::NRAUDIOCHANNELS, "2");
+    resource->addAttribute(ResourceAttribute::SIZE, "4711");
     obj->addResource(resource);
 
-    resource = std::make_shared<CdsResource>(ContentHandler::SUBTITLE, CdsResource::Purpose::Subtitle);
+    resource = std::make_shared<CdsResource>(ContentHandler::SUBTITLE, ResourcePurpose::Subtitle);
     std::string type = "srt";
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo("text/" + type));
-    resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, "/home/resource/subtitle.srt");
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo("text/" + type));
+    resource->addAttribute(ResourceAttribute::RESOURCE_FILE, "/home/resource/subtitle.srt");
     resource->addParameter("type", type);
     obj->addResource(resource);
 
-    resource = std::make_shared<CdsResource>(ContentHandler::SUBTITLE, CdsResource::Purpose::Subtitle);
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo("text/" + type));
-    resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, "/home/resource/subtitle.srt");
-    resource->addAttribute(CdsResource::Attribute::LANGUAGE, "fr");
+    resource = std::make_shared<CdsResource>(ContentHandler::SUBTITLE, ResourcePurpose::Subtitle);
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo("text/" + type));
+    resource->addAttribute(ResourceAttribute::RESOURCE_FILE, "/home/resource/subtitle.srt");
+    resource->addAttribute(ResourceAttribute::LANGUAGE, "fr");
     resource->addParameter("type", type);
     obj->addResource(resource);
 
-    resource = std::make_shared<CdsResource>(ContentHandler::FANART, CdsResource::Purpose::Thumbnail);
-    resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo("jpg"));
-    resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, "/home/resource/cover.jpg");
-    resource->addAttribute(CdsResource::Attribute::RESOLUTION, "200x200");
+    resource = std::make_shared<CdsResource>(ContentHandler::FANART, ResourcePurpose::Thumbnail);
+    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo("jpg"));
+    resource->addAttribute(ResourceAttribute::RESOURCE_FILE, "/home/resource/cover.jpg");
+    resource->addAttribute(ResourceAttribute::RESOLUTION, "200x200");
     obj->addResource(resource);
 
     std::ostringstream expectedXml;
@@ -311,16 +311,16 @@ TEST_F(UpnpXmlTest, RenderObjectItemWithResources)
     expectedXml << "<upnp:album>Album</upnp:album>\n";
     expectedXml << "<upnp:date>2002-01-01</upnp:date>\n";
     expectedXml << "<upnp:originalTrackNumber>7</upnp:originalTrackNumber>\n";
-    expectedXml << "<upnp:albumArtURI xmlns:dlna=\"urn:schemas-dlna-org:metadata-1-0\" dlna:profileID=\"JPEG_TN\">http://server/content/media/object_id/42/res_id/3/ext/file.jpg</upnp:albumArtURI>\n";
-    expectedXml << "<sec:CaptionInfoEx protocolInfo=\"http-get:*:text/srt:*\" sec:type=\"srt\">http://server/content/media/object_id/42/res_id/1/type/srt/ext/file.srt</sec:CaptionInfoEx>\n";
-    expectedXml << "<res size=\"4711\" duration=\"123456\" bitrate=\"16044\" nrAudioChannels=\"2\" protocolInfo=\"http-get:*:audio/mpeg:DLNA.ORG_PN=MP3;DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=01700000000000000000000000000000\">http://server/content/media/object_id/42/res_id/0/group/default/ext/file.mp3</res>\n";
-    expectedXml << "<res protocolInfo=\"http-get:*:text/srt:DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=00d00000000000000000000000000000\">http://server/content/media/object_id/42/res_id/1/type/srt/ext/file.srt</res>\n";
-    expectedXml << "<res protocolInfo=\"http-get:*:text/srt:DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=00d00000000000000000000000000000\" dc:language=\"fr\">http://server/content/media/object_id/42/res_id/2/type/srt/ext/file.fr.srt</res>\n";
-    expectedXml << "<res resolution=\"200x200\" protocolInfo=\"http-get:*:jpg:DLNA.ORG_OP=01;DLNA.ORG_CI=0\">http://server/content/media/object_id/42/res_id/3/ext/file.jpg</res>\n";
+    expectedXml << "<upnp:albumArtURI xmlns:dlna=\"urn:schemas-dlna-org:metadata-1-0\" dlna:profileID=\"JPEG_TN\">http://server/media/object_id/42/res_id/3/ext/file.jpg</upnp:albumArtURI>\n";
+    expectedXml << "<sec:CaptionInfoEx protocolInfo=\"http-get:*:text/srt:*\" sec:type=\"srt\">http://server/media/object_id/42/res_id/1/type/srt/ext/file.srt</sec:CaptionInfoEx>\n";
+    expectedXml << "<res size=\"4711\" duration=\"123456\" bitrate=\"16044\" nrAudioChannels=\"2\" protocolInfo=\"http-get:*:audio/mpeg:DLNA.ORG_PN=MP3;DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=01700000000000000000000000000000\">http://server/media/object_id/42/res_id/0/group/default/ext/file.mp3</res>\n";
+    expectedXml << "<res protocolInfo=\"http-get:*:text/srt:DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=00d00000000000000000000000000000\">http://server/media/object_id/42/res_id/1/type/srt/ext/file.srt</res>\n";
+    expectedXml << "<res protocolInfo=\"http-get:*:text/srt:DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=00d00000000000000000000000000000\" dc:language=\"fr\">http://server/media/object_id/42/res_id/2/type/srt/ext/file.fr.srt</res>\n";
+    expectedXml << "<res resolution=\"200x200\" protocolInfo=\"http-get:*:jpg:DLNA.ORG_OP=01;DLNA.ORG_CI=0\">http://server/media/object_id/42/res_id/3/ext/file.jpg</res>\n";
     expectedXml << "</item>\n";
     expectedXml << "</DIDL-Lite>\n";
 
-    EXPECT_CALL(*config, getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP))
+    EXPECT_CALL(*config, getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP))
         .WillRepeatedly(Return(" / "));
     EXPECT_CALL(*config, getTranscodingProfileListOption(_))
         .WillRepeatedly(Return(std::make_shared<TranscodingProfileList>()));
@@ -383,7 +383,7 @@ TEST_F(UpnpXmlTest, FirstResourceAddsLocalResourceIdToExternalUrlWhenOnlineWithP
     std::string result = subject->getFirstResourcePath(item);
 
     EXPECT_NE(result, "");
-    EXPECT_STREQ(result.c_str(), "http://server/content/online/object_id/12345/res_id/0");
+    EXPECT_STREQ(result.c_str(), "http://server/online/object_id/12345/res_id/0");
 }
 
 TEST_F(UpnpXmlTest, FirstResourceAddsLocalResourceIdToItem)
@@ -397,5 +397,5 @@ TEST_F(UpnpXmlTest, FirstResourceAddsLocalResourceIdToItem)
     std::string result = subject->getFirstResourcePath(item);
 
     EXPECT_NE(result, "");
-    EXPECT_STREQ(result.c_str(), "http://server/content/media/object_id/12345/res_id/0");
+    EXPECT_STREQ(result.c_str(), "http://server/media/object_id/12345/res_id/0");
 }
