@@ -3,7 +3,7 @@
 
     duk_helper.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -24,8 +24,11 @@
 #include "duk_helper.h"
 #include <duk_config.h>
 #include <duktape.h>
-#include <fmt/core.h>
 #include <fmt/format.h>
+#if FMT_VERSION >= 100202
+#include <fmt/ranges.h>
+#endif
+#include <iostream>
 #include <regex>
 
 std::vector<std::string> DukTestHelper::arrayToVector(duk_context* ctx, duk_idx_t idx)
@@ -186,4 +189,13 @@ void DukTestHelper::createObject(duk_context* ctx, const std::map<std::string, s
         duk_put_prop_string(ctx, -2, "metaData");
     }
 }
+
+void DukTestHelper::printError(duk_context* ctx, const std::string& message, const std::string& item)
+{
+#if DUK_VERSION > 20399
+    std::cerr << message << item << ": " << duk_safe_to_stacktrace(ctx, -1) << std::endl;
+#else
+    std::cerr << message << item << ": " << duk_safe_to_string(ctx, -1) << std::endl;
+#endif
+}
 #endif //HAVE_JS
