@@ -4,7 +4,7 @@ Gerbera - https://gerbera.io/
 
     script_test_fixture.cc - this file is part of Gerbera.
 
-    Copyright (C) 2020-2023 Gerbera Contributors
+    Copyright (C) 2020-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -27,10 +27,10 @@ Gerbera - https://gerbera.io/
 #include <gtest/gtest.h>
 
 #include "cds/cds_objects.h"
-#include "content/autoscan.h"
+#include "config/result/autoscan.h"
 #include "content/onlineservice/atrailers_content_handler.h"
 #include "content/scripting/script_names.h"
-#include "metadata/metadata_handler.h"
+#include "metadata/metadata_enums.h"
 #include "util/grb_fs.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
@@ -39,6 +39,8 @@ Gerbera - https://gerbera.io/
 #include "duk_helper.h"
 #include "script_test_fixture.h"
 
+std::unique_ptr<CommonScriptMock> CommonScriptTestFixture::commonScriptMock;
+
 void ScriptTestFixture::SetUp()
 {
     ctx = duk_create_heap(nullptr, nullptr, nullptr, nullptr, nullptr);
@@ -50,7 +52,10 @@ void ScriptTestFixture::SetUp()
         std::string scriptContent = GrbFile(scriptFile).readTextFile();
         duk_push_thread_stash(ctx, ctx);
         duk_push_string(ctx, scriptFile.c_str());
-        duk_pcompile_lstring_filename(ctx, 0, scriptContent.c_str(), scriptContent.length());
+        if (duk_pcompile_lstring_filename(ctx, 0, scriptContent.c_str(), scriptContent.length()) != DUK_EXEC_SUCCESS) {
+            DukTestHelper::printError(ctx, "Failed to load script ", scriptFile);
+            return;
+        }
         duk_put_global_string(ctx, "script_under_test");
         duk_pop(ctx);
     }
@@ -67,10 +72,14 @@ void ScriptTestFixture::loadCommon(duk_context* ctx) const
         fs::path commonScript = fs::path(SCRIPTS_DIR) / "js" / "common.js";
         std::string script = GrbFile(commonScript).readTextFile();
         duk_push_string(ctx, commonScript.c_str());
-        duk_pcompile_lstring_filename(ctx, 0, script.c_str(), script.length());
 
+        if (duk_pcompile_lstring_filename(ctx, 0, script.c_str(), script.length())) {
+            DukTestHelper::printError(ctx, "Failed to load script ", commonScript);
+            return;
+        }
         if (duk_pcall(ctx, 0) != DUK_EXEC_SUCCESS) {
-            std::cerr << "Failed to execute script: " << duk_safe_to_string(ctx, -1) << std::endl;
+            DukTestHelper::printError(ctx, "Failed to execute script ", commonScript);
+            return;
         }
         duk_pop(ctx); // commonScript
     } else if (scriptName != "common.js") {
@@ -81,15 +90,17 @@ void ScriptTestFixture::loadCommon(duk_context* ctx) const
                 try {
                     std::string script = GrbFile(entryPath).readTextFile();
                     duk_push_string(ctx, entryPath.c_str());
-                    duk_pcompile_lstring_filename(ctx, 0, script.c_str(), script.length());
-
+                    if (duk_pcompile_lstring_filename(ctx, 0, script.c_str(), script.length())) {
+                        DukTestHelper::printError(ctx, "Failed to load script ", entryPath);
+                        return;
+                    }
                     if (duk_pcall(ctx, 0) != DUK_EXEC_SUCCESS) {
-                        std::cerr << "Failed to execute script: " << duk_safe_to_string(ctx, -1) << std::endl;
+                        DukTestHelper::printError(ctx, "Failed to execute script ", entryPath);
+                        return;
                     }
                     duk_pop(ctx); // entryPath
-                    log_debug("Loaded {}", entryPath.c_str());
                 } catch (const std::runtime_error& e) {
-                    log_error("Unable to load {}: {}", entryPath.c_str(), e.what());
+                    std::cerr << "Unable to load  " << entryPath << ": " << e.what() << std::endl;
                 }
             }
         }
@@ -283,11 +294,11 @@ duk_ret_t ScriptTestFixture::dukMockMetafile(duk_context* ctx, const std::string
         auto dukArray = duk_push_array(ctx);
         duk_push_string(ctx, fmt::to_string(10).c_str());
         duk_put_prop_index(ctx, dukArray, 0);
-        duk_put_prop_string(ctx, -2, MetadataHandler::getMetaFieldName(M_TRACKNUMBER).c_str());
+        duk_put_prop_string(ctx, -2, MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER).c_str());
         dukArray = duk_push_array(ctx);
         duk_push_string(ctx, fmt::to_string(0).c_str());
         duk_put_prop_index(ctx, dukArray, 0);
-        duk_put_prop_string(ctx, -2, MetadataHandler::getMetaFieldName(M_PARTNUMBER).c_str());
+        duk_put_prop_string(ctx, -2, MetaEnumMapper::getMetaFieldName(MetadataFields::M_PARTNUMBER).c_str());
         dukArray = duk_push_array(ctx);
         duk_push_string(ctx, "done");
         duk_put_prop_index(ctx, dukArray, 0);
@@ -318,11 +329,11 @@ void ScriptTestFixture::dukMockMetafile(duk_context* ctx, const std::map<std::st
         auto dukArray = duk_push_array(ctx);
         duk_push_string(ctx, fmt::to_string(10).c_str());
         duk_put_prop_index(ctx, dukArray, 0);
-        duk_put_prop_string(ctx, -2, MetadataHandler::getMetaFieldName(M_TRACKNUMBER).c_str());
+        duk_put_prop_string(ctx, -2, MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER).c_str());
         dukArray = duk_push_array(ctx);
         duk_push_string(ctx, fmt::to_string(0).c_str());
         duk_put_prop_index(ctx, dukArray, 0);
-        duk_put_prop_string(ctx, -2, MetadataHandler::getMetaFieldName(M_PARTNUMBER).c_str());
+        duk_put_prop_string(ctx, -2, MetaEnumMapper::getMetaFieldName(MetadataFields::M_PARTNUMBER).c_str());
         dukArray = duk_push_array(ctx);
         duk_push_string(ctx, "done");
         duk_put_prop_index(ctx, dukArray, 0);
@@ -334,7 +345,7 @@ void ScriptTestFixture::dukMockMetafile(duk_context* ctx, const std::map<std::st
 void ScriptTestFixture::addGlobalFunctions(duk_context* ctx, const duk_function_list_entry* funcs, const std::map<std::string_view, std::string_view>& config, const std::vector<boxConfig>& boxDefaults)
 {
 
-    for (auto&&[meta,str]: MetadataHandler::mt_keys) {
+    for (auto&&[meta,str]: MetaEnumMapper::mt_keys) {
         duk_push_lstring(ctx, str.data(), str.size());
         auto sym = mt_names.at(meta);
         duk_put_global_lstring(ctx, sym.data(), sym.size());
@@ -357,6 +368,11 @@ void ScriptTestFixture::addGlobalFunctions(duk_context* ctx, const duk_function_
         duk_put_global_lstring(ctx, sym.data(), sym.size());
     }
 
+    for (auto&& [field, sym] : boxKeyNames) {
+        duk_push_lstring(ctx, sym.data(), sym.length());
+        duk_put_global_lstring(ctx, field.data(), field.length());
+    }
+
     if (config.empty()) {
         addConfig(ctx, { { "/import/scripting/virtual-layout/attribute::audio-layout", audioLayout }, { "/import/scripting/virtual-layout/structured-layout/attribute::skip-chars", "" } }, boxDefaults);
     } else {
@@ -415,11 +431,7 @@ void ScriptTestFixture::executeScript(duk_context* ctx)
     duk_get_global_string(ctx, "script_under_test");
     if (duk_is_function(ctx, -1)) {
         if (duk_pcall(ctx, 0) != DUK_EXEC_SUCCESS) {
-#if DUK_VERSION > 20399
-            std::cerr << "Failed to execute script: " << duk_safe_to_stacktrace(ctx, -1) << std::endl;
-#else
-            std::cerr << "Failed to execute script: " << duk_safe_to_string(ctx, -1) << std::endl;
-#endif
+            DukTestHelper::printError(ctx, "Failed to execute script ", scriptName);
         }
         duk_pop(ctx); // script_under_test
     }
@@ -433,7 +445,7 @@ void ScriptTestFixture::callFunction(duk_context* ctx, void(dukMockFunction)(duk
     std::string containerType;
     // Push function onto stack
     if (!duk_get_global_string(ctx, functionName.c_str()) || !duk_is_function(ctx, -1)) {
-        std::cerr << "javascript function not found: " << functionName << std::endl;
+        std::cerr << "javascript function not found: " << functionName << '\n';
         duk_pop(ctx);
         return;
     }
@@ -460,14 +472,10 @@ void ScriptTestFixture::callFunction(duk_context* ctx, void(dukMockFunction)(duk
     duk_push_sprintf(ctx, "%s", containerType.c_str());
     narg++;
 
-    if (duk_pcall(ctx, (duk_idx_t)narg) != DUK_EXEC_SUCCESS) {
+    if (duk_pcall(ctx, static_cast<duk_idx_t>(narg)) != DUK_EXEC_SUCCESS) {
         // Note: The invoked function will be blamed for execution errors, not the actual offending line of code
         // https://github.com/svaarala/duktape/blob/master/doc/error-objects.rst
-#if DUK_VERSION > 20399
-        std::cerr << "javascript runtime error: " << functionName << " " << duk_safe_to_stacktrace(ctx, -1) << std::endl;
-#else
-        std::cerr << "javascript runtime error: " << functionName << " " << duk_safe_to_string(ctx, -1) << std::endl;
-#endif
+        DukTestHelper::printError(ctx, "JavaScript runtime error ", functionName);
         duk_pop(ctx);
         return;
     }
@@ -486,6 +494,29 @@ std::vector<std::string> ScriptTestFixture::createContainerChain(duk_context* ct
     return array;
 }
 
+std::pair<std::string, int> ScriptTestFixture::getLastPath2(duk_context* ctx)
+{
+    std::string inputPath = duk_to_string(ctx, 0);
+    int length = duk_to_int(ctx, 1);
+    length = length <= 0 ? 1 : length;
+    fs::path path = inputPath;
+    path = path.remove_filename();
+
+    int pos = 0;
+    int idx = 0;
+    int psize = std::distance(path.begin(), path.end()) - 1;
+    auto dukArray = duk_push_array(ctx);
+    for (auto&& i = path.begin(); i != path.end(); i++) {
+        if (pos > 0 && psize - length <= pos && pos < psize) {
+            duk_push_string(ctx, (*i).c_str());
+            duk_put_prop_index(ctx, dukArray, idx);
+	    idx++;
+        }
+	pos++;
+    }
+    return {inputPath, length};
+}
+
 std::string ScriptTestFixture::getLastPath(duk_context* ctx)
 {
     std::string inputPath = duk_to_string(ctx, 0);
@@ -522,6 +553,13 @@ std::string ScriptTestFixture::getPlaylistType(duk_context* ctx)
     return playlistMimeType;
 }
 
+std::tuple<std::string, std::string> ScriptTestFixture::print2(duk_context* ctx)
+{
+    std::string mode = duk_to_string(ctx, 0);
+    std::string result = duk_to_string(ctx, 1);
+    return {mode, result};
+}
+
 std::string ScriptTestFixture::print(duk_context* ctx)
 {
     std::string result = duk_to_string(ctx, 0);
