@@ -3,7 +3,7 @@
 
     test_internal_m3u_playlist.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -36,39 +36,16 @@
 // Extends ScriptTestFixture to allow
 // for unique testing of the Internal URL M3U Playlist
 // processing
-class InternalUrlM3UPlaylistTest : public ScriptTestFixture {
+class InternalUrlM3UPlaylistTest : public CommonScriptTestFixture {
 public:
-    // As Duktape requires static methods, so must the mock expectations be
-    static std::unique_ptr<CommonScriptMock> commonScriptMock;
-
     InternalUrlM3UPlaylistTest()
     {
-        commonScriptMock = std::make_unique<::testing::NiceMock<CommonScriptMock>>();
         scriptName = "playlists.js";
         functionName = "importPlaylist";
         objectName = "playlist";
     }
-
-    ~InternalUrlM3UPlaylistTest() override
-    {
-        commonScriptMock.reset();
-    }
 };
 
-std::unique_ptr<CommonScriptMock> InternalUrlM3UPlaylistTest::commonScriptMock;
-
-static duk_ret_t getPlaylistType(duk_context* ctx)
-{
-    std::string playlistMimeType = ScriptTestFixture::getPlaylistType(ctx);
-    return InternalUrlM3UPlaylistTest::commonScriptMock->getPlaylistType(playlistMimeType);
-}
-
-static duk_ret_t print(duk_context* ctx)
-{
-    std::string msg = ScriptTestFixture::print(ctx);
-    return InternalUrlM3UPlaylistTest::commonScriptMock->print(msg);
-}
-
 static duk_ret_t addContainerTree(duk_context* ctx)
 {
     std::map<std::string, std::string> map {
@@ -80,18 +57,6 @@ static duk_ret_t addContainerTree(duk_context* ctx)
     return InternalUrlM3UPlaylistTest::commonScriptMock->addContainerTree(tree);
 }
 
-static duk_ret_t createContainerChain(duk_context* ctx)
-{
-    std::vector<std::string> array = ScriptTestFixture::createContainerChain(ctx);
-    return InternalUrlM3UPlaylistTest::commonScriptMock->createContainerChain(array);
-}
-
-static duk_ret_t getLastPath(duk_context* ctx)
-{
-    std::string inputPath = ScriptTestFixture::getLastPath(ctx);
-    return InternalUrlM3UPlaylistTest::commonScriptMock->getLastPath(inputPath);
-}
-
 // Proxy the Duktape script with `readln`
 // Mimics reading the playlist file line by line
 // Uses the `CommonScriptMock` to track expectations
@@ -151,10 +116,12 @@ static duk_ret_t getCdsObject(duk_context* ctx)
 // that are called from the playlists.js script
 // * These are static methods, which makes mocking difficult.
 static duk_function_list_entry js_global_functions[] = {
-    { "print", print, DUK_VARARGS },
-    { "getPlaylistType", getPlaylistType, 1 },
-    { "createContainerChain", createContainerChain, 1 },
-    { "getLastPath", getLastPath, 1 },
+    { "print", CommonScriptTestFixture::js_print, DUK_VARARGS },
+    { "print2", CommonScriptTestFixture::js_print2, DUK_VARARGS },
+    { "getPlaylistType", CommonScriptTestFixture::js_getPlaylistType, 1 },
+    { "createContainerChain", CommonScriptTestFixture::js_createContainerChain, 1 },
+    { "getLastPath", CommonScriptTestFixture::js_getLastPath, 1 },
+    { "getLastPath2", CommonScriptTestFixture::js_getLastPath2, 2 },
     { "readln", readln, 1 },
     { "addCdsObject", addCdsObject, 3 },
     { "copyObject", copyObject, 1 },
@@ -207,9 +174,10 @@ TEST_F(InternalUrlM3UPlaylistTest, AddsCdsObjectFromPlaylistWithInternalUrlPlayl
     // and will proxy through the mock objects
     // for verification.
     EXPECT_CALL(*commonScriptMock, getPlaylistType(Eq("audio/x-mpegurl"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, print(Eq("Processing playlist: /location/of/playlist.m3u"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, print2(Eq("Info"), Eq("Processing playlist: /location/of/playlist.m3u"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, print2(Eq("Debug"), Eq("Playlist 'example.mp3' Adding entry: 1 /home/gerbera/example.mp3"))).WillRepeatedly(Return(1));
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Playlists", "All Playlists", "Playlist Title"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, getLastPath(Eq("/location/of/playlist.m3u"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, getLastPath2(Eq("/location/of/playlist.m3u"), Eq(1))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Playlists", "Directories", "of", "Playlist Title"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, readln(Eq("#EXTM3U"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, readln(Eq("#EXTINF:123, Example Artist, Example Title"))).WillOnce(Return(1));
