@@ -3,7 +3,7 @@
 
     test_nfo_metafile.cc - this file is part of Gerbera.
 
-    Copyright (C) 2022-2023 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,7 +29,8 @@
 #include <pugixml.hpp>
 
 #include "cds/cds_objects.h"
-#include "upnp_common.h"
+#include "exceptions.h"
+#include "upnp/upnp_common.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 
@@ -40,33 +41,16 @@
 // Extends ScriptTestFixture to allow
 // for unique testing of the External Metafile
 // processing
-class MetafileNfoTest : public ScriptTestFixture {
+class MetafileNfoTest : public CommonScriptTestFixture {
 public:
-    // As Duktape requires static methods, so must the mock expectations be
-    static std::unique_ptr<CommonScriptMock> commonScriptMock;
-
     MetafileNfoTest()
     {
-        commonScriptMock = std::make_unique<::testing::NiceMock<CommonScriptMock>>();
         scriptName = "metadata.js";
         functionName = "importMetadata";
         objectName = "obj";
     }
-
-    ~MetafileNfoTest() override
-    {
-        commonScriptMock.reset();
-    }
 };
 
-std::unique_ptr<CommonScriptMock> MetafileNfoTest::commonScriptMock;
-
-static duk_ret_t print(duk_context* ctx)
-{
-    std::string msg = ScriptTestFixture::print(ctx);
-    return MetafileNfoTest::commonScriptMock->print(msg);
-}
-
 static pugi::xml_node nullNode;
 static pugi::xml_document xmlDoc;
 static pugi::xml_node root;
@@ -162,13 +146,13 @@ duk_ret_t updateCdsObject(duk_context* ctx)
     std::map<std::string, std::string> result;
 
     if (duk_is_undefined(ctx, -1)) {
-        std::cerr << "Could not retrieve stack " << objectName << " object" << std::endl;
+        std::cerr << "Could not retrieve stack " << objectName << " object" << '\n';
         return 0;
     }
 
     duk_get_prop_string(ctx, -1, "title");
     if (duk_is_null_or_undefined(ctx, -1) || !duk_to_string(ctx, -1)) {
-        std::cerr << "Could not retrieve stack " << objectName << ".title" << std::endl;
+        std::cerr << "Could not retrieve stack " << objectName << ".title" << '\n';
         duk_pop(ctx);
         return 0;
     }
@@ -177,7 +161,7 @@ duk_ret_t updateCdsObject(duk_context* ctx)
 
     duk_get_prop_string(ctx, -1, "upnpclass");
     if (duk_is_null_or_undefined(ctx, -1) || !duk_to_string(ctx, -1)) {
-        std::cerr << "Could not retrieve stack " << objectName << ".upnpclass" << std::endl;
+        std::cerr << "Could not retrieve stack " << objectName << ".upnpclass" << '\n';
         duk_pop(ctx);
         return 0;
     }
@@ -186,7 +170,7 @@ duk_ret_t updateCdsObject(duk_context* ctx)
 
     duk_get_prop_string(ctx, -1, "description");
     if (duk_is_null_or_undefined(ctx, -1) || !duk_to_string(ctx, -1)) {
-        std::cerr << "Could not retrieve stack " << objectName << ".description" << std::endl;
+        std::cerr << "Could not retrieve stack " << objectName << ".description" << '\n';
         duk_pop(ctx);
         return 0;
     }
@@ -216,7 +200,8 @@ duk_ret_t updateCdsObject(duk_context* ctx)
 // that are called from the playlists.js script
 // * These are static methods, which makes mocking difficult.
 static duk_function_list_entry js_global_functions[] = {
-    { "print", print, DUK_VARARGS },
+    { "print", CommonScriptTestFixture::js_print, DUK_VARARGS },
+    { "print2", CommonScriptTestFixture::js_print2, DUK_VARARGS },
     { "readXml", readXml, 1 },
     { "updateCdsObject", updateCdsObject, 1 },
     { nullptr, nullptr, 0 },
@@ -271,7 +256,7 @@ TEST_F(MetafileNfoTest, SetsPropertiesFromFile)
     };
 
     // Expecting the common script calls..and will proxy through the mock objects for verification.
-    EXPECT_CALL(*commonScriptMock, print(Eq(fmt::format("Processing metafile: {} for {} nfo", fileName, location)))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, print2(Eq("Info"), Eq(fmt::format("Processing metafile: {} for {} nfo", fileName, location)))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, readXml(Eq("<movie ></movie>"))).Times(2).WillRepeatedly(Return(1));
     EXPECT_CALL(*commonScriptMock, readXml(Eq(fmt::format("<title >{}</title>", title)))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, readXml(Eq("<originaltitle >It was MediaTomb</originaltitle>"))).WillOnce(Return(1));
