@@ -3,7 +3,7 @@
 
     test_tools.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2023 Gerbera Contributors
+    Copyright (C) 2016-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -64,7 +64,7 @@ TEST(ToolsTest, readWriteBinaryFile)
 
     std::vector<std::byte> expected(source.size());
     std::transform(source.begin(), source.end(), expected.begin(),
-        [](char c) { return std::byte(c); });
+        [](char c) { return static_cast<std::byte>(c); });
 
     EXPECT_EQ(*result, expected);
     fs::remove(testFile);
@@ -98,6 +98,22 @@ TEST(ToolsTest, renderWebUriV6)
     EXPECT_EQ(GrbNet::renderWebUri("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 7777), "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:7777");
 }
 
+TEST(ToolsTest, pathTrimTest)
+{
+    std::string test = "/regular/path";
+    EXPECT_EQ(rtrimPath(test), test);
+    std::string test2 = "/regular/path//";
+    EXPECT_EQ(rtrimPath(test2), test);
+}
+
+TEST(ToolsTest, subdirTest)
+{
+    std::string test = "/regular/path";
+    std::string test2 = "/regular/path/sub/";
+    EXPECT_EQ(isSubDir(test, test2), false);
+    EXPECT_EQ(isSubDir(test2, test), true);
+}
+
 TEST(ToolsTest, splitStringTest)
 {
     auto parts = splitString("", ',');
